@@ -22,7 +22,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: confirmation.h,v 1.12 2004/03/30 15:39:18 jcsjcs Exp $
+|  $Id: confirmation.h,v 1.14 2006/05/14 15:58:54 jcsjcs Exp $
 */
 
 #ifndef __CONFIRMATION_H__
@@ -36,7 +36,6 @@
 #include <gtk/gtk.h>
 
 typedef void (*ConfHandler)(gpointer user_data1, gpointer user_data2);
-typedef void (*ConfHandlerOpt)(gboolean state);
 
 /* states for gtkpod_confirmation options */
 typedef enum {
@@ -48,17 +47,17 @@ typedef enum {
 
 GtkResponseType gtkpod_confirmation (gint id,
 				     gboolean modal,
-				     gchar *title,
-				     gchar *label,
-				     gchar *text,
-				     gchar *option1_text,
+				     const gchar *title,
+				     const gchar *label,
+				     const gchar *text,
+				     const gchar *option1_text,
 				     CONF_STATE option1_state,
-				     ConfHandlerOpt option1_handler,
-				     gchar *option2_text,
+				     const gchar *option1_key,
+				     const gchar *option2_text,
 				     CONF_STATE option2_state,
-				     ConfHandlerOpt option2_handler,
-				     gboolean never_again,
-				     ConfHandlerOpt confirm_again_handler,
+				     const gchar *option2_key,
+				     gboolean confirm_again,
+				     const gchar *confirm_again_key,
 				     ConfHandler ok_handler,
 				     ConfHandler apply_handler,
 				     ConfHandler cancel_handler,
@@ -70,7 +69,8 @@ enum {
     CONF_ID_IPOD_DIR = 0,
     CONF_ID_GTKPOD_WARNING,
     CONF_ID_DANGLING0,
-    CONF_ID_DANGLING1
+    CONF_ID_DANGLING1,
+    CONF_ID_SYNC_SUMMARY
 } CONF_ID;
 
 void CONF_NULL_HANDLER (gpointer d1, gpointer d2);
