@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-12-11 17:06:58 jcs>
+/* Time-stamp: <2006-09-21 23:31:59 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 | 
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: details.c,v 1.8 2005/12/11 08:08:08 jcsjcs Exp $
+|  $Id: details.c,v 1.16 2006/09/21 15:03:13 jcsjcs Exp $
 */
 
 /* This file provides functions for the details window */
@@ -92,13 +92,13 @@ static void details_store_window_state (Detail *detail)
     g_return_if_fail (detail);
 
     gtk_window_get_size (GTK_WINDOW (detail->window), &defx, &defy);
-    prefs_set_int_value (DETAILS_WINDOW_DEFX, defx);
-    prefs_set_int_value (DETAILS_WINDOW_DEFY, defy);
+    prefs_set_int (DETAILS_WINDOW_DEFX, defx);
+    prefs_set_int (DETAILS_WINDOW_DEFY, defy);
 
     if ((w = gtkpod_xml_get_widget (detail->xml, "details_notebook")))
     {
 	gint page = gtk_notebook_get_current_page (GTK_NOTEBOOK (w));
-	prefs_set_int_value (DETAILS_WINDOW_NOTEBOOK_PAGE, page);
+	prefs_set_int (DETAILS_WINDOW_NOTEBOOK_PAGE, page);
     }
 }
 
@@ -130,7 +130,7 @@ static void details_text_changed (GtkWidget *widget,
     g_return_if_fail (detail->track);
     etr = detail->track->userdata;
     g_return_if_fail (etr);
-    
+
     detail->changed = TRUE;
     etr->tchanged = TRUE;
     details_update_buttons (detail);
@@ -485,12 +485,19 @@ static gboolean details_copy_artwork (Track *frtrack, Track *totrack)
 	totrack->artwork = itdb_artwork_duplicate (frtrack->artwork);
 	totrack->artwork_size = frtrack->artwork_size;
 	totrack->artwork_count = frtrack->artwork_count;
+	totrack->has_artwork = frtrack->has_artwork;
 	g_free (toetr->thumb_path_locale);
 	g_free (toetr->thumb_path_utf8);
 	toetr->thumb_path_locale = g_strdup (fretr->thumb_path_locale);
 	toetr->thumb_path_utf8 = g_strdup (fretr->thumb_path_utf8);
 	changed = TRUE;
     }
+    /* make sure artwork gets removed, even if both thumb_paths were
+       unset ("") */
+    if (!frtrack->artwork->thumbnails)
+    {
+	changed |= gp_track_remove_thumbnails (totrack);
+    }
     return changed;
 }
 
@@ -553,10 +560,24 @@ static void details_setup_widget (Detail *detail, T_item item)
     g_return_if_fail (detail);
     g_return_if_fail ((item > 0) && (item < T_ITEM_NUM));
 
-    buf = g_strdup_printf ("details_label_%d", item);
-    w = gtkpod_xml_get_widget (detail->xml, buf);
-    gtk_label_set_text (GTK_LABEL (w), gettext (get_t_string (item)));
-    g_free (buf);
+    /* Setup label */
+    switch (item)
+    {
+    case T_COMPILATION:
+    case T_CHECKED:
+    case T_REMEMBER_PLAYBACK_POSITION:
+    case T_SKIP_WHEN_SHUFFLING:
+	buf = g_strdup_printf ("details_checkbutton_%d", item);
+	w = gtkpod_xml_get_widget (detail->xml, buf);
+	gtk_button_set_label (GTK_BUTTON (w),
+			      gettext (get_t_string (item)));
+	break;
+    default:
+	buf = g_strdup_printf ("details_label_%d", item);
+	w = gtkpod_xml_get_widget (detail->xml, buf);
+	gtk_label_set_text (GTK_LABEL (w), gettext (get_t_string (item)));
+	g_free (buf);
+    }
 
     buf = NULL;
     w = NULL;
@@ -575,9 +596,12 @@ static void details_setup_widget (Detail *detail, T_item item)
     case T_PODCASTRSS:
     case T_PC_PATH:
     case T_IPOD_PATH:
+    case T_THUMB_PATH:
     case T_IPOD_ID:
     case T_SIZE:
     case T_TRACKLEN:
+    case T_STARTTIME:
+    case T_STOPTIME:
     case T_BITRATE:
     case T_SAMPLERATE:
     case T_PLAYCOUNT:
@@ -604,6 +628,8 @@ static void details_setup_widget (Detail *detail, T_item item)
     case T_COMPILATION:
     case T_TRANSFERRED:
     case T_CHECKED:
+    case T_REMEMBER_PLAYBACK_POSITION:
+    case T_SKIP_WHEN_SHUFFLING:
 	buf = g_strdup_printf ("details_checkbutton_%d", item);
 	w = gtkpod_xml_get_widget (detail->xml, buf);
 	g_signal_connect (w, "toggled",
@@ -675,9 +701,12 @@ static void details_set_item (Detail *detail, Track *track, T_item item)
     case T_PODCASTRSS:
     case T_PC_PATH:
     case T_IPOD_PATH:
+    case T_THUMB_PATH:
     case T_IPOD_ID:
     case T_SIZE:
     case T_TRACKLEN:
+    case T_STARTTIME:
+    case T_STOPTIME:
     case T_BITRATE:
     case T_SAMPLERATE:
     case T_PLAYCOUNT:
@@ -730,6 +759,28 @@ static void details_set_item (Detail *detail, Track *track, T_item item)
 					      FALSE);
 	}
 	break;
+    case T_REMEMBER_PLAYBACK_POSITION:
+	if ((w = gtkpod_xml_get_widget (detail->xml, checkbutton)))
+	{
+	    if (track)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
+					      track->remember_playback_position);
+	    else
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
+					      FALSE);
+	}
+	break;
+    case T_SKIP_WHEN_SHUFFLING:
+	if ((w = gtkpod_xml_get_widget (detail->xml, checkbutton)))
+	{
+	    if (track)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
+					      track->skip_when_shuffling);
+	    else
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
+					      FALSE);
+	}
+	break;
     case T_CHECKED:
 	if ((w = gtkpod_xml_get_widget (detail->xml, checkbutton)))
 	{
@@ -807,6 +858,8 @@ static void details_get_item (Detail *detail, T_item item,
     case T_TIME_MODIFIED:
     case T_TIME_RELEASED:
     case T_TRACKLEN:
+    case T_STARTTIME:
+    case T_STOPTIME:
     case T_SOUNDCHECK:
 	if ((w = gtkpod_xml_get_widget (detail->xml, entry)))
 	{
@@ -814,12 +867,12 @@ static void details_get_item (Detail *detail, T_item item,
 
 	    text = gtk_entry_get_text (GTK_ENTRY (w));
 
-	    /* for tracklen and soundcheck the displayed value is only
-	       a rounded figure -> unless 'assumechanged' is set,
-	       compare the string to the original one before assuming
-	       a change took place */
+	    /* for soundcheck the displayed value is only a rounded
+	       figure -> unless 'assumechanged' is set, compare the
+	       string to the original one before assuming a change
+	       took place */
 	    if (!assumechanged &&
-		((item == T_TRACKLEN) || (item == T_SOUNDCHECK)))
+		(item == T_SOUNDCHECK))
 	    {
 		gchar *buf;
 		track->itdb = detail->itdb;
@@ -832,7 +885,7 @@ static void details_get_item (Detail *detail, T_item item,
 	    }
 	    else
 	    {
-	    changed = track_set_text (track, text, item);
+		changed = track_set_text (track, text, item);
 	    }
 	    /* redisplay some items to be on the safe side */
 	    switch (item)
@@ -840,6 +893,8 @@ static void details_get_item (Detail *detail, T_item item,
 	    case T_TRACK_NR:
 	    case T_CD_NR:
 	    case T_TRACKLEN:
+	    case T_STARTTIME:
+	    case T_STOPTIME:
 	    case T_TIME_ADDED:
 	    case T_TIME_PLAYED:
 	    case T_TIME_MODIFIED:
@@ -881,6 +936,32 @@ static void details_get_item (Detail *detail, T_item item,
 	    }
 	}
 	break;
+    case T_REMEMBER_PLAYBACK_POSITION:
+	if ((w = gtkpod_xml_get_widget (detail->xml, checkbutton)))
+	{
+	    gboolean state;
+	    state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w));
+
+	    if (track->remember_playback_position != state)
+	    {
+		track->remember_playback_position = state;
+		changed = TRUE;
+	    }
+	}
+	break;
+    case T_SKIP_WHEN_SHUFFLING:
+	if ((w = gtkpod_xml_get_widget (detail->xml, checkbutton)))
+	{
+	    gboolean state;
+	    state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w));
+
+	    if (track->skip_when_shuffling != state)
+	    {
+		track->skip_when_shuffling = state;
+		changed = TRUE;
+	    }
+	}
+	break;
     case T_CHECKED:
 	if ((w = gtkpod_xml_get_widget (detail->xml, checkbutton)))
 	{
@@ -899,8 +980,10 @@ static void details_get_item (Detail *detail, T_item item,
     case T_PC_PATH:
     case T_IPOD_PATH:
     case T_IPOD_ID:
+    case T_THUMB_PATH:
 	/* These are read-only only */
 	break;
+	break;
     case T_ALL:
     case T_ITEM_NUM:
 	/* cannot happen because of assertion above */
@@ -1057,7 +1140,6 @@ static void details_update_thumbnail (Detail *detail)
 	/* Get large cover */
 	thumb = itdb_artwork_get_thumb_by_type (detail->track->artwork,
 						ITDB_THUMB_COVER_LARGE);
-
 	if (thumb)
 	{
 	    GdkPixbuf *pixbuf;
@@ -1069,6 +1151,7 @@ static void details_update_thumbnail (Detail *detail)
 		gdk_pixbuf_unref (pixbuf);
 	    }
 	}
+	details_set_item (detail,  detail->track, T_THUMB_PATH);
     }
 
     if (gtk_image_get_storage_type (img) == GTK_IMAGE_EMPTY)
