@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-12-14 23:13:26 jcs>
+/* Time-stamp: <2006-06-25 00:22:56 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,24 +24,27 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: display.c,v 1.130 2005/12/14 14:58:55 jcsjcs Exp $
+|  $Id: display.c,v 1.149 2006/06/24 15:39:22 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
-#include <string.h>
-#include <stdlib.h>
+#include "details.h"
 #include "display_private.h"
 #include "info.h"
-#include "details.h"
-#include "prefs.h"
-#include "prefs_window.h"
+#include "ipod_init.h"
 #include "misc.h"
 #include "misc_track.h"
+#include "prefs.h"
+#include "prefs_window.h"
+#include "repository.h"
+#include "syncdir.h"
 #include "tools.h"
-#include "podcast.h"
+#include <stdlib.h>
+#include <string.h>
+
 
 GtkWidget *gtkpod_window = NULL;
 
@@ -58,7 +61,8 @@ void display_create (void)
     g_return_if_fail (gtkpod_window);
 
     /* x,y-size */
-    prefs_get_size_gtkpod (&defx, &defy);
+    defx = prefs_get_int("size_gtkpod.x");
+		defy = prefs_get_int("size_gtkpod.y");
     gtk_window_set_default_size (GTK_WINDOW (gtkpod_window), defx, defy);
 /* we need to use the following line if the main window is already
    displayed */
@@ -88,8 +92,6 @@ void display_create (void)
 
     /* activate the delete menus correctly */
     display_adjust_menus ();
-    /* activate/deactive the menu item 'check iPod' */
-    display_set_check_ipod_menu ();
     /* activate status bars */
     gtkpod_statusbar_init ();
     gtkpod_tracks_statusbar_init ();
@@ -98,11 +100,7 @@ void display_create (void)
     /* CAREFUL: must be done after calling ..._space_statusbar_init() */
     display_set_info_window_menu ();
     /* check if info window should be opened */
-    if (prefs_get_info_window ())  info_open_window ();
-
-    /* if we need to fetch podcasts, then do so */
-/* FIXME: PODCASTS: remove Podcast menu */
-/*     if (prefs_get_pc_auto_fetch ())  podcast_fetch(); */
+    if (prefs_get_int("info_window"))  info_open_window ();
 }
 
 /* redisplay the entire display (playlists, sort tabs, track view) and
@@ -148,50 +146,24 @@ void display_cleanup (void)
 }
 
 
-/**
- * disable_import_buttons
- * Upon successfull itunes db importing we want to disable the import
- * buttons.  This retrieves the import buttons from the main gtkpod widget
- * and disables them from taking input.
- */
-void
-display_disable_gtkpod_import_buttons(void)
-{
-    GtkWidget *w = NULL;
-
-    g_return_if_fail (gtkpod_window);
-
-    if((w = gtkpod_xml_get_widget (main_window_xml, "import_button")))
-    {
-	gtk_widget_set_sensitive(w, FALSE);
-	/* in case this widget has been blocked, we need to tell
-	   update the desired state upon release */
-	update_blocked_widget (w, FALSE);
-    }
-
-    if((w = gtkpod_xml_get_widget (main_window_xml, "import_itunes_mi")))
-    {
-	gtk_widget_set_sensitive(w, FALSE);
-	/* in case this widget has been blocked, we need to tell
-	   update the desired state upon release */
-	update_blocked_widget (w, FALSE);
-    }
-}
-
-
 /* make sure only suitable delete menu items are available */
 void display_adjust_menus (void)
 {
-    GtkWidget *delete, *edit;
+    GtkWidget *delete;
+    GtkWidget *edit1, *edit2, *edit3, *edit4, *edit5;
     GtkWidget *dtfpl, *dtfip, *dtfdb, *dtfhd;
     GtkWidget *defpl, *defip, *defdb, *defhd;
     GtkWidget *dpl, *dpltfip, *dpltfdb, *dpltfhd;
     GtkWidget *dsep1, *dsep2;
-    GtkWidget *espl, *w;
+    GtkWidget *espl;
     Playlist *pl;
 
     delete = gtkpod_xml_get_widget (main_window_xml, "delete_menu");
-    edit = gtkpod_xml_get_widget (main_window_xml, "edit_menu");
+    edit1 = gtkpod_xml_get_widget (main_window_xml, "edit_details_menu");
+    edit2 = gtkpod_xml_get_widget (main_window_xml, "delete_menu");
+    edit3 = gtkpod_xml_get_widget (main_window_xml, "create_playlists_menu");
+    edit4 = gtkpod_xml_get_widget (main_window_xml, "randomize_current_playlist_menu");
+    edit5 = gtkpod_xml_get_widget (main_window_xml, "save_track_order_menu");
     dtfpl = gtkpod_xml_get_widget (main_window_xml,
 				   "delete_selected_tracks_from_playlist");
     dtfip = gtkpod_xml_get_widget (main_window_xml,
@@ -221,18 +193,16 @@ void display_adjust_menus (void)
     espl = gtkpod_xml_get_widget (main_window_xml,
 				  "edit_smart_playlist");
 
-    /* FIXME: PODCASTS: remove Podcast menu */
-    w = gtkpod_xml_get_widget (main_window_xml, "update_podcasts");
-    gtk_widget_hide (w);
-    w = gtkpod_xml_get_widget (main_window_xml, "update_podcasts_sep");
-    gtk_widget_hide (w);
-
     pl = pm_get_selected_playlist ();
 
     if (pl == NULL)
     {
 	gtk_widget_set_sensitive (delete, FALSE);
-	gtk_widget_set_sensitive (edit, FALSE);
+	gtk_widget_set_sensitive (edit1, FALSE);
+	gtk_widget_set_sensitive (edit2, FALSE);
+	gtk_widget_set_sensitive (edit3, FALSE);
+	gtk_widget_set_sensitive (edit4, FALSE);
+	gtk_widget_set_sensitive (edit5, FALSE);
     }
     else
     {
@@ -240,7 +210,11 @@ void display_adjust_menus (void)
 	g_return_if_fail (itdb);
 
 	gtk_widget_set_sensitive (delete, TRUE);
-	gtk_widget_set_sensitive (edit, TRUE);
+	gtk_widget_set_sensitive (edit1, TRUE);
+	gtk_widget_set_sensitive (edit2, TRUE);
+	gtk_widget_set_sensitive (edit3, TRUE);
+	gtk_widget_set_sensitive (edit4, TRUE);
+	gtk_widget_set_sensitive (edit5, TRUE);
 
 	gtk_widget_hide (dtfpl);
 	gtk_widget_hide (dtfip);
@@ -370,9 +344,9 @@ void display_show_hide_toolbar (void)
     GtkWidget *tb = gtkpod_xml_get_widget (main_window_xml, "toolbar");
     GtkWidget *mi = gtkpod_xml_get_widget (main_window_xml, "toolbar_menu");
 
-    if (prefs_get_display_toolbar ())
+    if (prefs_get_int("display_toolbar"))
     {
-	gtk_toolbar_set_style (GTK_TOOLBAR (tb), prefs_get_toolbar_style ());
+	gtk_toolbar_set_style (GTK_TOOLBAR (tb), prefs_get_int("toolbar_style"));
 	gtk_widget_show (tb);
 	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (mi), TRUE);
     }
@@ -389,7 +363,7 @@ void display_set_info_window_menu (void)
 {
     GtkWidget *mi = gtkpod_xml_get_widget (main_window_xml, "info_window_menu");
 
-    if (prefs_get_info_window ())
+    if (prefs_get_int("info_window"))
     {
 	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (mi), TRUE);
     }
@@ -399,17 +373,6 @@ void display_set_info_window_menu (void)
     }
 }
 
-/** Takes care about 'Check IPOD Files' Menu Item and makes it unavailable
- * if gtkpod is offline.
- */
-void display_set_check_ipod_menu (void)
-{
-    GtkWidget *w = NULL;
-
-    if((w = gtkpod_xml_get_widget (main_window_xml, "check_ipod_files_mi")))
-        gtk_widget_set_sensitive(w, !prefs_get_offline());
-}
-
 /* make the tooltips visible or hide it depending on the value set in
  * the prefs (tooltips_main) */
 void display_show_hide_tooltips (void)
@@ -436,7 +399,7 @@ void display_show_hide_tooltips (void)
     g_return_if_fail (mt);
 
 
-    if (prefs_get_display_tooltips_main ())
+    if (prefs_get_int("display_tooltips_main"))
     {
 	gtk_tooltips_enable (mt);
 	gtk_check_menu_item_set_active (mi, TRUE);
@@ -469,8 +432,9 @@ void display_update_default_sizes (void)
     /* x,y size of main window */
     if (gtkpod_window)
     {
-	gtk_window_get_size (GTK_WINDOW (gtkpod_window), &x, &y);
-	prefs_set_size_gtkpod (x, y);
+			gtk_window_get_size (GTK_WINDOW (gtkpod_window), &x, &y);
+			prefs_set_int("size_gtkpod.x", x);
+			prefs_set_int("size_gtkpod.y", y);
     }
     tm_update_default_sizes ();
     st_update_default_sizes ();
@@ -505,9 +469,9 @@ GList *display_get_selection (guint32 inst)
 	if (pl)  return g_list_copy (pl->members);
 	else     return NULL;
     }
-    if ((inst >= 0) && (inst < prefs_get_sort_tab_num ()))
+    if ((inst >= 0) && (inst < prefs_get_int("sort_tab_num")))
 	return g_list_copy (st_get_selected_members (inst));
-    if (inst >= prefs_get_sort_tab_num ())
+    if (inst >= prefs_get_int("sort_tab_num"))
 	return tm_get_selected_tracks ();
     return NULL;
 }
@@ -905,26 +869,19 @@ void display_install_autoscroll_row_timeout (GtkWidget *widget)
 */
 
 
-/* FIXME: PODCASTS: remove Podcast menu */
-void on_update_podcasts_activate (GtkButton       *button,
-				  gpointer         user_data)
-{
-#if 0
-    podcast_fetch ();
-#endif
-}
-
 void on_edit_details_selected_playlist (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
     Playlist *pl = pm_get_selected_playlist ();
 
-    if (!pl)
+    if (pl)
     {
-	gtkpod_statusbar_message (_("No playlist selected"));
-	return;
+	details_edit (pl->members);
+    }
+    else
+    {
+	message_sb_no_playlist_selected ();
     }
-    details_edit (pl->members);
 }
 
 void on_edit_details_selected_tab_entry (GtkMenuItem     *menuitem,
@@ -957,7 +914,7 @@ void on_edit_details_selected_tracks (GtkMenuItem     *menuitem,
     }
     else
     {
-	gtkpod_statusbar_message (_("No tracks selected"));
+	message_sb_no_tracks_selected ();
     }
 }
 
@@ -967,7 +924,16 @@ void
 on_new_playlist_button                 (GtkButton       *button,
 					gpointer         user_data)
 {
-  add_new_pl_or_spl_user_name (gp_get_active_itdb (), NULL, -1);
+    iTunesDB *itdb = gp_get_selected_itdb();
+
+    if (itdb)
+    {
+	add_new_pl_or_spl_user_name (itdb, NULL, -1);
+    }
+    else
+    {
+	message_sb_no_itdb_selected ();
+    }
 }
 
 
@@ -976,53 +942,88 @@ void
 on_new_playlist1_activate              (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
-  add_new_pl_or_spl_user_name (gp_get_active_itdb(), NULL, -1);
+    iTunesDB *itdb = gp_get_selected_itdb();
+
+    if (itdb)
+    {
+	add_new_pl_or_spl_user_name (itdb, NULL, -1);
+    }
+    else
+    {
+	message_sb_no_itdb_selected ();
+    }
 }
 
 void
-on_edit_preferences1_activate          (GtkMenuItem     *menuitem,
+on_edit_preferences_activate          (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
     if(!widgets_blocked)  prefs_window_create (-1);
 }
 
-
 void
-on_offline1_activate                   (GtkMenuItem     *menuitem,
-					gpointer         user_data)
+on_edit_repository_options_activate          (GtkMenuItem     *menuitem,
+					      gpointer         user_data)
 {
-  prefs_set_offline (
-     gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (menuitem)));
-  display_set_check_ipod_menu ();
+    Playlist *pl = pm_get_selected_playlist ();
+
+    if (pl)
+    {
+	repository_edit (pl->itdb, pl);
+    }
+    else
+    {
+	repository_edit (NULL, NULL);
+    }
 }
 
 void
-on_import_itunes_mi_activate           (GtkMenuItem     *menuitem,
-					gpointer         user_data)
+on_load_ipods_mi           (GtkMenuItem     *menuitem,
+			    gpointer         user_data)
 {
-  gp_merge_ipod_itdbs ();
+    gp_load_ipods ();
 }
 
 
 void
-on_import_button_clicked               (GtkButton       *button,
-					gpointer         user_data)
+on_load_ipods_clicked               (GtkButton       *button,
+				     gpointer         user_data)
 {
-  gp_merge_ipod_itdbs ();
+    gp_load_ipods ();
 }
 
 
+void
+on_save_changes_mi           (GtkMenuItem     *menuitem,
+			      gpointer         user_data)
+{
+    handle_export ();
+}
+
+
+
+void
+on_save_changes_clicked               (GtkButton       *button,
+				     gpointer         user_data)
+{
+    handle_export ();
+}
+
+
+
 void on_edit_smart_playlist (GtkMenuItem *mi,
 			     gpointer data)
 {
     Playlist *pl = pm_get_selected_playlist ();
 
-    if (!pl)
+    if (pl)
     {
-	gtkpod_statusbar_message (_("No playlist selected"));
-	return;
+	spl_edit (pl);
+    }
+    else
+    {
+	message_sb_no_playlist_selected ();
     }
-    spl_edit (pl);
 }
 
 
@@ -1037,7 +1038,7 @@ static void delete_selected_tracks (DeleteAction deleteaction)
     }
     else
     {
-	gtkpod_statusbar_message (_("No tracks selected"));
+	message_sb_no_tracks_selected ();
     }
 }    
 
@@ -1053,10 +1054,8 @@ static void delete_selected_entry (DeleteAction deleteaction,
     entry = st_get_selected_entry (inst);
     if (!entry)
     {
-	gchar *str = g_strdup_printf(_("No entry selected in Sort Tab %d"),
-				     inst+1);
-	gtkpod_statusbar_message (str);
-	g_free (str);
+	gtkpod_statusbar_message (_("No entry selected in Sort Tab %d"),
+				  inst+1);
 	return;
     }
     delete_entry_head (inst, deleteaction);
@@ -1066,12 +1065,14 @@ static void delete_selected_playlist (DeleteAction deleteaction)
 {
     Playlist *pl = pm_get_selected_playlist ();
 
-    if (!pl)
+    if (pl)
     {
-	gtkpod_statusbar_message (_("No playlist selected"));
-	return;
+	delete_playlist_head (deleteaction);
+    }
+    else
+    {
+	message_sb_no_playlist_selected ();
     }
-    delete_playlist_head (deleteaction);
 }
 
 void on_delete_selected_tracks_from_database (GtkMenuItem *mi,
@@ -1168,17 +1169,79 @@ void
 on_ipod_directories_menu               (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
-    iTunesDB *itdb = gp_get_active_itdb ();
-    if (!itdb)
+    iTunesDB *itdb = gp_get_ipod_itdb ();
+    if (itdb)
     {
-	gtkpod_statusbar_message (_("Currently no iPod database selected"));
+	ExtraiTunesDBData *eitdb = itdb->userdata;
+
+	g_return_if_fail (eitdb);
+
+
+	if (!eitdb->itdb_imported)
+	{
+	    gchar *mountpoint = get_itdb_prefs_string (itdb, KEY_MOUNTPOINT);
+	    gchar *str = g_strdup_printf (_("iPod at '%s' is not loaded.\nPlease load it first."), mountpoint);
+	    GtkWidget *dialog = gtk_message_dialog_new (
+		GTK_WINDOW (gtkpod_window),
+		GTK_DIALOG_DESTROY_WITH_PARENT,
+		GTK_MESSAGE_WARNING,
+		GTK_BUTTONS_OK,
+		str);
+	    gtk_dialog_run (GTK_DIALOG (dialog));
+	    gtk_widget_destroy (dialog);
+	    g_free (str);
+	    g_free (mountpoint);
+	}
+	else
+	{
+	     gp_ipod_init (itdb);
+	}
     }
     else
     {
-	ipod_directories_head (itdb->mountpoint);
+	message_sb_no_ipod_itdb_selected ();
     }
 }
 
+void
+on_check_ipod_files_activate           (GtkMenuItem     *menuitem,
+					gpointer         user_data)
+{
+    iTunesDB *itdb = gp_get_ipod_itdb ();
+    if (itdb)
+    {
+	ExtraiTunesDBData *eitdb = itdb->userdata;
+
+	g_return_if_fail (eitdb);
+
+
+	if (!eitdb->itdb_imported)
+	{
+	    gchar *mountpoint = get_itdb_prefs_string (itdb, KEY_MOUNTPOINT);
+	    gchar *str = g_strdup_printf (_("iPod at '%s' is not loaded.\nPlease load it first."), mountpoint);
+	    GtkWidget *dialog = gtk_message_dialog_new (
+		GTK_WINDOW (gtkpod_window),
+		GTK_DIALOG_DESTROY_WITH_PARENT,
+		GTK_MESSAGE_WARNING,
+		GTK_BUTTONS_OK,
+		str);
+	    gtk_dialog_run (GTK_DIALOG (dialog));
+	    gtk_widget_destroy (dialog);
+	    g_free (str);
+	    g_free (mountpoint);
+	}
+	else
+	{
+	    check_db (itdb);
+	}
+    }
+    else
+    {
+	message_sb_no_ipod_itdb_selected ();
+    }
+}
+
+
 void
 on_stop_button_clicked                 (GtkButton       *button,
 					gpointer         user_data)
@@ -1236,34 +1299,28 @@ void
 on_sync_playlist_activate (GtkMenuItem     *menuitem,
 			     gpointer         user_data)
 {
-    gp_do_selected_playlist (sync_tracks);
+    Playlist *pl;
+
+    pl = pm_get_selected_playlist();
+    if (pl)
+    {
+	sync_playlist (pl, NULL,
+		       KEY_SYNC_CONFIRM_DIRS, 0,
+		       KEY_SYNC_DELETE_TRACKS, 0,
+		       KEY_SYNC_CONFIRM_DELETE, 0,
+		       KEY_SYNC_SHOW_SUMMARY, 0);
+    }
+    else
+    {
+	message_sb_no_playlist_selected ();
+    }
 }
 
-/* sync tracks in tab entry */
-void
-on_sync_tab_entry_activate        (GtkMenuItem     *menuitem,
-				     gpointer         user_data)
-{
-    gint inst = get_sort_tab_number (
-	_("Sync dirs of selected entry in which sort tab?"));
-
-    if (inst != -1) gp_do_selected_entry (sync_tracks, inst);
-}
-
-void
-on_sync_tracks_activate            (GtkMenuItem     *menuitem,
-				     gpointer         user_data)
-{
-    gp_do_selected_tracks (sync_tracks);
-}
-
-
 void
 on_save_track_order1_activate           (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
     tm_rows_reordered ();
-    pm_rows_reordered ();
 }
 
 
@@ -1271,8 +1328,9 @@ void
 on_toolbar_menu_activate               (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
-    prefs_set_display_toolbar (
+    prefs_set_int("display_toolbar",
 	gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (menuitem)));
+    display_show_hide_toolbar();
 }
 
 
@@ -1280,7 +1338,8 @@ void
 on_more_sort_tabs_activate             (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
-    prefs_set_sort_tab_num (prefs_get_sort_tab_num()+1, TRUE);
+    prefs_set_int("sort_tab_num", prefs_get_int("sort_tab_num")+1);
+    st_show_visible();
 }
 
 
@@ -1288,7 +1347,8 @@ void
 on_less_sort_tabs_activate             (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
-    prefs_set_sort_tab_num (prefs_get_sort_tab_num()-1, TRUE);
+    prefs_set_int("sort_tab_num", prefs_get_int("sort_tab_num")-1);
+    st_show_visible();
 }
 
 void
@@ -1299,10 +1359,12 @@ on_export_playlist_activate  (GtkMenuItem     *menuitem,
 
     if (!pl)
     {
-	gtkpod_statusbar_message (_("No playlist selected"));
-	return;
+	export_files_init (pl->members, NULL, FALSE, NULL);
+    }
+    else
+    {
+	message_sb_no_playlist_selected ();
     }
-    export_files_init (pl->members, NULL, FALSE, NULL);
 }
 
 
@@ -1319,10 +1381,8 @@ on_export_tab_entry_activate (GtkMenuItem     *menuitem,
     entry = st_get_selected_entry (inst);
     if (!entry)
     {
-	gchar *str = g_strdup_printf(_("No entry selected in Sort Tab %d"),
-				     inst+1);
-	gtkpod_statusbar_message (str);
-	g_free (str);
+	gtkpod_statusbar_message (_("No entry selected in Sort Tab %d"),
+				  inst+1);
 	return;
     }
     export_files_init (entry->members, NULL, FALSE, NULL);
@@ -1342,7 +1402,7 @@ on_export_tracks_activate     (GtkMenuItem     *menuitem,
     }
     else
     {
-	gtkpod_statusbar_message (_("No tracks selected"));
+	message_sb_no_tracks_selected ();
     }
 }
 
@@ -1353,12 +1413,14 @@ on_playlist_file_playlist_activate     (GtkMenuItem     *menuitem,
 {
     Playlist *pl = pm_get_selected_playlist ();
 
-    if (!pl)
+    if (pl)
     {
-	gtkpod_statusbar_message (_("No playlist selected"));
-	return;
+	export_playlist_file_init (pl->members);
+    }
+    else
+    {
+	message_sb_no_playlist_selected ();
     }
-    export_playlist_file_init (pl->members);
 }
 
 
@@ -1375,10 +1437,8 @@ on_playlist_file_tab_entry_activate    (GtkMenuItem     *menuitem,
     entry = st_get_selected_entry (inst);
     if (!entry)
     {
-	gchar *str = g_strdup_printf(_("No entry selected in Sort Tab %d"),
-				     inst+1);
-	gtkpod_statusbar_message (str);
-	g_free (str);
+	gtkpod_statusbar_message (_("No entry selected in Sort Tab %d"),
+				  inst+1);
 	return;
     }
     export_playlist_file_init (entry->members);
@@ -1398,7 +1458,7 @@ on_playlist_file_tracks_activate       (GtkMenuItem     *menuitem,
     }
     else
     {
-	gtkpod_statusbar_message (_("No tracks selected"));
+	message_sb_no_tracks_selected ();
     }
 }
 
@@ -1408,9 +1468,13 @@ on_play_playlist_activate              (GtkMenuItem     *menuitem,
 {
     Playlist *pl = pm_get_selected_playlist ();
     if (pl)
+    {
 	tools_play_tracks (pl->members);
+    }
     else
-	gtkpod_statusbar_message (_("No playlist selected"));
+    {
+	message_sb_no_playlist_selected ();
+    }
 }
 
 
@@ -1427,10 +1491,8 @@ on_play_tab_entry_activate             (GtkMenuItem     *menuitem,
     entry = st_get_selected_entry (inst);
     if (!entry)
     {
-	gchar *str = g_strdup_printf(_("No entry selected in Sort Tab %d"),
-				     inst+1);
-	gtkpod_statusbar_message (str);
-	g_free (str);
+	gtkpod_statusbar_message (_("No entry selected in Sort Tab %d"),
+				  inst+1);
 	return;
     }
     tools_play_tracks (entry->members);
@@ -1449,7 +1511,9 @@ on_play_tracks_activate                 (GtkMenuItem     *menuitem,
 	tracks = NULL;
     }
     else
-	gtkpod_statusbar_message (_("No tracks selected"));
+    {
+	message_sb_no_tracks_selected ();
+    }
 }
 
 
@@ -1459,9 +1523,13 @@ on_enqueue_playlist_activate           (GtkMenuItem     *menuitem,
 {
     Playlist *pl = pm_get_selected_playlist ();
     if (pl)
+    {
 	tools_enqueue_tracks (pl->members);
+    }
     else
-	gtkpod_statusbar_message (_("No playlist selected"));
+    {
+	message_sb_no_playlist_selected ();
+    }
 }
 
 
@@ -1478,10 +1546,8 @@ on_enqueue_tab_entry_activate          (GtkMenuItem     *menuitem,
     entry = st_get_selected_entry (inst);
     if (!entry)
     {
-	gchar *str = g_strdup_printf(_("No entry selected in Sort Tab %d"),
-				     inst+1);
-	gtkpod_statusbar_message (str);
-	g_free (str);
+	gtkpod_statusbar_message (_("No entry selected in Sort Tab %d"),
+				  inst+1);
 	return;
     }
     tools_enqueue_tracks (entry->members);
@@ -1500,7 +1566,9 @@ on_enqueue_tracks_activate              (GtkMenuItem     *menuitem,
 	tracks = NULL;
     }
     else
-	gtkpod_statusbar_message (_("No tracks selected"));
+    {
+	message_sb_no_tracks_selected ();
+    }    
 }
 
 
@@ -1515,8 +1583,10 @@ void
 on_tooltips_menu_activate              (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
-    prefs_set_display_tooltips_main (
+    prefs_set_int("display_tooltips_main",
 	gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (menuitem)));
+    display_show_hide_tooltips();
+    
 }
 
 void
@@ -1537,28 +1607,64 @@ void
 on_pl_for_each_artist_activate         (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
-    generate_category_playlists (gp_get_active_itdb(), T_ARTIST);
+    iTunesDB *itdb = gp_get_selected_itdb();
+
+    if (itdb)
+    {
+	generate_category_playlists (itdb, T_ARTIST);
+    }
+    else
+    {
+	message_sb_no_itdb_selected ();
+    }
 }
 
 void
 on_pl_for_each_album_activate         (GtkMenuItem     *menuitem,
 				       gpointer         user_data)
 {
-    generate_category_playlists (gp_get_active_itdb(), T_ALBUM);
+    iTunesDB *itdb = gp_get_selected_itdb();
+
+    if (itdb)
+    {
+	generate_category_playlists (itdb, T_ALBUM);
+    }
+    else
+    {
+	message_sb_no_itdb_selected ();
+    }
 }
 
 void
 on_pl_for_each_genre_activate         (GtkMenuItem     *menuitem,
 				       gpointer         user_data)
 {
-    generate_category_playlists (gp_get_active_itdb(), T_GENRE);
+    iTunesDB *itdb = gp_get_selected_itdb();
+
+    if (itdb)
+    {
+	generate_category_playlists (itdb, T_GENRE);
+    }
+    else
+    {
+	message_sb_no_itdb_selected ();
+    }
 }
 
 void
 on_pl_for_each_composer_activate         (GtkMenuItem     *menuitem,
 					  gpointer         user_data)
 {
-    generate_category_playlists (gp_get_active_itdb(), T_COMPOSER);
+    iTunesDB *itdb = gp_get_selected_itdb();
+
+    if (itdb)
+    {
+	generate_category_playlists (itdb, T_COMPOSER);
+    }
+    else
+    {
+	message_sb_no_itdb_selected ();
+    }
 }
 
 
@@ -1566,7 +1672,16 @@ void
 on_pl_for_each_year_activate           (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-    generate_category_playlists (gp_get_active_itdb(), T_YEAR);
+    iTunesDB *itdb = gp_get_selected_itdb();
+
+    if (itdb)
+    {
+	generate_category_playlists (itdb, T_YEAR);
+    }
+    else
+    {
+	message_sb_no_itdb_selected ();
+    }
 }
 
 
@@ -1574,7 +1689,16 @@ void
 on_most_listened_tracks1_activate       (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
-    most_listened_pl(gp_get_active_itdb());
+    iTunesDB *itdb = gp_get_selected_itdb();
+
+    if (itdb)
+    {
+	most_listened_pl (itdb);
+    }
+    else
+    {
+	message_sb_no_itdb_selected ();
+    }
 }
 
 
@@ -1583,7 +1707,16 @@ on_all_tracks_never_listened_to1_activate
 					(GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
-    never_listened_pl(gp_get_active_itdb());
+    iTunesDB *itdb = gp_get_selected_itdb();
+
+    if (itdb)
+    {
+	never_listened_pl (itdb);
+    }
+    else
+    {
+	message_sb_no_itdb_selected ();
+    }
 }
 
 void
@@ -1591,7 +1724,16 @@ on_most_rated_tracks_playlist_s1_activate
 					(GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
-    most_rated_pl(gp_get_active_itdb());
+    iTunesDB *itdb = gp_get_selected_itdb();
+
+    if (itdb)
+    {
+	most_rated_pl (itdb);
+    }
+    else
+    {
+	message_sb_no_itdb_selected ();
+    }
 }
 
 
@@ -1599,14 +1741,32 @@ void
 on_most_recent_played_tracks_activate   (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
-    last_listened_pl(gp_get_active_itdb());
+    iTunesDB *itdb = gp_get_selected_itdb();
+
+    if (itdb)
+    {
+	last_listened_pl (itdb);
+    }
+    else
+    {
+	message_sb_no_itdb_selected ();
+    }
 }
 
 void
 on_played_since_last_time1_activate    (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
-    since_last_pl(gp_get_active_itdb());
+    iTunesDB *itdb = gp_get_selected_itdb();
+
+    if (itdb)
+    {
+	since_last_pl (itdb);
+    }
+    else
+    {
+	message_sb_no_itdb_selected ();
+    }
 }
 
 void
@@ -1625,7 +1785,7 @@ on_normalize_selected_playlist_activate
     if (pl)
 	nm_tracks_list (pl->members);
     else
-	gtkpod_statusbar_message (_("No playlist selected"));
+	message_sb_no_playlist_selected ();
 }
 
 
@@ -1647,10 +1807,8 @@ on_normalize_selected_tab_entry_activate
     }
     else
     {
-	gchar *str = g_strdup_printf(_("No entry selected in Sort Tab %d"),
-				     inst+1);
-	gtkpod_statusbar_message (str);
-	g_free (str);
+	gtkpod_statusbar_message (_("No entry selected in Sort Tab %d"),
+				  inst+1);
 	return;
     }
 }
@@ -1680,12 +1838,18 @@ void
 on_normalize_all_tracks                (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
-    iTunesDB *itdb = gp_get_active_itdb ();
-    Playlist *mpl;
-    g_return_if_fail (itdb);
-    mpl = itdb_playlist_mpl (itdb);
-    g_return_if_fail (mpl);
-    nm_tracks_list (mpl->members);
+    iTunesDB *itdb = gp_get_selected_itdb ();
+
+    if (itdb)
+    {
+	Playlist *mpl = itdb_playlist_mpl (itdb);
+	g_return_if_fail (mpl);
+	nm_tracks_list (mpl->members);
+    }
+    else
+    {
+	message_sb_no_itdb_selected ();
+    }
 }
 
 
@@ -1693,7 +1857,16 @@ void
 on_normalize_newly_added_tracks        (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
-    nm_new_tracks (gp_get_active_itdb ());
+    iTunesDB *itdb = gp_get_selected_itdb();
+
+    if (itdb)
+    {
+	nm_new_tracks (itdb);
+    }
+    else
+    {
+	message_sb_no_itdb_selected ();
+    }
 }
 
 
@@ -1706,19 +1879,24 @@ on_info_window1_activate               (GtkMenuItem     *menuitem,
     else info_close_window ();
 }
 
-void
-on_check_ipod_files_activate           (GtkMenuItem     *menuitem,
-					gpointer         user_data)
-{
-    check_db (gp_get_ipod_itdb());
-}
-
-
 void
 on_sync_all_activate                   (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	tools_sync_all ();
+    struct itdbs_head *itdbs_head = gp_get_itdbs_head (gtkpod_window);
+    GList *gl;
+
+    g_return_if_fail (itdbs_head);
+
+    for (gl=itdbs_head->itdbs; gl; gl=gl->next)
+    {
+	iTunesDB *itdb = gl->data;
+	g_return_if_fail (gl);
+	if (itdb->usertype & GP_ITDB_TYPE_IPOD)
+	{
+	    tools_sync_all (itdb);
+	}
+    }
 }
 
 
@@ -1726,7 +1904,15 @@ void
 on_sync_calendar_activate              (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
-    tools_sync_calendar ();
+    iTunesDB *itdb = gp_get_ipod_itdb ();
+    if (itdb)
+    {
+	tools_sync_calendar (itdb);
+    }
+    else
+    {
+	message_sb_no_ipod_itdb_selected ();
+    }
 }
 
 
@@ -1734,7 +1920,15 @@ void
 on_sync_contacts_activate              (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
-    tools_sync_contacts ();
+    iTunesDB *itdb = gp_get_ipod_itdb ();
+    if (itdb)
+    {
+	tools_sync_contacts (itdb);
+    }
+    else
+    {
+	message_sb_no_ipod_itdb_selected ();
+    }
 }
 
 
@@ -1742,8 +1936,15 @@ void
 on_sync_notes_activate                 (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	tools_sync_notes ();
-
+    iTunesDB *itdb = gp_get_ipod_itdb ();
+    if (itdb)
+    {
+	tools_sync_notes (itdb);
+    }
+    else
+    {
+	message_sb_no_ipod_itdb_selected ();
+    }
 }
 
 void
@@ -1751,7 +1952,16 @@ on_all_tracks_not_listed_in_any_playlist1_activate
                                         (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-    generate_not_listed_playlist (gp_get_active_itdb ());
+    iTunesDB *itdb = gp_get_selected_itdb();
+
+    if (itdb)
+    {
+	generate_not_listed_playlist (itdb);
+    }
+    else
+    {
+	message_sb_no_itdb_selected ();
+    }
 }
 
 
@@ -1759,7 +1969,16 @@ void
 on_random_playlist_activate            (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-    generate_random_playlist(gp_get_active_itdb ());
+    iTunesDB *itdb = gp_get_selected_itdb();
+
+    if (itdb)
+    {
+	generate_random_playlist(itdb);
+    }
+    else
+    {
+	message_sb_no_itdb_selected ();
+    }
 }
 
 
@@ -1774,5 +1993,14 @@ void
 on_pl_for_each_rating_activate         (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-    each_rating_pl (gp_get_active_itdb ());
+    iTunesDB *itdb = gp_get_selected_itdb();
+
+    if (itdb)
+    {
+	each_rating_pl (itdb);
+    }
+    else
+    {
+	message_sb_no_itdb_selected ();
+    }
 }
