@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-07-02 00:32:38 jcs>
+/* Time-stamp: <2006-09-21 23:12:28 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 | 
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: display.h,v 1.100 2005/07/01 16:53:43 jcsjcs Exp $
+|  $Id: display.h,v 1.116 2006/09/21 15:03:13 jcsjcs Exp $
 */
 
 #ifndef __DISPLAY_H__
@@ -45,6 +45,14 @@ GladeXML *main_window_xml;
 /* pointer to main window */
 extern GtkWidget *gtkpod_window;
 
+/* types for sort */
+enum
+{
+    SORT_ASCENDING = GTK_SORT_ASCENDING,
+    SORT_DESCENDING = GTK_SORT_DESCENDING,
+    SORT_NONE = 10*(GTK_SORT_ASCENDING+GTK_SORT_DESCENDING),
+};
+
 /* Categories in each sort tab (page numbers) */
 typedef enum {
   ST_CAT_ARTIST = 0,
@@ -89,6 +97,7 @@ enum {
 typedef struct {
   gchar *name;
   gboolean master; /* set if this is the "All" entry */
+  gboolean compilation; /* set if this is the "Compilation" entry */
   GList *members;  /* GList with member tracks (pointer to "Track") */
 } TabEntry;
 
@@ -125,7 +134,7 @@ typedef struct {
   TimeInfo ti_played;                /* TimeInfo "played" (sp)        */
   GtkTooltipsData *sp_tooltips_data; /* ptr to tooltips in special st */
   /* function used for string comparisons, set in on_st_switch_page   */
-  gint (*string_compare_func) (gchar *str1, gchar *str2);
+  gint (*string_compare_func) (const gchar *str1, const gchar *str2);
 } SortTab;
 
 /* "Column numbers" in sort tab model */
@@ -141,9 +150,10 @@ typedef enum  {
  * etc.). Since the labels to the buttons are set in prefs_window.c
  * when creating the window, you only need to name the buttons in the
  * intended order using glade-2. There is no need to label them. */
-/* Note: the tm_col_strings[] in display_tracks.c are ordered according
-   to this enum list. Therefore, if you change the order, you must
-   also adjust the order of the tm_col_strings[] array. */
+/* Note: add corresponding entries to T_item and TM_to_T() as well 
+ * (below and in misc_conversion.c). 
+ * IMPORTANT: Do not change the order -- always add new entries at the
+ * end */
 typedef enum  {
   TM_COLUMN_TITLE = 0,
   TM_COLUMN_ARTIST,
@@ -169,16 +179,27 @@ typedef enum  {
   TM_COLUMN_SOUNDCHECK,
   TM_COLUMN_SAMPLERATE,
   TM_COLUMN_BPM,
-  TM_COLUMN_FDESC,
+  TM_COLUMN_FILETYPE,
   TM_COLUMN_GROUPING,         /* 25 */
   TM_COLUMN_COMPILATION,
+  TM_COLUMN_COMMENT,
+  TM_COLUMN_CATEGORY,
+  TM_COLUMN_DESCRIPTION,
+  TM_COLUMN_PODCASTURL,       /* 30 */
+  TM_COLUMN_PODCASTRSS,
+  TM_COLUMN_SUBTITLE,
+  TM_COLUMN_TIME_RELEASED,
+  TM_COLUMN_THUMB_PATH,
   TM_NUM_COLUMNS
 } TM_item;
 
 /* A means to address the fields by uniform IDs. May be extended as
  * needed. You should extend "track_get_item_pointer()" defined in
  * track.c as well for string fields. */
+/* Add corresponding entries to t_strings[] and t_tooltips[] in
+   misc_conversion.c! */
 /* Used in prefs_window.c to label the sort_ign_field<num> buttons */
+/* Used in details.c to label the detail_label_<num> labels */
 typedef enum {
     T_ALL = 0,      /* all fields */
     T_ALBUM,
@@ -187,7 +208,7 @@ typedef enum {
     T_GENRE,
     T_COMMENT,      /*  5 */
     T_COMPOSER,
-    T_FDESC,
+    T_FILETYPE,
     T_PC_PATH,
     T_IPOD_PATH,
     T_IPOD_ID,      /* 10 */
@@ -209,6 +230,18 @@ typedef enum {
     T_CD_NR,
     T_GROUPING,
     T_COMPILATION,
+    T_CATEGORY,
+    T_DESCRIPTION,  /* 30 */
+    T_PODCASTURL,
+    T_PODCASTRSS,
+    T_SUBTITLE,
+    T_TIME_RELEASED,
+    T_CHECKED,      /* 35 */
+    T_STARTTIME,
+    T_STOPTIME,
+    T_REMEMBER_PLAYBACK_POSITION,
+    T_SKIP_WHEN_SHUFFLING,
+    T_THUMB_PATH,   /* 40 */
     T_ITEM_NUM,
 } T_item;
 
@@ -238,8 +271,7 @@ enum {
 
 void display_create (void);
 void display_cleanup (void);
-void display_adjust_delete_menus (void);
-void display_disable_gtkpod_import_buttons(void);
+void display_adjust_menus (void);
 void display_show_hide_toolbar (void);
 void display_reset (gint inst);
 GList *display_get_selection (guint32 inst);
@@ -261,9 +293,14 @@ void pm_name_changed (Playlist *playlist);
 void pm_track_changed (Track *track);
 void pm_sort (GtkSortType order);
 void pm_stop_editing (gboolean cancel);
+void pm_set_renderer_pix (GtkCellRenderer *renderer,
+			  Playlist *playlist);
+void pm_set_renderer_text (GtkCellRenderer *renderer,
+			   Playlist *playlist);
 
 void st_stop_editing (gint inst, gboolean cancel);
 void st_page_selected (GtkNotebook *notebook, guint page);
+void st_redisplay (guint32 inst);
 void st_sort (GtkSortType order);
 void st_remove_entry (TabEntry *entry, guint32 inst);
 gint st_get_instance_from_treeview (GtkTreeView *tv);
@@ -282,7 +319,6 @@ TimeInfo *sp_update_date_interval_from_string (guint32 inst,
 void tm_add_track_to_track_model (Track *track, GtkTreeIter *into_iter);
 guint tm_get_nr_of_tracks(void);
 void tm_rows_reordered(void);
-void pm_rows_reordered(void);
 gboolean tm_add_filelist (gchar *data, GtkTreePath *path,
 			  GtkTreeViewDropPosition pos);
 void tm_stop_editing (gboolean cancel);
@@ -301,7 +337,6 @@ void display_update_default_sizes (void);
 void display_set_default_sizes (void);
 void display_show_hide_tooltips (void);
 void display_set_info_window_menu (void);
-void display_set_check_ipod_menu (void);
 void display_stop_update (gint inst);
 
 void spl_edit (Playlist *spl);
