@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-06-16 22:47:26 jcs>
+/* Time-stamp: <2006-09-21 23:56:52 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: display_itdb.c,v 1.28 2005/06/17 13:30:02 jcsjcs Exp $
+|  $Id: display_itdb.c,v 1.58 2006/09/21 15:03:13 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -34,6 +34,8 @@
 #include <math.h>
 #include <string.h>
 
+#include "charset.h"
+#include "details.h"
 #include "display_itdb.h"
 #include "display.h"
 #include "md5.h"
@@ -42,12 +44,21 @@
 #include "misc_track.h"
 #include "info.h"
 #include "prefs.h"
+#include "syncdir.h"
 
 
 /* A struct containing a list with available iTunesDBs. A pointer to
    this struct is stored in gtkpod_window as itdbs_head */
 static struct itdbs_head *itdbs_head = NULL;
 
+/* for convenience */
+struct itdbs_head *gp_get_itdbs_head (GtkWidget *gtkpod_win)
+{
+    g_return_val_if_fail (gtkpod_win, NULL);
+    return g_object_get_data (G_OBJECT (gtkpod_win),
+			      "itdbs_head");
+}
+
 void gp_itdb_extra_destroy (ExtraiTunesDBData *eitdb)
 {
     if (eitdb)
@@ -113,6 +124,8 @@ ExtraTrackData *gp_track_extra_duplicate (ExtraTrackData *etr)
 	etr_dup->year_str = g_strdup (etr->year_str);
 	etr_dup->pc_path_locale = g_strdup (etr->pc_path_locale);
 	etr_dup->pc_path_utf8 = g_strdup (etr->pc_path_utf8);
+	etr_dup->thumb_path_locale = g_strdup (etr->thumb_path_locale);
+	etr_dup->thumb_path_utf8 = g_strdup (etr->thumb_path_utf8);
 	etr_dup->hostname = g_strdup (etr->hostname);
 	etr_dup->md5_hash = g_strdup (etr->md5_hash);
 	etr_dup->charset = g_strdup (etr->charset);
@@ -130,6 +143,17 @@ iTunesDB *gp_itdb_new (void)
 }
 
 
+/* Free itdb and take care of dependencies */
+void gp_itdb_free (iTunesDB *itdb)
+{
+    if (space_get_ipod_itdb () == itdb)
+    {
+	space_set_ipod_itdb (NULL);
+    }
+    itdb_free (itdb);
+}
+
+
     /* Add and initialize ExtraiTunesDBData if missing */
 void gp_itdb_add_extra (iTunesDB *itdb)
 {
@@ -209,6 +233,7 @@ Track *gp_track_new (void)
     Track *track = itdb_track_new ();
     /* Add ExtraTrackData */
     gp_track_add_extra (track);
+    gp_track_set_flags_default (track);
     return track;
 }
 
@@ -258,7 +283,131 @@ Track *gp_track_add (iTunesDB *itdb, Track *track)
     return result;
 }
 
-/* add itdb to itdbs */
+
+/* Remove track and notify all windows of the change
+   NOTE: you need to notify the main display via pm_remove_track()
+   before when you make sure @track is no longer referenced in any
+   playlist -- see gp_playlist_remove_track for details */
+void gp_track_remove (Track *track)
+{
+    /* currently only the details window may be accessing the tracks */
+    details_remove_track (track);
+    /* remove from database */
+    itdb_track_remove (track);
+}
+
+
+/* Unlink track and notify all windows of the change
+   NOTE: you need to notify the main display via pm_remove_track()
+   before when you make sure @track is no longer referenced in any
+   playlist -- see gp_playlist_remove_track for details */
+void gp_track_unlink (Track *track)
+{
+    /* currently only the details window may be accessing the tracks */
+    details_remove_track (track);
+    /* remove from database */
+    itdb_track_unlink (track);
+}
+
+
+/* Set a thumbnail and update ExtraTrackData (e.g. filename) */
+static gboolean gp_track_set_thumbnails_internal (Track *track,
+						  const gchar *filename,
+						  const guchar *image_data,
+						  gsize image_data_len)
+{
+    gboolean result = FALSE;
+    ExtraTrackData *etr;
+
+    g_return_val_if_fail (track, FALSE);
+
+    etr = track->userdata;
+    g_return_val_if_fail (etr, FALSE);
+
+    if (filename)
+    {
+	result = itdb_track_set_thumbnails (track, filename);
+    }
+    else if (image_data)
+    {
+	result = itdb_track_set_thumbnails_from_data (track,
+						      image_data,
+						      image_data_len);
+    }
+
+    g_free (etr->thumb_path_locale);
+    g_free (etr->thumb_path_utf8);
+
+    if (filename && (result == TRUE))
+    {
+	etr->thumb_path_locale = g_strdup (filename);
+	etr->thumb_path_utf8 = charset_to_utf8 (filename);
+    }
+    else
+    {
+	etr->thumb_path_locale = g_strdup ("");
+	etr->thumb_path_utf8 = g_strdup ("");
+    }
+
+    if (result == FALSE)
+    {
+	if (filename)
+	    gtkpod_warning (_("Failed to set cover art: '%s'\n"), filename);
+    }
+
+    return result;
+}
+
+/* Set a thumbnail and update data in ExtraTrackData */
+gboolean gp_track_set_thumbnails_from_data (Track *track,
+					    const guchar *image_data,
+					    gsize image_data_len)
+{
+    g_return_val_if_fail (track, FALSE);
+    g_return_val_if_fail (image_data, FALSE);
+
+    return gp_track_set_thumbnails_internal (track, NULL,
+					     image_data, image_data_len);
+}
+
+
+/* Set a thumbnail and store the filename in ExtraTrackData */
+gboolean gp_track_set_thumbnails (Track *track, const gchar *filename)
+{
+    g_return_val_if_fail (track, FALSE);
+    g_return_val_if_fail (filename, FALSE);
+
+    return gp_track_set_thumbnails_internal (track, filename,
+					     NULL, 0);
+}
+
+
+/* Remove a thumbnail and remove the filename in ExtraTrackData */
+/* Return value:
+   FALSE: track did not have any thumbnails, so no change was done
+   TRUE: track did have thumbnails which were removed */
+gboolean gp_track_remove_thumbnails (Track *track)
+{
+    gboolean changed=FALSE;
+
+    ExtraTrackData *etr;
+    g_return_val_if_fail (track, FALSE);
+    etr = track->userdata;
+    g_return_val_if_fail (etr, FALSE);
+
+    if (track->artwork->thumbnails)
+	changed = TRUE;
+
+    itdb_track_remove_thumbnails (track);
+    g_free (etr->thumb_path_locale);
+    g_free (etr->thumb_path_utf8);
+    etr->thumb_path_locale = g_strdup ("");
+    etr->thumb_path_utf8 = g_strdup ("");
+
+    return changed;
+}
+
+/* add itdb to itdbs (and add to display) */
 void gp_itdb_add (iTunesDB *itdb, gint pos)
 {
     ExtraiTunesDBData *eitdb;
@@ -273,6 +422,17 @@ void gp_itdb_add (iTunesDB *itdb, gint pos)
     pm_add_itdb (itdb, pos);
 }
 
+/* Remove itdb to itdbs (and remove from display). Call
+ * itdb_free() to free the memory of the itdb. */
+void gp_itdb_remove (iTunesDB *itdb)
+{
+    g_return_if_fail (itdbs_head);
+    g_return_if_fail (itdb);
+
+    pm_remove_playlist (itdb_playlist_mpl (itdb), FALSE);
+    itdbs_head->itdbs = g_list_remove (itdbs_head->itdbs, itdb);
+}
+
 /* Also replaces @old_itdb in the itdbs GList and take care that the
  * displayed itdb gets replaced as well */
 void gp_replace_itdb (iTunesDB *old_itdb, iTunesDB *new_itdb)
@@ -323,7 +483,7 @@ void gp_replace_itdb (iTunesDB *old_itdb, iTunesDB *new_itdb)
     new_eitdb->itdbs_head = itdbs_head;
     old_link->data = new_itdb;
     /* free old_itdb */
-    itdb_free (old_itdb);
+    gp_itdb_free (old_itdb);
 
     /* display replacement */
     pm_add_itdb (new_itdb, pos);
@@ -334,11 +494,13 @@ void gp_replace_itdb (iTunesDB *old_itdb, iTunesDB *new_itdb)
 	Playlist *pl = itdb_playlist_by_name (new_itdb, old_pl_name);
 	if (pl) pm_select_playlist (pl);
     }
+
+    /* Set prefs system with name of MPL */
+    mpl = itdb_playlist_mpl (new_itdb);
+    set_itdb_prefs_string (new_itdb, "name", mpl->name);
 }    
 
 
-
-
 /* add playlist to itdb and to display */
 void gp_playlist_add (iTunesDB *itdb, Playlist *pl, gint32 pos)
 {
@@ -443,6 +605,8 @@ void gp_playlist_remove_track (Playlist *plitem, Track *track,
 			       DeleteAction deleteaction)
 {
     iTunesDB *itdb;
+    Playlist *mpl;
+    gboolean remove_track = FALSE;
 
     g_return_if_fail (track);
     itdb = track->itdb;
@@ -453,8 +617,10 @@ void gp_playlist_remove_track (Playlist *plitem, Track *track,
     case DELETE_ACTION_IPOD:
     case DELETE_ACTION_LOCAL:
     case DELETE_ACTION_DATABASE:
-	/* remove from MPL in these cases */
-	plitem = NULL;
+	/* remove from MPL in these cases (unless we are removing
+	   podcasts) */
+	if (!(plitem && itdb_playlist_is_podcasts (plitem)))
+	    plitem = NULL;
 	break;
     case DELETE_ACTION_PLAYLIST:
 	/* cannot remove from MPL */
@@ -462,7 +628,10 @@ void gp_playlist_remove_track (Playlist *plitem, Track *track,
 	break;
     }
 
-    if (plitem == NULL)  plitem = itdb_playlist_mpl (track->itdb);
+    mpl = itdb_playlist_mpl (track->itdb);
+
+    if (plitem == NULL)
+	plitem = mpl;
     
     g_return_if_fail (plitem);
 
@@ -472,7 +641,29 @@ void gp_playlist_remove_track (Playlist *plitem, Track *track,
     /* remove track from playlist */
     itdb_playlist_remove_track (plitem, track);
 
-    if (plitem->type == ITDB_PL_TYPE_MPL)
+    /* if we removed a podcasts, remove it from memory as well, unless
+       it's present in the MPL (this happens if this podcast was on
+       the iPod as podcast as well as standard track) */
+    if (itdb_playlist_is_podcasts (plitem))
+    {
+	/* just for safety: remove possible duplicates of @track in
+	   the podcast playlist before removing it from memory */
+	while (g_list_find (plitem->members, track))
+	{
+	    pm_remove_track (plitem, track);
+	    itdb_playlist_remove_track (plitem, track);
+	}
+	if (!itdb_playlist_contains_track (mpl, track))
+	{
+	    remove_track = TRUE;
+	}
+	else
+	{   /* strip the podcast flags */
+	    gp_track_set_flags_default (track);
+	}
+    }
+
+    if (itdb_playlist_is_mpl (plitem))
     { /* if it's the MPL, we remove the track permanently */
 	GList *gl = g_list_nth (itdb->playlists, 1);
 	ExtraiTunesDBData *eitdb = itdb->userdata;
@@ -482,12 +673,19 @@ void gp_playlist_remove_track (Playlist *plitem, Track *track,
 	{  /* first we remove the track from all other playlists (i=1) */
 	    Playlist *pl = gl->data;
 	    g_return_if_fail (pl);
-	    pm_remove_track (pl, track);
-	    itdb_playlist_remove_track (pl, track);
+	    while (g_list_find (pl->members, track))
+	    {
+		pm_remove_track (pl, track);
+		itdb_playlist_remove_track (pl, track);
+	    }
 	    gl=gl->next;
 	}
-	md5_track_remove (track);
+	remove_track = TRUE;
+    }
 
+    if (remove_track)
+    {
+	md5_track_remove (track);
 	if (itdb->usertype & GP_ITDB_TYPE_IPOD)
 	{
 	    switch (deleteaction)
@@ -495,17 +693,17 @@ void gp_playlist_remove_track (Playlist *plitem, Track *track,
 	    case DELETE_ACTION_DATABASE:
 		/* ATTENTION: this might create a dangling file on the
 		   iPod! */
-		itdb_track_remove (track);
+		gp_track_remove (track);
 		break;
 	    case DELETE_ACTION_IPOD:
 		if (track->transferred)
 		{
-		    itdb_track_unlink (track);
+		    gp_track_unlink (track);
 		    mark_track_for_deletion (itdb, track);
 		}
 		else
 		{
-		    itdb_track_remove (track);
+		    gp_track_remove (track);
 		}
 		break;
 	    case DELETE_ACTION_PLAYLIST:
@@ -521,11 +719,11 @@ void gp_playlist_remove_track (Playlist *plitem, Track *track,
 	    switch (deleteaction)
 	    {
 	    case DELETE_ACTION_LOCAL:
-		itdb_track_unlink (track);
+		gp_track_unlink (track);
 		mark_track_for_deletion (itdb, track);
 		break;
 	    case DELETE_ACTION_DATABASE:
-		itdb_track_remove (track);
+		gp_track_remove (track);
 		break;
 	    case DELETE_ACTION_PLAYLIST:
 	    case DELETE_ACTION_IPOD:
@@ -543,6 +741,8 @@ void gp_playlist_remove_track (Playlist *plitem, Track *track,
    playlist @pl. It then lets the display model know.
    @display: if TRUE, track is added the display.  Otherwise it's only
    added to memory */
+/* All tracks added to the podcast playlist will get the mark_unplayed
+   flag get set */
 void gp_playlist_add_track (Playlist *pl, Track *track, gboolean display)
 {
     iTunesDB *itdb;
@@ -552,7 +752,12 @@ void gp_playlist_add_track (Playlist *pl, Track *track, gboolean display)
     itdb = pl->itdb;
     g_return_if_fail (itdb);
 
-    pl->members = g_list_append (pl->members, track);
+    itdb_playlist_add_track (pl, track, -1);
+    if (itdb_playlist_is_podcasts (pl))
+    {   /* have the iPod display a bullet in front of the track as it
+	   has been newly added */
+	track->mark_unplayed = 0x02;
+    }
     if (display)  pm_add_track (pl, track, TRUE);
 
     data_changed (itdb);
@@ -570,17 +775,24 @@ void gp_track_validate_entries (Track *track)
     etr = track->userdata;
     g_return_if_fail (etr);
 
-    if (!track->album)           track->album = g_strdup ("");
-    if (!track->artist)          track->artist = g_strdup ("");
     if (!track->title)           track->title = g_strdup ("");
+    if (!track->artist)          track->artist = g_strdup ("");
+    if (!track->album)           track->album = g_strdup ("");
     if (!track->genre)           track->genre = g_strdup ("");
-    if (!track->comment)         track->comment = g_strdup ("");
     if (!track->composer)        track->composer = g_strdup ("");
-    if (!track->fdesc)           track->fdesc = g_strdup ("");
+    if (!track->comment)         track->comment = g_strdup ("");
+    if (!track->filetype)        track->filetype = g_strdup ("");
     if (!track->grouping)        track->grouping = g_strdup ("");
+    if (!track->category)        track->category = g_strdup ("");
+    if (!track->description)     track->description = g_strdup ("");
+    if (!track->podcasturl)      track->podcasturl = g_strdup ("");
+    if (!track->podcastrss)      track->podcastrss = g_strdup ("");
+    if (!track->subtitle)        track->subtitle = g_strdup ("");
+    if (!track->ipod_path)       track->ipod_path = g_strdup ("");
     if (!etr->pc_path_utf8)      etr->pc_path_utf8 = g_strdup ("");
     if (!etr->pc_path_locale)    etr->pc_path_locale = g_strdup ("");
-    if (!track->ipod_path)       track->ipod_path = g_strdup ("");
+    if (!etr->thumb_path_utf8)   etr->thumb_path_utf8 = g_strdup ("");
+    if (!etr->thumb_path_locale) etr->thumb_path_locale = g_strdup ("");
     /* Make sure year_str is identical to year */
     g_free (etr->year_str);
     etr->year_str = g_strdup_printf ("%d", track->year);
@@ -599,7 +811,7 @@ void gp_track_validate_entries (Track *track)
 void init_data (GtkWidget *window)
 {
     gchar *cfgdir;
-    gint type;
+    gint i;
 
     g_return_if_fail (window);
     g_return_if_fail (itdbs_head == NULL);
@@ -610,222 +822,197 @@ void init_data (GtkWidget *window)
 
     g_object_set_data (G_OBJECT (window), "itdbs_head", itdbs_head);
 
-    if (prefs_get_int_value ("itdb_0_type", &type))
-    {   /* databases have been set up previously -- use what is stored
-	 * in the prefs */
-	gint i;
-	for (i=0;;++i)
-	{
-	    gchar *property = g_strdup_printf ("itdb_%d_type", i);
-	    gboolean valid = prefs_get_int_value (property, &type);
-	    g_free (property);
-	    if (valid)
-	    {
-		iTunesDB *itdb = NULL;
-		Playlist *pl = NULL;
-		ExtraiTunesDBData *eitdb;
-		gchar *filename = NULL;
-		gchar *mountpoint = NULL;
-		gchar *offline_filename = NULL;
-
-		if (type & GP_ITDB_TYPE_LOCAL)
-		{
-		    gchar *fn = g_strdup_printf ("itdb_%d_filename", i);
-
-		    filename = prefs_get_string (fn);
-
-		    if (!filename)
-		    {
-			gchar *local = g_strdup_printf ("local%d.itdb",i);
-			filename = g_build_filename (cfgdir, local, NULL);
-			g_free (local);
-		    }
-		    g_free (fn);
-		    if (g_file_test (filename, G_FILE_TEST_EXISTS))
-			itdb = gp_import_itdb (NULL, type,
-					       NULL, NULL, filename);
-		}
-		else if (type & GP_ITDB_TYPE_IPOD)
-		{
-		    gchar *mp = g_strdup_printf ("itdb_%d_mountpoint", i);
-		    gchar *fn = g_strdup_printf ("itdb_%d_filename", i);
-		    
-
-		    offline_filename = prefs_get_string (fn);
-		    mountpoint = prefs_get_string (mp);
-
-		    if (!offline_filename)
-		    {
-			gchar *local = g_strdup_printf ("gtkpod_%d.itdb",i);
-			offline_filename = g_build_filename (
-			    cfgdir, local, NULL);
-			g_free (local);
-		    }
-		    if (!mountpoint)
-			mountpoint = g_strdup (prefs_get_ipod_mount ());
-		    g_free (fn);
-		    g_free (mp);
-		}
-		else
-		{
-		    g_return_if_reached ();
-		}
-
-
-		if (!itdb)
-		{
-		    gchar *nm, *name;
-
-		    itdb = gp_itdb_new ();
-		    eitdb = itdb->userdata;
-		    g_return_if_fail (eitdb);
-		    itdb->usertype = type;
-		    itdb->filename = filename;
-		    itdb->mountpoint = mountpoint;
-		    eitdb->offline_filename = offline_filename;
-
-		    nm = g_strdup_printf ("itdb_%d_name", i);
-		    if (!prefs_get_string_value (nm, &name))
-		    {
-			if (type & GP_ITDB_TYPE_LOCAL)
-			    name = g_strdup (_("Local"));
-			else
-			    name = g_strdup ("gtkpod");
-		    }
-		    pl = gp_playlist_new (name, FALSE);
-		    g_free (name);
-		    g_free (nm);
-		    pl->type = ITDB_PL_TYPE_MPL;
-		    itdb_playlist_add (itdb, pl, -1);
-
-		    eitdb->data_changed = FALSE;
-		    eitdb->itdb_imported = FALSE;
-		}
-		else
-		{
-		    g_free (filename);
-		    g_free (mountpoint);
-		    g_free (offline_filename);
-		}
-
-		/* add to the display */
-		gp_itdb_add (itdb, -1);
-	    }
-	    else
-	    {
-		break; /* for (i=0;;++i) */
-	    }
-	}
-    }
-    else
-    {   /* first run -- set up a local database and an iPod database */
-	iTunesDB *itdb;
-	ExtraiTunesDBData *eitdb;
-	gchar *fn;
-	Playlist *pl;
+    if (!prefs_get_int_value ("itdb_0_type", NULL))
+    {
+	/* databases have not been set up previously -- take care of
+	   this */
+	gchar *mountpoint, *filename;
 
 	/* iPod database */
-	itdb = gp_itdb_new ();
-	eitdb = itdb->userdata;
-	g_return_if_fail (eitdb);
-	itdb->usertype = GP_ITDB_TYPE_IPOD;
-	itdb->mountpoint = g_strdup (prefs_get_ipod_mount ());
-	eitdb->offline_filename = g_build_filename (
-	    cfgdir, "iTunesDB", NULL);
-	gp_itdb_add (itdb, -1);
-	pl = gp_playlist_new ("gtkpod", FALSE);
-	pl->type = ITDB_PL_TYPE_MPL;  /* MPL! */
-	gp_playlist_add (itdb, pl, -1);
-	g_return_if_fail (itdb->userdata);
-	eitdb->data_changed = FALSE;
+	mountpoint = prefs_get_string ("initial_mountpoint");
+	filename = g_build_filename (cfgdir, "iTunesDB", NULL);
+	prefs_set_int ("itdb_0_type", GP_ITDB_TYPE_IPOD);
+	prefs_set_string ("itdb_0_name", _("iPod"));
+	prefs_set_string ("itdb_0_filename", filename);
+	prefs_set_string ("itdb_0_mountpoint", mountpoint);
+	g_free (mountpoint);
+	g_free (filename);
 
-	/* local database. First check if a database file already
-	   exists -- if yes load it */
-	itdb = NULL;
-	fn = g_build_filename (cfgdir, "local_0.itdb", NULL);
-	if (g_file_test (fn, G_FILE_TEST_EXISTS))
-	{
-	    itdb = gp_import_itdb (NULL, GP_ITDB_TYPE_LOCAL,
-				   NULL, NULL, fn);
-	}
-	if (!itdb)
-	{   /* local database does not exist or cannot be loaded */
-	    itdb = gp_itdb_new ();
-	    eitdb = itdb->userdata;
-	    g_return_if_fail (eitdb);
-	    itdb->usertype = GP_ITDB_TYPE_LOCAL;
-	    itdb->filename = g_strdup (fn);
-	    pl = gp_playlist_new (_("Local"), FALSE);
-	    pl->type = ITDB_PL_TYPE_MPL;  /* MPL! */
-	    itdb_playlist_add (itdb, pl, -1);
-	}
-	gp_itdb_add (itdb, -1);
-	g_free (fn);
+	/* Local database */
+	filename = g_build_filename (cfgdir, "local_0.itdb", NULL);
+	prefs_set_int ("itdb_1_type", GP_ITDB_TYPE_LOCAL);
+	prefs_set_string ("itdb_1_name", _("Local"));
+	prefs_set_string ("itdb_1_filename", filename);
+	g_free (filename);
+
+	/* Podcasts database */
+	filename = g_build_filename (cfgdir, "podcasts.itdb", NULL);
+	prefs_set_int ("itdb_2_type",
+		       GP_ITDB_TYPE_PODCASTS|GP_ITDB_TYPE_LOCAL);
+	prefs_set_string ("itdb_2_name", _("Podcasts"));
+	prefs_set_string ("itdb_2_filename", filename);
+	g_free (filename);
     }
 
-    /* set md5 */
-    prefs_set_md5tracks (prefs_get_md5tracks ());
 
-    /* update prefs with new information in case it's needed */
-    gp_update_itdb_prefs ();
+    for (i=0;;++i)
+    {
+	ExtraiTunesDBData *eitdb;
+	iTunesDB *itdb = setup_itdb_n (i);
+
+	if (itdb == NULL)
+	    break;
+
+	/* add to the display */
+	gp_itdb_add (itdb, -1);
+
+	/* update/sync playlists according to options set */
+	eitdb = itdb->userdata;
+	g_return_if_fail (eitdb);
+	if (eitdb->itdb_imported)
+	{
+	    /* take care of autosync... */
+	    sync_all_playlists (itdb);
+
+	    /* update all live SPLs */
+	    itdb_spl_update_live (itdb);
+	}
+    }
 
     g_free (cfgdir);
 }
 
 
-/* Update the information about the itdbs in the prefs structure */
-void gp_update_itdb_prefs (void)
+
+/* Create an repository according to the settings in the preferences
+   system. */
+iTunesDB *setup_itdb_n (gint i)
 {
-    struct itdbs_head *itdbs_head;
-    GList *gl;
-    gint i=0;
-
-    g_return_if_fail (gtkpod_window);
-    itdbs_head = g_object_get_data (G_OBJECT (gtkpod_window),
-				    "itdbs_head");
-    g_return_if_fail (itdbs_head);
-    for (gl=itdbs_head->itdbs; gl; gl=gl->next)
+    iTunesDB *itdb = NULL;
+    gchar *property = get_itdb_prefs_key (i, "type");
+    gint type;
+    gboolean valid = prefs_get_int_value (property, &type);
+    g_free (property);
+    if (valid)
     {
-	gchar *prop, *prop2;
-	Playlist *mpl;
+	gchar *cfgdir = prefs_get_cfgdir ();
+	Playlist *pl = NULL;
 	ExtraiTunesDBData *eitdb;
-	iTunesDB *itdb = gl->data;
+	gchar *filename = NULL;
+	gchar *mountpoint = NULL;
+	gchar *offline_filename = NULL;
 
-	g_return_if_fail (itdb);
-	eitdb = itdb->userdata;
-	g_return_if_fail (eitdb);
-
-	prop = g_strdup_printf ("itdb_%d_type", i);
-	prefs_set_int_value (prop, itdb->usertype);
-	g_free (prop);
-
-	mpl = itdb_playlist_mpl (itdb);
-	g_return_if_fail (mpl);
-	prop = g_strdup_printf ("itdb_%d_name", i);
-	prefs_set_string_value (prop, mpl->name);
-	g_free (prop);
-
-	prop = g_strdup_printf ("itdb_%d_filename", i);
-	prop2 = g_strdup_printf ("itdb_%d_mountpoint", i);
-	if (itdb->usertype & GP_ITDB_TYPE_LOCAL)
+	if (type & GP_ITDB_TYPE_LOCAL)
 	{
-	    prefs_set_string_value (prop, itdb->filename);
-	    prefs_set_string_value (prop2, NULL);
+	    gchar *fn = get_itdb_prefs_key (i, "filename");
+
+	    filename = prefs_get_string (fn);
+
+	    if (!filename)
+	    {
+		gchar *local = g_strdup_printf ("local%d.itdb",i);
+		filename = g_build_filename (cfgdir, local, NULL);
+		g_free (local);
+	    }
+	    g_free (fn);
+	    if (g_file_test (filename, G_FILE_TEST_EXISTS))
+		itdb = gp_import_itdb (NULL, type,
+				       NULL, NULL, filename);
 	}
-	else if (itdb->usertype & GP_ITDB_TYPE_IPOD)
+	else if (type & GP_ITDB_TYPE_IPOD)
 	{
-	    prefs_set_string_value (prop, eitdb->offline_filename);
-	    prefs_set_string_value (prop2, itdb->mountpoint);
+	    gchar *key;
+
+	    key = get_itdb_prefs_key (i, KEY_MOUNTPOINT);
+	    mountpoint = prefs_get_string (key);
+	    g_free (key);
+
+	    key = get_itdb_prefs_key (i, "filename");
+	    offline_filename = prefs_get_string (key);
+	    g_free (key);
+
+	    if (!offline_filename)
+	    {
+		gchar *local = g_strdup_printf ("gtkpod_%d.itdb",i);
+		offline_filename = g_build_filename (
+		    cfgdir, local, NULL);
+		g_free (local);
+	    }
 	}
 	else
 	{
-	    g_return_if_reached ();
+	    g_return_val_if_reached (NULL);
 	}
-	g_free (prop);
-	g_free (prop2);
 
-	++i;
+	if (!itdb)
+	{
+	    gchar *nm, *name;
+
+	    itdb = gp_itdb_new ();
+	    eitdb = itdb->userdata;
+	    g_return_val_if_fail (eitdb, NULL);
+	    itdb->usertype = type;
+	    itdb->filename = filename;
+	    itdb_set_mountpoint (itdb, mountpoint);
+	    eitdb->offline_filename = offline_filename;
+
+	    nm = g_strdup_printf ("itdb_%d_name", i);
+	    if (!prefs_get_string_value (nm, &name))
+	    {
+		if (type & GP_ITDB_TYPE_PODCASTS)
+		    name = g_strdup (_("Podcasts"));
+		else if (type & GP_ITDB_TYPE_LOCAL)
+		    name = g_strdup (_("Local"));
+		else
+		    name = g_strdup (_("iPod"));
+	    }
+	    pl = gp_playlist_new (name, FALSE);
+	    g_free (name);
+	    g_free (nm);
+	    itdb_playlist_set_mpl (pl);
+	    itdb_playlist_add (itdb, pl, -1);
+
+	    eitdb->data_changed = FALSE;
+	    eitdb->itdb_imported = FALSE;
+	}
+	else
+	{
+	    g_free (filename);
+	    g_free (offline_filename);
+	}
+	g_free (mountpoint);
+
+	/* Check if Podcast Playlist is present on IPOD itdb
+	 * and add if not. If Podcast Playlist is present on
+	 * local itdb remove it. */
+	pl = itdb_playlist_podcasts (itdb);
+	if ((type & GP_ITDB_TYPE_IPOD) && !pl)
+	{   /* add podcast playlist */
+	    pl = gp_playlist_new (_("Podcasts"), FALSE);
+	    itdb_playlist_set_podcasts (pl);
+	    itdb_playlist_add (itdb, pl, -1);
+	    eitdb = itdb->userdata;
+	    g_return_val_if_fail (eitdb, NULL);
+	    eitdb->data_changed = FALSE;
+	}
+	if ((type & GP_ITDB_TYPE_LOCAL) && pl)
+	{   /* Remove podcast playlist. Normally no playlist
+	     * should be present, except for a few people who
+	     * used the CVS version between September and
+	     * October 2005. */
+	    if (itdb_playlist_tracks_number (pl) == 0)
+	    {
+		gp_playlist_remove (pl);
+	    }
+	    else
+	    {   /* OK, let's be nice and just drop the
+		   'podcast' flag instead of removing */
+		pl->podcastflag = 0;
+	    }
+	}
+	g_free (cfgdir);
     }
+    return itdb;
 }
 
 
@@ -855,14 +1042,13 @@ gboolean gp_increase_playcount (gchar *md5, gchar *file, gint num)
 	if (!track)	  track = gp_track_by_filename (itdb, file);
 	if (track)
 	{
-	    gchar *buf1, *buf;
+	    gchar *buf1;
 	    track->playcount += num;
 	    data_changed (itdb);
 	    pm_track_changed (track);
 	    buf1 = get_track_info (track, TRUE);
-	    buf = g_strdup_printf (_("Increased playcount for '%s'"), buf1);
-	    gtkpod_statusbar_message (buf);
-	    g_free (buf);
+	    gtkpod_statusbar_message (_("Increased playcount for '%s'"),
+				      buf1);
 	    g_free (buf1);
 	    if (itdb->usertype & GP_ITDB_TYPE_IPOD)    result = TRUE;
 	}
@@ -871,12 +1057,11 @@ gboolean gp_increase_playcount (gchar *md5, gchar *file, gint num)
 }
 
 
-/* determine "active" itdb -- it's either the itdb of the playlist
- * currently selected, or the first itdb if none is selected */
-iTunesDB *gp_get_active_itdb (void)
+/* get the currently selected itdb. NULL is
+ * returned if no itdb is active. */
+iTunesDB *gp_get_selected_itdb (void)
 {
     Playlist *pl = pm_get_selected_playlist ();
-    struct itdbs_head *itdbs_head;
 
     /* If playlist is selected, use the itdb of the playlist as the
        active itdb */
@@ -886,34 +1071,69 @@ iTunesDB *gp_get_active_itdb (void)
 	return pl->itdb;
     }
 
-    /* Otherwise choose the first itdb */
-    g_return_val_if_fail (gtkpod_window, NULL);
-    itdbs_head = g_object_get_data (G_OBJECT (gtkpod_window),
-				    "itdbs_head");
-    g_return_val_if_fail (itdbs_head, NULL);
-    g_return_val_if_fail (itdbs_head->itdbs, NULL);
-    g_return_val_if_fail (itdbs_head->itdbs->data, NULL);
-    return itdbs_head->itdbs->data;
+    return NULL;
 }
 
 
-/* get the "ipod" itdb, that's the first itdb with
-   type==GP_ITDB_TYPE_IPOD. Returns NULL and prints error when no
-   matching itdb can be found */
+/* Get the "ipod" itdb. If only one iPod itdb exists, this itdb is
+ * returned. If more than one iPod itdb exists, the currently selected
+ * itdb is returned if it's an iPod itdb, otherwise NULL is returned.
+ */
 iTunesDB *gp_get_ipod_itdb (void)
 {
     struct itdbs_head *itdbs_head;
+    iTunesDB *itdb;
     GList *gl;
+    gint i;
+
+    /* if an iPod itdb is selected, return this */
+    itdb = gp_get_selected_itdb ();
+    if (itdb && (itdb->usertype & GP_ITDB_TYPE_IPOD))
+	return itdb;
+
+    itdb = NULL;
 
     g_return_val_if_fail (gtkpod_window, NULL);
     itdbs_head = g_object_get_data (G_OBJECT (gtkpod_window),
 				    "itdbs_head");
+
+    if (itdbs_head == NULL) return NULL;
+
+    i=0;
+    for (gl=itdbs_head->itdbs; gl; gl=gl->next)
+    {
+	iTunesDB *itdbgl = gl->data;
+	g_return_val_if_fail (itdbgl, NULL);
+	if (itdbgl->usertype & GP_ITDB_TYPE_IPOD)
+	{
+	    itdb = itdbgl;
+	    ++i;
+	}
+    }
+    /* return iPod itdb if only one was found */
+    if (i == 1)
+	return itdb;
+
+    return NULL;
+}
+
+
+/* return the podcast itdb */
+iTunesDB *gp_get_podcast_itdb ()
+{
+    GList *gl;
+
     g_return_val_if_fail (itdbs_head, NULL);
+
     for (gl=itdbs_head->itdbs; gl; gl=gl->next)
     {
 	iTunesDB *itdb = gl->data;
 	g_return_val_if_fail (itdb, NULL);
-	if (itdb->usertype & GP_ITDB_TYPE_IPOD)  return itdb;
+
+	if (itdb->usertype & GP_ITDB_TYPE_PODCASTS)
+	    return itdb;
     }
-    g_return_val_if_reached (NULL);
+    return NULL;
 }
+
+
