@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-12-10 14:43:34 jcs>
+/* Time-stamp: <2006-06-28 01:25:27 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 | 
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: display_playlists.c,v 1.71 2005/12/10 08:21:59 jcsjcs Exp $
+|  $Id: display_playlists.c,v 1.92 2006/06/27 16:29:43 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -66,10 +66,8 @@ static GtkTargetEntry pm_drop_types [] = {
 };
 
 
-/* SEMIBOLD was introduced with version 1.8 */
-#ifndef PANGO_WEIGHT_SEMIBOLD
-#define PANGO_WEIGHT_SEMIBOLD 600
-#endif
+void pm_rows_reordered (void);
+
 
 
 /* ---------------------------------------------------------------- */
@@ -306,7 +304,7 @@ on_pm_dnd_get_file_foreach(GtkTreeModel *tm, GtkTreePath *tp,
 	Track *track = gl->data;
 
 	g_return_if_fail (track);
-	name = get_file_name_verified (track);
+	name = get_file_name_from_source (track, SOURCE_PREFER_LOCAL);
 	if (name)
 	{
 	    g_string_append_printf (filelist, "file:%s\n", name);
@@ -341,7 +339,7 @@ on_pm_dnd_get_uri_foreach(GtkTreeModel *tm, GtkTreePath *tp,
 	Track *track = gl->data;
 
 	g_return_if_fail (track);
-	name = get_file_name_verified (track);
+	name = get_file_name_from_source (track, SOURCE_PREFER_LOCAL);
 	if (name)
 	{
 	    gchar *uri = g_filename_to_uri (name, NULL, NULL);
@@ -491,7 +489,7 @@ static GdkDragAction pm_pm_get_action (Playlist *src, Playlist *dest,
     else
     {   /* DND between different itdbs */
 	/* Do not allow drags from the iPod in offline mode */
-	if (prefs_get_offline () &&
+	if (get_offline (src->itdb) &&
 	    (src->itdb->usertype & GP_ITDB_TYPE_IPOD))
 	{   /* give a notice on the statusbar -- otherwise the user
 	     * will never know why the drag is not possible */
@@ -533,7 +531,7 @@ static GdkDragAction pm_tm_get_action (Track *src, Playlist *dest,
     else
     {   /* drag between different itdbs */
 	/* Do not allow drags from the iPod in offline mode */
-	if (prefs_get_offline () &&
+	if (get_offline (src->itdb) &&
 	    (src->itdb->usertype & GP_ITDB_TYPE_IPOD))
 	{   /* give a notice on the statusbar -- otherwise the user
 	     * will never know why the drag is not possible */
@@ -553,7 +551,7 @@ static void pm_tm_tracks_moved_or_copied (gchar *tracks, gboolean moved)
     g_return_if_fail (tracks);
     if (!moved)
     {
-	gchar *buf, *ptr = tracks;
+	gchar *ptr = tracks;
 	gint n = 0;
 
 	/* count the number of tracks */
@@ -562,13 +560,10 @@ static void pm_tm_tracks_moved_or_copied (gchar *tracks, gboolean moved)
 	    ++n;
 	    ++ptr;
 	}
-	/* create message */
-	buf = g_strdup_printf (
+	/* display message in statusbar */
+	gtkpod_statusbar_message (
 	    ngettext ("Copied one track",
 		      "Copied %d tracks", n), n);
-	/* display message in statusbar */
-	gtkpod_statusbar_message (buf);
-	g_free (buf);
     }
 }
 
@@ -851,7 +846,7 @@ static void pm_drag_data_received (GtkWidget       *widget,
 		gtk_drag_finish (dc, TRUE, FALSE, time);
 		break;
 	    case GDK_ACTION_MOVE:
-		if (prefs_get_pm_sort () != SORT_NONE)
+		if (prefs_get_int("pm_sort") != SORT_NONE)
 		{
 		    gtkpod_statusbar_message (_("Can't reorder sorted treeview."));
 		    gtk_drag_finish (dc, FALSE, FALSE, time);
@@ -888,6 +883,7 @@ static void pm_drag_data_received (GtkWidget       *widget,
 		gtk_drag_finish (dc, FALSE, FALSE, time);
 		g_return_if_reached ();
 	    }
+	    pm_rows_reordered ();
 	    return;
 	}
 	else
@@ -942,6 +938,7 @@ static void pm_drag_data_received (GtkWidget       *widget,
 	    }
 
 	}
+	pm_rows_reordered ();
 	break;
     default:
 	gtkpod_warning (_("This DND type (%d) is not (yet) supported. If you feel implementing this would be useful, please contact the author.\n\n"), info);
@@ -968,14 +965,22 @@ on_playlist_treeview_key_release_event (GtkWidget       *widget,
 
     if(!widgets_blocked && (mods & GDK_CONTROL_MASK))
     {
+	iTunesDB *itdb = gp_get_selected_itdb();
+
 	switch(event->keyval)
 	{
 /* 	    case GDK_u: */
 /* 		gp_do_selected_playlist (update_tracks); */
 /* 		break; */
 	    case GDK_n:
-		add_new_pl_or_spl_user_name (gp_get_active_itdb(),
-					     NULL, -1);
+		if (itdb)
+		{
+		    add_new_pl_or_spl_user_name (itdb, NULL, -1);
+		}
+		else
+		{
+		    message_sb_no_itdb_selected ();
+		}
 		break;
 	    default:
 		break;
@@ -1025,11 +1030,11 @@ void pm_add_track (Playlist *playlist, Track *track, gboolean display)
 /* Used by pm_name_changed() to find the playlist that
    changed name. If found, emit a "row changed" signal to display the change */
 static gboolean sr_model_playlist_name_changed (GtkTreeModel *model,
-					GtkTreePath *path,
-					GtkTreeIter *iter,
-					gpointer data)
+						GtkTreePath *path,
+						GtkTreeIter *iter,
+						gpointer data)
 {
-  Playlist *playlist;
+  Playlist *playlist=NULL;
 
   gtk_tree_model_get (model, iter, PM_COLUMN_PLAYLIST, &playlist, -1);
   if(playlist == (Playlist *)data) {
@@ -1131,7 +1136,7 @@ void pm_add_playlist (Playlist *playlist, gint pos)
       /* If it's the first playlist (no playlist selected AND playlist is
 	 the MPL, we select it, unless prefs_get_mpl_autoselect() says
 	 otherwise */
-      if (itdb_playlist_is_mpl(playlist) && prefs_get_mpl_autoselect())
+      if (itdb_playlist_is_mpl(playlist) && prefs_get_int("mpl_autoselect"))
       {
 	  selection = gtk_tree_view_get_selection (playlist_treeview);
 	  gtk_tree_selection_select_iter (selection, &iter);
@@ -1140,18 +1145,14 @@ void pm_add_playlist (Playlist *playlist, gint pos)
 }
 
 
-/* Remove "playlist" from the display model. 
-   "select": TRUE: a new playlist is selected
-             FALSE: no selection is taking place
-                    (useful when quitting program) */
-void pm_remove_playlist (Playlist *playlist, gboolean select)
-{
-    gboolean pm_delete_playlist_fe (GtkTreeModel *model,
-				    GtkTreePath *path,
-				    GtkTreeIter *iter,
-				    gpointer data)
+
+    /* former inline function: used by pm_remove_playlist() */
+    static gboolean pm_delete_playlist_fe (GtkTreeModel *model,
+					   GtkTreePath *path,
+					   GtkTreeIter *iter,
+					   gpointer data)
 	{
-	    Playlist *playlist;
+	    Playlist *playlist=NULL;
 	    
 	    gtk_tree_model_get (model, iter, PM_COLUMN_PLAYLIST, &playlist, -1);
 	    if(playlist == (Playlist *)data) {
@@ -1160,6 +1161,13 @@ void pm_remove_playlist (Playlist *playlist, gboolean select)
 	    }
 	    return FALSE;
 	}
+
+/* Remove "playlist" from the display model. 
+   "select": TRUE: a new playlist is selected
+             FALSE: no selection is taking place
+                    (useful when quitting program) */
+void pm_remove_playlist (Playlist *playlist, gboolean select)
+{
     GtkTreeModel *model;
     gboolean have_iter = FALSE;
     GtkTreeIter i;
@@ -1168,11 +1176,21 @@ void pm_remove_playlist (Playlist *playlist, gboolean select)
   g_return_if_fail (playlist);
   model = gtk_tree_view_get_model (playlist_treeview);
   g_return_if_fail (model);
-  
+
   ts = gtk_tree_view_get_selection (playlist_treeview);
+
+  if (itdb_playlist_is_mpl (playlist) && current_playlist &&
+      (playlist->itdb == current_playlist->itdb))
+  {   /* We are about to remove the entire itdb (playlist is MPL) and
+       * a playlist of this itdb is selected --> clear display
+       * (pm_unselect_playlist probably works as well, but the
+       * unselect won't be done until later (callback)) */
+      st_init (-1, 0);
+      current_playlist = NULL;
+  }
+
   if (select && (current_playlist == playlist))
-  {
-      /* We are about to delete the currently selected
+  {   /* We are about to delete the currently selected
 	 playlist. Try to select the next. */
       if (gtk_tree_selection_get_selected (ts, NULL, &i))
       {
@@ -1192,6 +1210,7 @@ void pm_remove_playlist (Playlist *playlist, gboolean select)
 	  gtk_tree_path_free (path);
       }
   }
+
   /* find the pl and delete it */
   gtk_tree_model_foreach (model, pm_delete_playlist_fe, playlist);
   /* select our new iter !!! */
@@ -1230,16 +1249,13 @@ void pm_remove_all_playlists (gboolean clear_sort)
 }
 
 
-
-/* Select specified playlist */
-void pm_select_playlist (Playlist *playlist)
-{
-    gboolean pm_select_playlist_fe (GtkTreeModel *model,
-				    GtkTreePath *path,
-				    GtkTreeIter *iter,
-				    gpointer data)
+    /* former inline function: used by pm_select_playlist */
+    static gboolean pm_select_playlist_fe (GtkTreeModel *model,
+					   GtkTreePath *path,
+					   GtkTreeIter *iter,
+					   gpointer data)
 	{
-	    Playlist *playlist;
+	    Playlist *playlist=NULL;
 
 	    gtk_tree_model_get (model, iter,
 				PM_COLUMN_PLAYLIST, &playlist, -1);
@@ -1252,6 +1268,10 @@ void pm_select_playlist (Playlist *playlist)
 	    }
 	    return FALSE;
 	}
+
+/* Select specified playlist */
+void pm_select_playlist (Playlist *playlist)
+{
     GtkTreeModel *model;
 
     g_return_if_fail (playlist_treeview);
@@ -1265,15 +1285,13 @@ void pm_select_playlist (Playlist *playlist)
 }
 
 
-/* Unselect specified playlist */
-void pm_unselect_playlist (Playlist *playlist)
-{
-    gboolean pm_unselect_playlist_fe (GtkTreeModel *model,
-				      GtkTreePath *path,
-				      GtkTreeIter *iter,
-				      gpointer data)
+    /* former inline function: used by pm_unselect_playlist */
+    static gboolean pm_unselect_playlist_fe (GtkTreeModel *model,
+					     GtkTreePath *path,
+					     GtkTreeIter *iter,
+					     gpointer data)
 	{
-	    Playlist *playlist;
+	    Playlist *playlist=NULL;
 
 	    gtk_tree_model_get (model, iter,
 				PM_COLUMN_PLAYLIST, &playlist, -1);
@@ -1286,6 +1304,10 @@ void pm_unselect_playlist (Playlist *playlist)
 	    }
 	    return FALSE;
 	}
+
+/* Unselect specified playlist */
+void pm_unselect_playlist (Playlist *playlist)
+{
     GtkTreeModel *model;
 
     g_return_if_fail (playlist_treeview);
@@ -1323,6 +1345,17 @@ static void pm_selection_changed_cb (gpointer user_data1, gpointer user_data2)
       gtk_tree_model_get (model, &iter, 
 			  PM_COLUMN_PLAYLIST, &new_playlist,
 			  -1);
+
+      /* If new playlist is in an iPod itdb, set the mountpoint for
+       * the free space display to this iPod (there may be several
+       * iPods connected */
+      g_return_if_fail (new_playlist);
+      g_return_if_fail (new_playlist->itdb);
+      if (new_playlist->itdb->usertype & GP_ITDB_TYPE_IPOD)
+      {
+	  space_set_ipod_itdb (new_playlist->itdb);
+      }
+
       /* remove all entries from sort tab 0 */
       /* printf ("removing entries: %x\n", current_playlist);*/
       st_init (-1, 0);
@@ -1338,7 +1371,7 @@ static void pm_selection_changed_cb (gpointer user_data1, gpointer user_data2)
 	  float ms;
 	  GList *gl;
 
-	  if (!prefs_get_block_display ())
+	  if (!prefs_get_int("block_display"))
 	  {
 	      block_selection (-1);
 	      g_get_current_time (&time);
@@ -1350,7 +1383,7 @@ static void pm_selection_changed_cb (gpointer user_data1, gpointer user_data2)
 	      if (stop_add == -1)  break;
 	      st_add_track (track, FALSE, TRUE, 0);
 	      --count;
-	      if ((count < 0) && !prefs_get_block_display ())
+	      if ((count < 0) && !prefs_get_int("block_display"))
 	      {
 		  gtkpod_tracks_statusbar_update();
 		  while (gtk_events_pending ())       gtk_main_iteration ();
@@ -1368,7 +1401,7 @@ static void pm_selection_changed_cb (gpointer user_data1, gpointer user_data2)
 	  }
 	  st_enable_disable_view_sort (0, TRUE);
 	  if (stop_add != -1) st_add_track (NULL, TRUE, TRUE, 0);
-	  if (!prefs_get_block_display ())
+	  if (!prefs_get_int("block_display"))
 	  {
 	      while (gtk_events_pending ())	      gtk_main_iteration ();
 	      release_selection (-1);
@@ -1542,7 +1575,7 @@ static void pm_unsort ()
 void pm_sort (GtkSortType order)
 {
     GtkTreeModel *model= gtk_tree_view_get_model (playlist_treeview);
-    g_return_if_fail (playlist_treeview);
+    g_return_if_fail (model);
     if (order != SORT_NONE)
     {
 	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (model),
@@ -1558,6 +1591,9 @@ void pm_sort (GtkSortType order)
     }
 }
 
+
+#if 0
+FIXME: see comments at pm_data_compare_func()
 /**
  * pm_track_column_button_clicked
  * @tvc - the tree view colum that was clicked
@@ -1572,16 +1608,16 @@ pm_track_column_button_clicked(GtkTreeViewColumn *tvc, gpointer data)
     gint cnt = pm_sort_counter (1);
     if (cnt >= 3)
     {
-	prefs_set_pm_sort (SORT_NONE);
+	prefs_set_int("pm_sort", SORT_NONE);
 	pm_unsort (); /* also resets the sort_counter */
     }
     else
     {
-	prefs_set_pm_sort (gtk_tree_view_column_get_sort_order (tvc));
-	if (prefs_get_pm_autostore ())  pm_rows_reordered ();
+	prefs_set_int("pm_sort", gtk_tree_view_column_get_sort_order (tvc));
+	pm_rows_reordered ();
     }
 }
-
+#endif
 
 
 /**
@@ -1644,20 +1680,30 @@ gint pm_data_compare_func (GtkTreeModel *model,
 			GtkTreeIter *b,
 			gpointer user_data)
 {
-  Playlist *playlist1;
-  Playlist *playlist2;
+  Playlist *playlist1=NULL;
+  Playlist *playlist2=NULL;
   GtkSortType order;
   gint corr, colid;
 
-/*  g_return_val_if_fail (model, 0);
+  return 0;  /* FIXME: see below -- deactivated for now */
+
+  g_return_val_if_fail (model, 0);
   g_return_val_if_fail (a, 0);
-  g_return_val_if_fail (b, 0);*/
+  g_return_val_if_fail (b, 0);
 
   if (gtk_tree_sortable_get_sort_column_id (GTK_TREE_SORTABLE (model),
 					    &colid, &order) == FALSE)
       return 0;
+
   gtk_tree_model_get (model, a, colid, &playlist1, -1);
   gtk_tree_model_get (model, b, colid, &playlist2, -1);
+
+/* FIXME: this function crashes because it is provided illegal
+ * GtkTreeIters. */
+/* FIXME: after the introduction of a GtkTreeView rather than a
+ * ListView, sorting should be done by a customs sort mechanism
+ * anyway. */
+
   g_return_val_if_fail (playlist1 && playlist2, 0);
 
   /* We make sure that the master playlist always stays on top */
@@ -1679,41 +1725,162 @@ pm_cell_edited (GtkCellRendererText *renderer,
 		const gchar         *new_text,
 		gpointer             data)
 {
-  GtkTreeModel *model;
+  GtkTreeModel *model = data;
   GtkTreeIter iter;
-  Playlist *playlist;
-  gint column;
+  Playlist *playlist = NULL;
 
-  model = (GtkTreeModel *)data;
   g_return_if_fail (model);
+  g_return_if_fail (new_text);
   if (!gtk_tree_model_get_iter_from_string (model, &iter, path_string))
   {
       g_return_if_reached ();
   }
 
-  column = GPOINTER_TO_INT ( g_object_get_data (G_OBJECT (renderer), "column") );
-  gtk_tree_model_get (model, &iter, column, &playlist, -1);
+  gtk_tree_model_get (model, &iter, PM_COLUMN_PLAYLIST, &playlist, -1);
   g_return_if_fail (playlist);
 
-  /*printf("pm_cell_edited: column: %d  track:%lx\n", column, track);*/
+  /*printf("pm_cell_edited: column: %d  track:%lx\n", PM_COLUMN_PLAYLIST, track);*/
 
-  switch (column)
+  /* We only do something, if the name actually got changed */
+  if (!playlist->name ||
+      g_utf8_collate (playlist->name, new_text) != 0)
   {
-  case PM_COLUMN_PLAYLIST:
-      g_return_if_fail (new_text);
-      /* We only do something, if the name actually got changed */
-      if (!playlist->name ||
-	  g_utf8_collate (playlist->name, new_text) != 0)
-      {
-	  g_free (playlist->name);
-	  playlist->name = g_strdup (new_text);
-	  data_changed (playlist->itdb);
+      g_free (playlist->name);
+      playlist->name = g_strdup (new_text);
+      data_changed (playlist->itdb);
+      if (itdb_playlist_is_mpl (playlist))
+      {   /* Need to change name in prefs system */
+	  set_itdb_prefs_string (playlist->itdb, "name", new_text);
       }
-      break;
   }
 }
 
 
+/**
+ * pm_set_renderer_text
+ *
+ * Set the playlist name in appropriate style.
+ *
+ * @renderer: renderer to be set
+ * @playlist: playlist to consider.
+ */
+void pm_set_renderer_text (GtkCellRenderer *renderer,
+			   Playlist *playlist)
+{
+    if (playlist)
+    {
+	ExtraiTunesDBData *eitdb;
+
+	g_return_if_fail (playlist->itdb);
+	eitdb = playlist->itdb->userdata;
+	g_return_if_fail (eitdb);
+
+	if (itdb_playlist_is_mpl (playlist))
+	{   /* mark MPL */
+	    g_object_set (G_OBJECT (renderer),
+			  "text", playlist->name, 
+			  "weight", PANGO_WEIGHT_BOLD,
+			  NULL);
+	    if (eitdb->data_changed)
+	    {
+		g_object_set (G_OBJECT (renderer),
+			      "style", PANGO_STYLE_ITALIC,
+			      NULL);
+	    }
+	    else
+	    {
+		g_object_set (G_OBJECT (renderer),
+			      "style", PANGO_STYLE_NORMAL,
+			      NULL);
+	    }
+	}
+	else
+	{
+	    if (itdb_playlist_is_podcasts (playlist))
+	    {
+		g_object_set (G_OBJECT (renderer),
+			      "text", playlist->name, 
+			      "weight", PANGO_WEIGHT_SEMIBOLD,
+			      "style", PANGO_STYLE_ITALIC,
+			      NULL);
+	    }
+	    else
+	    {
+		g_object_set (G_OBJECT (renderer),
+			      "text", playlist->name, 
+			      "weight", PANGO_WEIGHT_NORMAL,
+			      "style", PANGO_STYLE_NORMAL,
+			      NULL);
+	    }
+	}
+    }
+    else
+    {
+	g_object_set (G_OBJECT (renderer),
+		      "text", "",
+		      NULL);
+    }
+}
+
+
+/**
+ * pm_set_renderer_pix
+ *
+ * Set the appropriate playlist icon.
+ *
+ * @renderer: renderer to be set
+ * @playlist: playlist to consider.
+ */
+void pm_set_renderer_pix (GtkCellRenderer *renderer,
+			  Playlist *playlist)
+{
+    const gchar *stock_id=NULL;
+
+    g_return_if_fail (renderer);
+
+    if (playlist)
+    {
+	iTunesDB *itdb;
+	ExtraiTunesDBData *eitdb;
+
+	g_return_if_fail (playlist->itdb);
+	itdb = playlist->itdb;
+	g_return_if_fail (itdb->userdata);
+	eitdb = itdb->userdata;
+
+	if (playlist->is_spl)
+	{
+	    stock_id = GTK_STOCK_PROPERTIES;
+	}
+	else if (!itdb_playlist_is_mpl (playlist))
+	{
+	    stock_id = GTK_STOCK_JUSTIFY_LEFT;
+	}
+	else
+	{
+	    if (itdb->usertype & GP_ITDB_TYPE_LOCAL)
+	    {
+		stock_id = GTK_STOCK_HARDDISK;
+	    }
+	    else
+	    {
+		if (eitdb->itdb_imported)
+		{
+		    stock_id = GTK_STOCK_CONNECT;
+		}
+		else
+		{
+		    stock_id = GTK_STOCK_DISCONNECT;
+		}
+	    }
+	}
+    }
+
+  g_object_set (G_OBJECT (renderer),
+		"stock-id", stock_id, NULL);
+}
+
+
 /* The playlist data is stored in a separate list
    and only pointers to the corresponding playlist structure are placed
    into the model.
@@ -1725,52 +1892,16 @@ static void pm_cell_data_func (GtkTreeViewColumn *tree_column,
 			       GtkTreeIter       *iter,
 			       gpointer           data)
 {
-  Playlist *playlist;
-  gint column;
+  Playlist *playlist = NULL;
 
   g_return_if_fail (renderer);
   g_return_if_fail (model);
   g_return_if_fail (iter);
 
-  column = GPOINTER_TO_INT ( g_object_get_data (G_OBJECT (renderer), "column") );
-  gtk_tree_model_get (model, iter, column, &playlist, -1);
+  gtk_tree_model_get (model, iter, PM_COLUMN_PLAYLIST, &playlist, -1);
   g_return_if_fail (playlist);
 
-  switch (column)
-    {  /* We only have one column, so this code is overkill... */
-    case PM_COLUMN_PLAYLIST: 
-	if (itdb_playlist_is_mpl (playlist))
-	{   /* mark MPL */
-	    g_object_set (G_OBJECT (renderer),
-			  "text", playlist->name, 
-			  "editable", TRUE,
-			  "weight", PANGO_WEIGHT_BOLD,
-			  "style", PANGO_STYLE_NORMAL,
-			  NULL);
-	}
-	else
-	{
-	    if (itdb_playlist_is_podcasts (playlist))
-	    {
-		g_object_set (G_OBJECT (renderer),
-			      "text", playlist->name, 
-			      "editable", TRUE,
-			      "weight", PANGO_WEIGHT_SEMIBOLD,
-			      "style", PANGO_STYLE_ITALIC,
-			      NULL);
-	    }
-	    else
-	    {
-		g_object_set (G_OBJECT (renderer),
-			      "text", playlist->name, 
-			      "editable", TRUE,
-			      "weight", PANGO_WEIGHT_NORMAL,
-			      "style", PANGO_STYLE_NORMAL,
-			      NULL);
-	    }
-	}
-	break;
-    }
+  pm_set_renderer_text (renderer, playlist);
 }
 
 
@@ -1781,79 +1912,16 @@ static void pm_cell_data_func_pix (GtkTreeViewColumn *tree_column,
 				   GtkTreeIter       *iter,
 				   gpointer           data)
 {
-  Playlist *playlist;
-  gint column;
+  Playlist *playlist=NULL;
 
   g_return_if_fail (renderer);
   g_return_if_fail (model);
   g_return_if_fail (iter);
 
-  column = GPOINTER_TO_INT ( g_object_get_data (G_OBJECT (renderer), "column") );
-  gtk_tree_model_get (model, iter, column, &playlist, -1);
+  gtk_tree_model_get (model, iter, PM_COLUMN_PLAYLIST, &playlist, -1);
   g_return_if_fail (playlist);
 
-  switch (column)
-    {  /* We only have one column, so this code is overkill... */
-    case PM_COLUMN_PLAYLIST: 
-	if (playlist->is_spl)
-	{
-	    g_object_set (G_OBJECT (renderer),
-			  "stock-id", GTK_STOCK_PROPERTIES, NULL);
-	}
-	else if (!itdb_playlist_is_mpl (playlist))
-	{
-	    g_object_set (G_OBJECT (renderer),
-			  "stock-id", GTK_STOCK_JUSTIFY_LEFT, NULL);
-	}
-	else
-	{
-	    g_object_set (G_OBJECT (renderer),
-			  "stock-id", NULL, NULL);
-	}
-	break;
-    }
-}
-
-
-/* Adds the columns to our playlist_treeview */
-static void pm_add_columns (void)
-{
-  GtkTreeViewColumn *column;
-  GtkCellRenderer *renderer;
-  GtkTreeModel *model;
-
-  model = gtk_tree_view_get_model (playlist_treeview);
-  g_return_if_fail (model);
-
-  /* playlist column */
-  column = gtk_tree_view_column_new ();
-  gtk_tree_view_column_set_title (column, _("Playlists"));
-  gtk_tree_view_column_set_sort_column_id (column, PM_COLUMN_PLAYLIST);
-  gtk_tree_view_column_set_resizable (column, TRUE);
-  gtk_tree_view_column_set_sort_order (column, GTK_SORT_ASCENDING);
-  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (model),
-				   PM_COLUMN_PLAYLIST,
-				   pm_data_compare_func, column, NULL);
-  gtk_tree_view_column_set_clickable(column, TRUE);
-  g_signal_connect (G_OBJECT (column), "clicked",
-		    G_CALLBACK (pm_track_column_button_clicked),
-				(gpointer)PM_COLUMN_PLAYLIST);
-  gtk_tree_view_append_column (playlist_treeview, column);
-
-  /* cell for playlist name */
-  renderer = gtk_cell_renderer_text_new ();
-  g_signal_connect (G_OBJECT (renderer), "edited",
-		    G_CALLBACK (pm_cell_edited), model);
-  g_object_set_data (G_OBJECT (renderer), "column", (gint *)PM_COLUMN_PLAYLIST);
-  gtk_tree_view_column_pack_end (column, renderer, FALSE);
-  gtk_tree_view_column_set_cell_data_func (column, renderer, pm_cell_data_func, NULL, NULL);
-
-
-  /* cell for graphic indicator */
-  renderer = gtk_cell_renderer_pixbuf_new ();
-  g_object_set_data (G_OBJECT (renderer), "column", (gint *)PM_COLUMN_PLAYLIST);
-  gtk_tree_view_column_pack_start (column, renderer, FALSE); 
-  gtk_tree_view_column_set_cell_data_func (column, renderer, pm_cell_data_func_pix, NULL, NULL);
+  pm_set_renderer_pix (renderer, playlist);
 }
 
 
@@ -1874,21 +1942,203 @@ static void pm_select_current_position (gint x, gint y)
     }
 }
 
+
+
+/* Return the number (0...) of the renderer the click was in or -1 if
+   no renderer was found. @cell (if != NULL) is filled with a pointer
+   to the renderer. */
+gint tree_view_get_cell_from_pos(GtkTreeView *view, guint x, guint y,
+				 GtkCellRenderer **cell)
+{
+    GtkTreeViewColumn *col = NULL;
+    GList             *node, *cells;
+    gint               pos = 0;
+    GdkRectangle       rect;
+    GtkTreePath        *path = NULL;
+    gint               cell_x, cell_y;
+
+    g_return_val_if_fail ( view != NULL, -1 );
+
+    if (cell)
+	*cell = NULL;
+
+    gtk_tree_view_get_path_at_pos (view, x, y, &path, &col,
+				   &cell_x, &cell_y);
+
+    if (col == NULL)
+	return -1; /* not found */
+
+    cells = gtk_tree_view_column_get_cell_renderers(col);
+
+    gtk_tree_view_get_cell_area (view, path, col, &rect);
+    gtk_tree_path_free (path);
+
+    /* gtk_tree_view_get_cell_area() should return the rectangle
+       _excluding_ the expander arrow(s), but until 2.8.17 it forgets
+       about the space occupied by the top level expander arrow. We
+       therefore need to add the width of one expander arrow */
+    if (!RUNTIME_GTK_CHECK_VERSION(2,8,18))
+    {
+	if (col ==  gtk_tree_view_get_expander_column (view))
+	{
+	    GValue *es = g_malloc0 (sizeof (GValue)); 
+	    g_value_init (es, G_TYPE_INT);
+	    gtk_widget_style_get_property (GTK_WIDGET (view),
+					   "expander_size",
+					   es);
+	    rect.x += g_value_get_int (es);
+	    rect.width -= g_value_get_int (es);
+	    g_free (es);
+	}
+    }
+
+    for (node = cells;  node != NULL;  node = node->next)
+    {
+	GtkCellRenderer *checkcell = (GtkCellRenderer*)node->data;
+	gint start_pos, width;
+
+	if (gtk_tree_view_column_cell_get_position (col, checkcell,
+						    &start_pos, &width))
+	{
+	    if (x >= (rect.x + start_pos) &&
+		x < (rect.x + start_pos + width))
+	    {
+		if (cell)
+		    *cell = checkcell;
+		g_list_free(cells);
+		return pos;
+	    }
+	}
+	++pos;
+    }
+
+    g_list_free(cells);
+    return -1; /* not found */
+}
+
+
+
 static gboolean
 pm_button_press (GtkWidget *w, GdkEventButton *e, gpointer data)
 {
+    gint cell_nr;
+    GtkTreeModel *model;
+    GtkTreePath *path;
+    GtkTreeIter iter;
+    Playlist *pl;
+    ExtraiTunesDBData *eitdb;
+
     g_return_val_if_fail (w && e, FALSE);
     switch(e->button)
     {
+    case 1:
+	cell_nr = tree_view_get_cell_from_pos (GTK_TREE_VIEW(w),
+					       e->x, e->y, NULL);
+	if (cell_nr == 0)
+	{
+	    /* */
+	    model= gtk_tree_view_get_model (GTK_TREE_VIEW (w));
+	    gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW(w),
+					   e->x, e->y,
+					   &path, NULL,
+					   NULL, NULL);
+	    gtk_tree_model_get_iter (model, &iter, path);
+	    gtk_tree_path_free (path);
+	    gtk_tree_model_get (model, &iter,
+				PM_COLUMN_PLAYLIST, &pl,
+				-1);
+	    if (pl == NULL)
+		break;
+
+	    g_return_val_if_fail (pl->itdb, FALSE);
+
+	    if (!itdb_playlist_is_mpl (pl))
+		break;
+
+	    if (pl->itdb->usertype & GP_ITDB_TYPE_IPOD)
+	    {
+
+		/* the user clicked on the connect/disconnect icon of
+		 * an iPod */
+		eitdb = pl->itdb->userdata;
+		g_return_val_if_fail (eitdb, FALSE);
+		if (!eitdb->itdb_imported)
+		{
+		    gp_load_ipod (pl->itdb);
+		}
+		else
+		{
+		    gp_eject_ipod (pl->itdb);
+		}
+		return TRUE;
+	    }
+	    if (pl->itdb->usertype & GP_ITDB_TYPE_LOCAL)
+	    {
+
+		/* the user clicked on the 'harddisk' icon of
+		 * a local repository */
+	    }
+	}
+	break;
     case 3:
 	pm_select_current_position (e->x, e->y);
 	pm_context_menu_init ();
 	return TRUE;
     default:
-	return FALSE;
+	break;
     }
+    return FALSE;
 }
 
+/* Adds the columns to our playlist_treeview */
+static void pm_add_columns (void)
+{
+  GtkTreeViewColumn *column;
+  GtkCellRenderer *renderer;
+  GtkTreeModel *model;
+
+  model = gtk_tree_view_get_model (playlist_treeview);
+  g_return_if_fail (model);
+
+
+  /* playlist column */
+  column = gtk_tree_view_column_new ();
+  gtk_tree_view_column_set_title (column, _("Playlists"));
+/* FIXME: see comments at pm_data_compare_func() */
+/*
+  gtk_tree_view_column_set_sort_column_id (column, PM_COLUMN_PLAYLIST);
+  gtk_tree_view_column_set_sort_order (column, GTK_SORT_ASCENDING);
+  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (model),
+				   PM_COLUMN_PLAYLIST,
+				   pm_data_compare_func, column, NULL);
+  gtk_tree_view_column_set_clickable(column, TRUE);
+  g_signal_connect (G_OBJECT (column), "clicked",
+		    G_CALLBACK (pm_track_column_button_clicked),
+				(gpointer)PM_COLUMN_PLAYLIST);
+*/
+  gtk_tree_view_append_column (playlist_treeview, column);
+
+  /* cell for graphic indicator */
+  renderer = gtk_cell_renderer_pixbuf_new ();
+
+  gtk_tree_view_column_pack_start (column, renderer, FALSE); 
+  gtk_tree_view_column_set_cell_data_func (column, renderer,
+					   pm_cell_data_func_pix,
+					   NULL, NULL);
+  /* cell for playlist name */
+  renderer = gtk_cell_renderer_text_new ();
+  g_signal_connect (G_OBJECT (renderer), "edited",
+		    G_CALLBACK (pm_cell_edited), model);
+  gtk_tree_view_column_pack_start (column, renderer, FALSE);
+  gtk_tree_view_column_set_cell_data_func (column, renderer,
+					   pm_cell_data_func,
+					   NULL, NULL);
+  g_object_set (G_OBJECT (renderer),
+		"editable", TRUE,
+		NULL);
+}
+
+
 /* Create playlist listview */
 void pm_create_treeview (void)
 {
@@ -2024,5 +2274,3 @@ pm_set_selected_playlist (Playlist *pl)
 {
     current_playlist = pl;
 }
-
-
