@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-12-03 20:53:34 jcs>
+/* Time-stamp: <2006-06-23 23:52:48 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,19 +24,22 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: display_songs.c,v 1.94 2005/12/03 16:36:26 jcsjcs Exp $
+|  $Id: display_songs.c,v 1.112 2006/09/21 15:03:13 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
-#include <gtk/gtk.h>
-#include <string.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <limits.h>
 #include <gdk/gdkkeysyms.h>
+#include <gtk/gtk.h>
+#include <limits.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <unistd.h>
 
 #include "prefs.h"
 #include "display_private.h"
@@ -200,7 +203,7 @@ on_tm_dnd_get_file_foreach(GtkTreeModel *tm, GtkTreePath *tp,
     gchar *name;
 
     gtk_tree_model_get(tm, iter, READOUT_COL, &track, -1);
-    name = get_file_name_verified (track);
+    name = get_file_name_from_source (track, SOURCE_PREFER_LOCAL);
     if (name)
     {
 	g_string_append_printf (filelist, "file:%s\n", name);
@@ -220,7 +223,7 @@ on_tm_dnd_get_uri_foreach(GtkTreeModel *tm, GtkTreePath *tp,
     gchar *name;
 
     gtk_tree_model_get(tm, iter, READOUT_COL, &track, -1);
-    name = get_file_name_verified (track);
+    name = get_file_name_from_source (track, SOURCE_PREFER_LOCAL);
     if (name)
     {
 	gchar *uri = g_filename_to_uri (name, NULL, NULL);
@@ -248,7 +251,6 @@ static void tm_drag_data_delete (GtkWidget *widget,
 {
     GtkTreeSelection *ts;
     Playlist *pl = pm_get_selected_playlist ();
-    gchar *buf;
     gint num;
 
 /*     puts ("tm_drag_data_delete"); */
@@ -277,19 +279,13 @@ static void tm_drag_data_delete (GtkWidget *widget,
 	}
 	g_string_free (reply, TRUE);
 
-	buf = g_strdup_printf (ngettext ("Moved one track",
-					 "Moved %d tracks", num), num);
+	gtkpod_statusbar_message (ngettext ("Moved one track",
+					    "Moved %d tracks", num), num);
     }
     else
     {
-	buf = g_strdup_printf (ngettext ("Copied one track",
-					 "Copied %d tracks", num), num);
-    }
-
-    if (buf)
-    {
-	gtkpod_statusbar_message (buf);
-	g_free (buf);
+	gtkpod_statusbar_message (ngettext ("Copied one track",
+					    "Copied %d tracks", num), num);
     }
 }
 
@@ -708,7 +704,7 @@ void tm_store_col_order (void)
     for (i=0; i<TM_NUM_COLUMNS; ++i)
     {
 	col = gtk_tree_view_get_column (track_treeview, i);
-	prefs_set_col_order (i, col->sort_column_id);
+	prefs_set_int_index("col_order", i, col->sort_column_id);
     }
 }
 
@@ -795,9 +791,9 @@ tm_cell_edited (GtkCellRendererText *renderer,
 
 
   column = (TM_item) g_object_get_data(G_OBJECT(renderer), "column");
-  multi_edit = prefs_get_multi_edit ();
+  multi_edit = prefs_get_int("multi_edit");
   if (column == TM_COLUMN_TITLE)
-      multi_edit &= prefs_get_multi_edit_title ();
+      multi_edit &= prefs_get_int("multi_edit_title");
   selection = gtk_tree_view_get_selection(track_treeview);
   row_list = gtk_tree_selection_get_selected_rows(selection, &model);
 
@@ -860,7 +856,20 @@ tm_cell_edited (GtkCellRendererText *renderer,
      case TM_COLUMN_SAMPLERATE:
      case TM_COLUMN_BPM:
 	 changed = track_set_text (track, new_text, TM_to_T (column));
-	 
+	 if (changed && (column == TM_COLUMN_TRACKLEN))
+	 {  /* be on the safe side and reset starttime, stoptime and
+	     * filesize */
+	     gchar *path = get_file_name_from_source (track,
+						      SOURCE_PREFER_LOCAL);
+	     track->starttime = 0;
+	     track->stoptime = 0;
+	     if (path)
+	     {
+		 struct stat filestat;
+		 stat (path, &filestat);
+		 track->size = filestat.st_size;
+	     }
+	 }
 	 /* redisplay some items to be on the safe side */
 	 switch (column)
 	 {
@@ -890,7 +899,7 @@ tm_cell_edited (GtkCellRendererText *renderer,
         pm_track_changed (track);    /* notify playlist model... */
         data_changed (track->itdb); /* indicate that data has changed */
 
-        if (prefs_get_id3_write())
+        if (prefs_get_int("id3_write"))
         {
 	    /* T_item tag_id;*/
            /* should we update all ID3 tags or just the one
@@ -991,6 +1000,7 @@ static void tm_cell_data_func (GtkTreeViewColumn *tree_column,
       break;
   case TM_COLUMN_PC_PATH:
   case TM_COLUMN_IPOD_PATH:
+  case TM_COLUMN_THUMB_PATH:
       g_object_set (G_OBJECT (renderer),
 		    "text", text,
 		    "editable", FALSE,
@@ -1030,7 +1040,7 @@ tm_cell_toggled (GtkCellRendererToggle *renderer,
   gboolean active;
 
   column = (TM_item) g_object_get_data(G_OBJECT(renderer), "column");
-  multi_edit = prefs_get_multi_edit ();
+  multi_edit = prefs_get_int("multi_edit");
   selection = gtk_tree_view_get_selection(track_treeview);
   row_list = gtk_tree_selection_get_selected_rows(selection, &model);
 
@@ -1089,7 +1099,7 @@ tm_cell_toggled (GtkCellRendererToggle *renderer,
         /* If the changed column is the compilation flag update the file
            if required */
         if (column == TM_COLUMN_COMPILATION)
-           if (prefs_get_id3_write())
+           if (prefs_get_int("id3_write"))
               write_tags_to_file (track);
         
      }
@@ -1345,19 +1355,23 @@ tm_get_selected_tracks(void)
 }
 
 
+
+    /* used by tm_get_all_tracks */
+    static gboolean on_all_tracks_list_foreach (GtkTreeModel *tm,
+						GtkTreePath *tp,
+						GtkTreeIter *i,
+						gpointer data)
+    {
+	on_tracks_list_foreach (tm, tp, i, data);
+	return FALSE;
+    }
+
+
 /* return a list containing pointers to all tracks currently being
    displayed. You must g_list_free() the list after use. */
 GList *
 tm_get_all_tracks(void)
 {
-    gboolean on_all_tracks_list_foreach (GtkTreeModel *tm,
-					 GtkTreePath *tp,
-					 GtkTreeIter *i,
-					 gpointer data)
-	{
-	    on_tracks_list_foreach (tm, tp, i, data);
-	    return FALSE;
-	}
     GList *result = NULL;
     GtkTreeModel *model = gtk_tree_view_get_model (track_treeview);
 
@@ -1384,13 +1398,13 @@ void tm_stop_editing (gboolean cancel)
 	/* Before removing the widget we set multi_edit to FALSE. That
 	   way at most one entry will be changed (this also doesn't
 	   seem to work the way intended) */
-	gboolean me = prefs_get_multi_edit ();
-	prefs_set_multi_edit (FALSE);
+	gboolean me = prefs_get_int("multi_edit");
+	prefs_set_int("multi_edit", FALSE);
 	if (!cancel && col->editable_widget)
 	    gtk_cell_editable_editing_done (col->editable_widget);
 	if (col->editable_widget)
 	    gtk_cell_editable_remove_widget (col->editable_widget);
-	prefs_set_multi_edit (me);
+	prefs_set_int("multi_edit", me);
     }
 }
 
@@ -1449,6 +1463,12 @@ static gint tm_data_compare (Track *track1, Track *track2,
   case TM_COLUMN_IPOD_PATH:
       cmp = g_utf8_collate (track1->ipod_path, track2->ipod_path);
       break;
+  case TM_COLUMN_THUMB_PATH:
+      etr1 = track1->userdata;
+      etr2 = track2->userdata;
+      g_return_val_if_fail (etr1 && etr2, 0);
+      cmp = g_utf8_collate (etr1->thumb_path_utf8, etr2->thumb_path_utf8);
+      break;
   case TM_COLUMN_TRANSFERRED:
       if(track1->transferred == track2->transferred)
 	  cmp = 0;
@@ -1551,6 +1571,7 @@ gboolean tm_search_equal_func (GtkTreeModel *model,
   case TM_COLUMN_YEAR:
   case TM_COLUMN_IPOD_PATH:
   case TM_COLUMN_COMPILATION:
+  case TM_COLUMN_THUMB_PATH:
     cmp = (compare_string_start_case_insensitive (
 	  track_get_item (track1, TM_to_T (column)),
 	  key) != 0);
@@ -1629,7 +1650,7 @@ gint tm_sort_counter (gint inc)
 
 void tm_adopt_order_in_sorttab (void)
 {
-    if (prefs_get_tm_sort () == SORT_NONE)
+    if (prefs_get_int("tm_sort_") == SORT_NONE)
     {
 	GList *gl, *tracks = NULL;
 
@@ -1637,7 +1658,7 @@ void tm_adopt_order_in_sorttab (void)
 	   the last sort tab or from the selected playlist if no sort
 	   tabs are being used */
 	tm_remove_all_tracks ();
-	tracks = display_get_selected_members (prefs_get_sort_tab_num()-1);
+	tracks = display_get_selected_members (prefs_get_int("sort_tab_num")-1);
 	for (gl=tracks; gl; gl=gl->next)
 	    tm_add_track_to_track_model ((Track *)gl->data, NULL);
     }
@@ -1651,7 +1672,7 @@ static void tm_unsort (void)
     {
 	GtkTreeModel *model= gtk_tree_view_get_model (track_treeview);
 
-	prefs_set_tm_sort (SORT_NONE);
+	prefs_set_int("tm_sort_", SORT_NONE);
 	if (!BROKEN_GTK_TREE_SORT)
 	{
 /* no need to comment this out -- searching still works, but for lack
@@ -1699,6 +1720,7 @@ static void tm_set_search_column (gint newcol)
     case TM_COLUMN_YEAR:
     case TM_COLUMN_IPOD_PATH:
     case TM_COLUMN_COMPILATION:
+    case TM_COLUMN_THUMB_PATH:
 	gtk_tree_view_set_enable_search (GTK_TREE_VIEW (track_treeview), TRUE);
 	break;
     case TM_COLUMN_TRACK_NR:
@@ -1722,7 +1744,7 @@ static void tm_set_search_column (gint newcol)
 	gtk_tree_view_set_enable_search (GTK_TREE_VIEW (track_treeview), FALSE);
 	break;
     }
-    prefs_set_int_value (TM_PREFS_SEARCH_COLUMN, newcol);
+    prefs_set_int (TM_PREFS_SEARCH_COLUMN, newcol);
 }
 
 
@@ -1769,13 +1791,13 @@ static void tm_sort_column_changed (GtkTreeSortable *ts,
     }
     else
     {
-	prefs_set_tm_sort (order);
+	prefs_set_int("tm_sort_", order);
     }
-    prefs_set_tm_sortcol (newcol);
+    prefs_set_int("tm_sortcol", newcol);
 
     tm_set_search_column (newcol);
 
-    if(prefs_get_tm_autostore ())  tm_rows_reordered ();
+    if(prefs_get_int("tm_autostore"))  tm_rows_reordered ();
     sort_window_update ();
 
     /* stable sorting: index original order */
@@ -1835,42 +1857,6 @@ void tm_sort (TM_item col, GtkSortType order)
 }
 
 
-/* Add one column at position @pos. This code is used over and over
-   by tm_add_column() -- therefore I put it into a separate function */
-static GtkTreeViewColumn *tm_add_text_column (TM_item col_id,
-					      const gchar *name,
-					      GtkCellRenderer *renderer,
-					      gint pos)
-{
-    GtkTreeViewColumn *column;
-    GtkTreeModel *model = gtk_tree_view_get_model (track_treeview);
-
-    if (!renderer)
-    {   /* text renderer -- editable/not editable is done in
-	   tm_cell_data_func() */
-	renderer = gtk_cell_renderer_text_new ();
-	g_signal_connect (G_OBJECT (renderer), "edited",
-			  G_CALLBACK (tm_cell_edited), model);
-    }
-    g_object_set_data (G_OBJECT (renderer), "column", (gint *)col_id);
-    column = gtk_tree_view_column_new ();
-    gtk_tree_view_column_set_title (column, name);
-    gtk_tree_view_column_pack_end (column, renderer, FALSE);
-    gtk_tree_view_column_set_cell_data_func (column, renderer,
-					     tm_cell_data_func, NULL, NULL);
-    gtk_tree_view_column_set_sort_column_id (column, col_id);
-    gtk_tree_view_column_set_resizable (column, TRUE);
-/*     gtk_tree_view_column_set_clickable(column, TRUE); */
-    gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
-    gtk_tree_view_column_set_fixed_width (column,
-					  prefs_get_tm_col_width (col_id));
-    gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (model), col_id,
-				     tm_data_compare_func, NULL, NULL);
-    gtk_tree_view_column_set_reorderable (column, TRUE);
-    gtk_tree_view_insert_column (track_treeview, column, pos);
-    tm_columns[col_id] = column;
-    return column;
-}
 
 
 
@@ -1879,7 +1865,7 @@ static GtkTreeViewColumn *tm_add_column (TM_item tm_item, gint pos)
 {
   GtkTreeModel *model = gtk_tree_view_get_model (track_treeview);
   GtkTreeViewColumn *col = NULL;
-  const gchar *text = NULL;
+  const gchar *text;
   GtkCellRenderer *renderer = NULL;  /* default */
   GtkTooltips *tt;
 
@@ -1894,9 +1880,23 @@ static GtkTreeViewColumn *tm_add_column (TM_item tm_item, gint pos)
 
   g_return_val_if_fail (text, NULL);
 
+  col = gtk_tree_view_column_new ();
+
   switch (tm_item)
   {
   case TM_COLUMN_TITLE:
+      /* Add additional toggle box for 'checked' property */
+      renderer = gtk_cell_renderer_toggle_new ();
+      g_object_set_data (G_OBJECT (renderer), "column",
+			 (gint *)tm_item);
+      g_signal_connect (G_OBJECT (renderer), "toggled",
+			G_CALLBACK (tm_cell_toggled), model);
+      gtk_tree_view_column_pack_start (col, renderer, FALSE);
+      gtk_tree_view_column_set_cell_data_func (col, renderer,
+					       tm_cell_data_func_toggle,
+					       NULL, NULL);
+      renderer = NULL;
+      break;
   case TM_COLUMN_ARTIST:
   case TM_COLUMN_ALBUM:
   case TM_COLUMN_GENRE:
@@ -1914,6 +1914,7 @@ static GtkTreeViewColumn *tm_add_column (TM_item tm_item, gint pos)
   case TM_COLUMN_SUBTITLE:
   case TM_COLUMN_PC_PATH:
   case TM_COLUMN_IPOD_PATH:
+  case TM_COLUMN_THUMB_PATH:
   case TM_COLUMN_SIZE:
       break;
   /* for some column names we want to use shorter alternatives to
@@ -1937,6 +1938,8 @@ static GtkTreeViewColumn *tm_add_column (TM_item tm_item, gint pos)
   case TM_COLUMN_COMPILATION:
       text = _("Cmpl");
       renderer = gtk_cell_renderer_toggle_new ();
+      g_signal_connect (G_OBJECT (renderer), "toggled",
+			G_CALLBACK (tm_cell_toggled), model);
       break;
   case TM_COLUMN_TRACKLEN:
       text = _("Time");
@@ -1969,26 +1972,38 @@ static GtkTreeViewColumn *tm_add_column (TM_item tm_item, gint pos)
       g_return_val_if_reached (NULL);
       break;
   }
-  col = tm_add_text_column (tm_item, text, renderer, pos);
-  if (col && (tm_item == TM_COLUMN_TITLE))
-  {
-      renderer = gtk_cell_renderer_toggle_new ();
-      g_object_set_data (G_OBJECT (renderer), "column",
-			 (gint *)tm_item);
-      g_signal_connect (G_OBJECT (renderer), "toggled",
-			G_CALLBACK (tm_cell_toggled), model);
-      gtk_tree_view_column_pack_start (col, renderer, FALSE);
-      gtk_tree_view_column_set_cell_data_func (col, renderer, tm_cell_data_func_toggle, NULL, NULL);
+
+  if (!renderer)
+  {   /* text renderer -- editable/not editable is done in
+	 tm_cell_data_func() */
+      renderer = gtk_cell_renderer_text_new ();
+      g_signal_connect (G_OBJECT (renderer), "edited",
+			G_CALLBACK (tm_cell_edited), model);
   }
-  if (col && (tm_item == TM_COLUMN_COMPILATION))
-  {
-      g_signal_connect (G_OBJECT (renderer), "toggled",
-			G_CALLBACK (tm_cell_toggled), model);
-  }      
-  if (col && (pos != -1))
+
+  g_object_set_data (G_OBJECT (renderer), "column",
+		     (gint *)tm_item);
+
+  gtk_tree_view_column_set_title (col, text);
+  gtk_tree_view_column_pack_start (col, renderer, FALSE);
+  gtk_tree_view_column_set_cell_data_func (col, renderer,
+					   tm_cell_data_func, NULL, NULL);
+  gtk_tree_view_column_set_sort_column_id (col, tm_item);
+  gtk_tree_view_column_set_resizable (col, TRUE);
+/*     gtk_tree_view_column_set_clickable(column, TRUE); */
+  gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
+  gtk_tree_view_column_set_fixed_width (col,
+					prefs_get_int_index("tm_col_width", tm_item));
+  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (model), tm_item,
+				   tm_data_compare_func, NULL, NULL);
+  gtk_tree_view_column_set_reorderable (col, TRUE);
+  gtk_tree_view_insert_column (track_treeview, col, pos);
+  tm_columns[tm_item] = col;
+
+  if (pos != -1)
   {
       gtk_tree_view_column_set_visible (col,
-					prefs_get_col_visible (tm_item));
+					prefs_get_int_index("col_visible", tm_item));
   }
   if (get_tm_tooltip (tm_item))
       gtk_tooltips_set_tip (tt, col->button, 
@@ -2005,7 +2020,7 @@ static void tm_add_columns (void)
 
     for (i=0; i<TM_NUM_COLUMNS; ++i)
     {
-	tm_add_column (prefs_get_col_order (i), -1);
+	tm_add_column (prefs_get_int_index("col_order", i), -1);
     }
     tm_show_preferred_columns();
 }
@@ -2034,14 +2049,21 @@ tm_button_press_event(GtkWidget *w, GdkEventButton *e, gpointer data)
     {
 	switch(e->button)
 	{
-	    case 3:
-		tm_select_current_position (e->x, e->y);
-		tm_context_menu_init ();
-		return TRUE;
-	    default:
-		break;
+	case 1:
+/*
+	    printf ("Pressed in cell %d (%f/%f)\n\n",
+		    tree_view_get_cell_from_pos(GTK_TREE_VIEW(w),
+						(guint)e->x, (guint)e->y, NULL),
+		    e->x, e->y);
+*/
+	    break;
+	case 3:
+	    tm_select_current_position (e->x, e->y);
+	    tm_context_menu_init ();
+	    return TRUE;
+	default:
+	    break;
 	}
-
     }
     return(FALSE);
 }
@@ -2169,7 +2191,7 @@ void tm_create_treeview (void)
 		    (gpointer)0);
 
   /* initialize sorting */
-  tm_sort (prefs_get_tm_sortcol (), prefs_get_tm_sort ());
+  tm_sort (prefs_get_int("tm_sortcol"), prefs_get_int("tm_sort_"));
   /* set correct column for typeahead */
   if (prefs_get_int_value (TM_PREFS_SEARCH_COLUMN, &col))
   {
@@ -2192,7 +2214,8 @@ tm_show_preferred_columns(void)
     for (i=0; i<TM_NUM_COLUMNS; ++i)
     {
 	tvc = gtk_tree_view_get_column (track_treeview, i);
-	visible = prefs_get_col_visible (prefs_get_col_order (i));
+	visible = prefs_get_int_index("col_visible", 
+      prefs_get_int_index("col_order", i));
 	gtk_tree_view_column_set_visible (tvc, visible);
     }
 }
@@ -2205,6 +2228,7 @@ void tm_update_default_sizes (void)
 {
     gint i;
     GtkTreeViewColumn *col;
+    gint col_width;
 
     /* column widths */
     for (i=0; i<TM_NUM_COLUMNS; ++i)
@@ -2212,8 +2236,13 @@ void tm_update_default_sizes (void)
 	col = tm_columns [i];
 	if (col)
 	{
-	    prefs_set_tm_col_width (i,
-				    gtk_tree_view_column_get_width (col));
+    col_width = gtk_tree_view_column_get_width (col);
+    
+    if (col_width > 0)
+	    prefs_set_int_index ("tm_col_width", i, col_width);
+    else
+      prefs_set_int_index ("tm_col_width", i, 80);
+   
 	}
     }
 }
@@ -2259,8 +2288,8 @@ void tm_enable_disable_view_sort (gboolean enable)
 	    fprintf (stderr, "Programming error: disable_count < 0\n");
 	if (disable_count == 0 && track_treeview)
 	{
-	    if ((prefs_get_tm_sort() != SORT_NONE) &&
-		prefs_get_disable_sorting ())
+	    if ((prefs_get_int("tm_sort_") != SORT_NONE) &&
+		sorting_disabled())
 	    {
 		/* Re-enable sorting */
 		GtkTreeModel *model = gtk_tree_view_get_model (track_treeview);
@@ -2268,15 +2297,15 @@ void tm_enable_disable_view_sort (gboolean enable)
 		{
 		    gtk_tree_sortable_set_sort_func (
 			GTK_TREE_SORTABLE (model),
-			prefs_get_tm_sortcol (),
+			prefs_get_int("tm_sortcol"),
 			tm_data_compare_func, NULL, NULL);
 		}
 		else
 		{
 		    gtk_tree_sortable_set_sort_column_id (
 			GTK_TREE_SORTABLE (model),
-			prefs_get_tm_sortcol (),
-			prefs_get_tm_sort ());
+			prefs_get_int("tm_sortcol"),
+			prefs_get_int("tm_sort_"));
 		}
 	    }
 	}
@@ -2285,8 +2314,8 @@ void tm_enable_disable_view_sort (gboolean enable)
     {
 	if (disable_count == 0 && track_treeview)
 	{
-	    if ((prefs_get_tm_sort() != SORT_NONE) &&
-		prefs_get_disable_sorting ())
+	    if ((prefs_get_int("tm_sort_") != SORT_NONE) &&
+		sorting_disabled ())
 	    {
 		/* Disable sorting */
 		GtkTreeModel *model = gtk_tree_view_get_model (track_treeview);
@@ -2294,7 +2323,7 @@ void tm_enable_disable_view_sort (gboolean enable)
 		{
 		    gtk_tree_sortable_set_sort_func (
 			GTK_TREE_SORTABLE (model),
-			prefs_get_tm_sortcol (),
+			prefs_get_int("tm_sortcol"),
 			tm_nosort_comp, NULL, NULL);
 		}
 		else
@@ -2302,7 +2331,7 @@ void tm_enable_disable_view_sort (gboolean enable)
 		    gtk_tree_sortable_set_sort_column_id (
 			GTK_TREE_SORTABLE (model),
 			GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID,
-			prefs_get_tm_sort ());
+			prefs_get_int("tm_sort_"));
 		}
 	    }
 	}
@@ -2419,4 +2448,3 @@ gboolean tm_add_filelist (gchar *data,
     C_FREE (buf);
     return TRUE;
 }
-
