@@ -1,4 +1,4 @@
-/* Time-stamp: <2006-02-25 22:56:41 jcs>
+/* Time-stamp: <2006-06-25 15:57:03 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: file_itunesdb.c,v 1.75 2006/02/25 14:01:43 jcsjcs Exp $
+|  $Id: file_itunesdb.c,v 1.99 2006/06/25 16:08:22 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -42,7 +42,9 @@
 #include "misc.h"
 #include "misc_track.h"
 #include "prefs.h"
+#include "syncdir.h"
 #include "tools.h"
+#include "ipod_init.h"
 
 /*------------------------------------------------------------------*\
  *                                                                  *
@@ -57,6 +59,7 @@ struct track_extended_info
     guint ipod_id;
     gchar *pc_path_locale;
     gchar *pc_path_utf8;
+    time_t mtime;
     gchar *thumb_path_locale;
     gchar *thumb_path_utf8;
     gchar *md5_hash;
@@ -66,12 +69,6 @@ struct track_extended_info
     gint32 oldsize;
     guint32 playcount;
     guint32 rating;        /* still read but never written */
-    guint32 peak_signal;
-    gdouble radio_gain;
-    gdouble audiophile_gain;
-    gboolean peak_signal_set;
-    gboolean radio_gain_set;
-    gboolean audiophile_gain_set;
     gboolean transferred;
 };
 
@@ -89,6 +86,10 @@ static GList *extendeddeletion = NULL;
 static float extendedinfoversion = 0.0;
 
 
+/* Some declarations */
+static gboolean gp_write_itdb (iTunesDB *itdb);
+
+
 /* fills in extended info if available */
 /* num/total are used to give updates in case the md5 checksums have
    to be matched against the files which is very time consuming */
@@ -110,17 +111,15 @@ void fill_in_extended_info (Track *track, gint32 total, gint32 num)
   }
   if (!sei && extendedinfohash_md5)
   {
-      gchar *buf = g_strdup_printf (
+      gtkpod_statusbar_message (
 	  _("Matching MD5 checksum for file %d/%d"),
 	  num, total);
-      gtkpod_statusbar_message (buf);
       while (widgets_blocked && gtk_events_pending ())
 	  gtk_main_iteration ();
-      g_free (buf);
 
       if (!etr->md5_hash)
       {
-	  gchar *filename = get_file_name_on_ipod (track);
+	  gchar *filename = get_file_name_from_source (track, SOURCE_IPOD);
 	  etr->md5_hash = md5_hash_on_filename (filename, FALSE);
 	  g_free (filename);
       }
@@ -132,7 +131,10 @@ void fill_in_extended_info (Track *track, gint32 total, gint32 num)
   if (sei) /* found info for this id! */
   {
       if (sei->pc_path_locale && !etr->pc_path_locale)
+      {
 	  etr->pc_path_locale = g_strdup (sei->pc_path_locale);
+	  etr->mtime = sei->mtime;
+      }
       if (sei->pc_path_utf8 && !etr->pc_path_utf8)
 	  etr->pc_path_utf8 = g_strdup (sei->pc_path_utf8);
       if (sei->thumb_path_locale && !etr->thumb_path_locale)
@@ -147,27 +149,6 @@ void fill_in_extended_info (Track *track, gint32 total, gint32 num)
 	  etr->hostname = g_strdup (sei->hostname);
       etr->oldsize = sei->oldsize;
       track->playcount += sei->playcount;
-      if (sei->peak_signal_set)
-      {
-	  etr->peak_signal_set = sei->peak_signal_set;
-	  etr->peak_signal = sei->peak_signal;
-      }
-      if (extendedinfoversion > 0.81)
-      {
-	  /* before 0.82 we used gint instead of double, so re-reading
-	     the tags is safer (0.81 was CVS only and was a bit messed
-	     up for a while) */
-	  if (sei->radio_gain_set)
-	  {
-	      etr->radio_gain_set = sei->radio_gain_set;
-	      etr->radio_gain = sei->radio_gain;
-	  }
-	  if (sei->audiophile_gain_set)
-	  {
-	      etr->audiophile_gain_set = sei->audiophile_gain_set;
-	      etr->audiophile_gain = sei->audiophile_gain;
-	  }
-      }
       /* FIXME: This means that the rating can never be reset to 0
        * by the iPod */
       if (track->rating == 0)
@@ -384,20 +365,9 @@ static gboolean read_extended_info (gchar *name, gchar *itunes)
 		sei->transferred = atoi (arg);
 	    else if (g_ascii_strcasecmp (line, "filename_ipod") == 0)
 		sei->ipod_path = g_strdup (arg);
-	    else if (g_ascii_strcasecmp (line, "peak_signal") == 0)
+	    else if (g_ascii_strcasecmp (line, "pc_mtime") == 0)
 	    {
-		sei->peak_signal_set = TRUE;
-		sei->peak_signal = g_ascii_strtod (arg, NULL);
-	    }
-	    else if (g_ascii_strcasecmp (line, "radio_gain") == 0)
-	    {
-		sei->radio_gain_set = TRUE;
-		sei->radio_gain = g_ascii_strtod (arg, NULL);
-	    }
-	    else if (g_ascii_strcasecmp (line, "audiophile_gain") == 0)
-	    {
-		sei->audiophile_gain_set = TRUE;
-		sei->audiophile_gain = g_ascii_strtod (arg, NULL);
+		sei->mtime = (time_t)g_ascii_strtoull (arg, NULL, 10);
 	    }
     }
     g_free (md5);
@@ -430,14 +400,21 @@ iTunesDB *gp_import_itdb (iTunesDB *old_itdb, const gint type,
     iTunesDB *itdb = NULL;
     GError *error = NULL;
     gint32 total, num;
+    gboolean offline;
+
 
     g_return_val_if_fail (!(type & GP_ITDB_TYPE_LOCAL) || name_loc, NULL);
     g_return_val_if_fail (!(type & GP_ITDB_TYPE_IPOD) || 
 			  (mp && name_off), NULL);
     g_return_val_if_fail (cfgdir, NULL);
 
+    if (old_itdb)
+	offline = get_offline (old_itdb);
+    else
+	offline = FALSE;
+
     block_widgets ();
-    if (prefs_get_offline() || (type & GP_ITDB_TYPE_LOCAL))
+    if (offline || (type & GP_ITDB_TYPE_LOCAL))
     { /* offline or local database - requires extended info */
 	gchar *name_ext;
 	gchar *name_db;
@@ -455,7 +432,7 @@ iTunesDB *gp_import_itdb (iTunesDB *old_itdb, const gint type,
 
 	if (g_file_test (name_db, G_FILE_TEST_EXISTS))
 	{
-	    if (prefs_get_write_extended_info ())
+	    if (prefs_get_int("write_extended_info"))
 	    {
 		if (!read_extended_info (name_ext, name_db))
 		{
@@ -508,13 +485,17 @@ iTunesDB *gp_import_itdb (iTunesDB *old_itdb, const gint type,
     }
     else
     { /* GP_ITDB_TYPE_IPOD _and_ iPod is connected */
-	const gchar *ext_db[] = { "iPod_Control","iTunes","iTunesDB.ext",NULL};
-	const gchar *db[] = {"iPod_Control","iTunes","iTunesDB",NULL};
-	gchar *name_ext = resolve_path (mp, ext_db);
-	gchar *name_db = resolve_path (mp, db);
+	gchar *name_ext=NULL, *name_db=NULL;
+
+	gchar *itunes_dir = itdb_get_itunes_dir (mp);
+	if (itunes_dir)
+	{
+	    name_ext = itdb_get_path (itunes_dir, "iTunesDB.ext");
+	    name_db  = itdb_get_path (itunes_dir, "iTunesDB");
+	}
 	if (name_db)
 	{
-	    if (prefs_get_write_extended_info ())
+	    if (prefs_get_int("write_extended_info"))
 	    {
 		if (!read_extended_info (name_ext, name_db))
 		{
@@ -547,12 +528,13 @@ iTunesDB *gp_import_itdb (iTunesDB *old_itdb, const gint type,
 	    gchar *name = g_build_filename (
 		mp,
 		"iPod_Control","iTunes","iTunesDB",NULL);
-	    gtkpod_warning (_("'%s' does not exist. Import aborted.\n\n"),
+	    gtkpod_warning (_("'%s' (or similar) does not exist. Import aborted.\n\n"),
 			    name);
 	    g_free (name);
 	}
 	g_free (name_ext);
 	g_free (name_db);
+	g_free (itunes_dir);
     }
 
     if (!itdb)
@@ -567,18 +549,20 @@ iTunesDB *gp_import_itdb (iTunesDB *old_itdb, const gint type,
     eitdb = itdb->userdata;
     g_return_val_if_fail (eitdb, (release_widgets(), NULL));
 
+    eitdb->offline = offline;
+
     /* fill in additional info */
     itdb->usertype = type;
     if (type & GP_ITDB_TYPE_IPOD)
     {
-	if (prefs_get_offline ())
+	if (offline)
 	{
 	    itdb_set_mountpoint (itdb, mp);
 	    g_free (itdb->filename);
 	    itdb->filename = NULL;
 	}
 	eitdb->offline_filename = g_strdup (name_off);
-    }	    
+    }
 
     total = g_list_length (itdb->tracks);
     num = 1;
@@ -592,9 +576,19 @@ iTunesDB *gp_import_itdb (iTunesDB *old_itdb, const gint type,
 	g_return_val_if_fail (etr, (release_widgets(), NULL));
 	fill_in_extended_info (track, total, num);
 	gp_track_validate_entries (track);
-	/* set new default value for flag1 (unknown) */
-	if (track->flag1 == 0)
-	    track->flag1 = 0x02;
+	/* properly set value for has_artwork */
+	if ((track->has_artwork == 0x00) ||
+	    ((track->has_artwork == 0x02) &&
+	     (extendedinfoversion > 0.0) && (extendedinfoversion <= 0.99)))
+	{   /* if has_artwork is not set (0x00), or it has been
+	       (potentially wrongly) set to 0x02 by gtkpod V0.99 or
+	       smaller, determine the correct(?) value */
+	    if (track->artwork->thumbnails)
+		track->has_artwork = 0x01;
+	    else
+		track->has_artwork = 0x02;
+	}
+
 	/* set unk208 to audio if unset (important only for iPod Video) */
 	if (track->unk208 == 0)
 	    track->unk208 = 0x00000001;
@@ -627,7 +621,7 @@ iTunesDB *gp_import_itdb (iTunesDB *old_itdb, const gint type,
     gp_md5_hash_tracks_itdb (itdb);
 
     /* mark the data as unchanged */
-    eitdb->data_changed = FALSE;
+    data_unchanged (itdb);
     /* set mark that this itdb struct contains an imported iTunesDB */
     eitdb->itdb_imported = TRUE;
 
@@ -700,16 +694,13 @@ iTunesDB *gp_import_itdb (iTunesDB *old_itdb, const gint type,
 	eitdb->data_changed = old_eitdb->data_changed;
     }
 
-    /* update all live SPLs */
-    itdb_spl_update_live (itdb);
-
     release_widgets();
 
     return itdb;
 }
 
 /* attempts to import all iPod databases */
-void gp_merge_ipod_itdbs (void)
+void gp_load_ipods (void)
 {
     struct itdbs_head *itdbs_head;
     GList *gl;
@@ -728,10 +719,9 @@ void gp_merge_ipod_itdbs (void)
 	g_return_if_fail (eitdb);
 	if ((itdb->usertype & GP_ITDB_TYPE_IPOD) && !eitdb->itdb_imported)
 	{
-	    gp_merge_itdb (itdb);
+	    gp_load_ipod (itdb);
 	}
     }
-    display_disable_gtkpod_import_buttons ();
 }
 
 
@@ -743,44 +733,235 @@ void gp_merge_ipod_itdbs (void)
  * ->eitdb->offline_filename must be set according to usertype and
  * will be used to read the new itdb
  *
+ * Return value: pointer to the new repository
  */
-void gp_merge_itdb (iTunesDB *old_itdb)
+iTunesDB *gp_merge_itdb (iTunesDB *old_itdb)
 {
     ExtraiTunesDBData *old_eitdb;
     iTunesDB *new_itdb;
 
-    g_return_if_fail (old_itdb);
+    g_return_val_if_fail (old_itdb, NULL);
     old_eitdb = old_itdb->userdata;
-    g_return_if_fail (old_eitdb);
+    g_return_val_if_fail (old_eitdb, NULL);
 
     if (old_itdb->usertype & GP_ITDB_TYPE_LOCAL)
     {
-	g_return_if_fail (old_itdb->filename);
+	g_return_val_if_fail (old_itdb->filename, NULL);
 
 	new_itdb = gp_import_itdb (old_itdb, old_itdb->usertype,
 				   NULL, NULL, old_itdb->filename);
     }
     else if (old_itdb->usertype & GP_ITDB_TYPE_IPOD)
     {
-	g_return_if_fail (old_itdb->mountpoint);
-	g_return_if_fail (old_eitdb->offline_filename);
+	const gchar *mountpoint = itdb_get_mountpoint (old_itdb);
+	g_return_val_if_fail (mountpoint, NULL);
+	g_return_val_if_fail (old_eitdb->offline_filename, NULL);
 
 	new_itdb = gp_import_itdb (old_itdb, old_itdb->usertype,
-				   old_itdb->mountpoint,
+				   mountpoint,
 				   old_eitdb->offline_filename,
 				   NULL);
     }
     else
-	g_return_if_reached ();
+	g_return_val_if_reached (NULL);
 
     if (new_itdb)
+    {
 	gp_replace_itdb (old_itdb, new_itdb);
 
-    gp_update_itdb_prefs ();
+	/* take care of autosync... */
+	sync_all_playlists (new_itdb);
+
+	/* update all live SPLs */
+	itdb_spl_update_live (new_itdb);
+    }
+
     gtkpod_tracks_statusbar_update ();
+
+    return new_itdb;
 }
 
 
+/**
+ * gp_load_ipod: loads the contents of an iPod into @itdb. If data
+ * already exists in @itdb, data is merged.
+ *
+ * @itdb: repository to load iPod contents into. mountpoint must be
+ * set, and the iPod must not be loaded already
+ * (eitdb->itdb_imported).
+ *
+ * Return value: the new repository holding the contents of the iPod.
+ */
+iTunesDB *gp_load_ipod (iTunesDB *itdb)
+{
+    ExtraiTunesDBData *eitdb;
+    iTunesDB *new_itdb = NULL;
+    gchar *mountpoint;
+    gchar *itunesdb;
+    gboolean load = TRUE;
+
+    g_return_val_if_fail (itdb, NULL);
+    g_return_val_if_fail (itdb->usertype & GP_ITDB_TYPE_IPOD, NULL);
+    eitdb = itdb->userdata;
+    g_return_val_if_fail (eitdb, NULL);
+    g_return_val_if_fail (eitdb->itdb_imported == FALSE, NULL);
+
+    mountpoint = get_itdb_prefs_string (itdb, KEY_MOUNTPOINT);
+    call_script ("gtkpod.load", mountpoint, NULL);
+
+    itdb_device_set_mountpoint (itdb->device, mountpoint);
+
+    itunesdb = itdb_get_itunesdb_path (mountpoint);
+    if (!itunesdb)
+    {
+	gchar *str = g_strdup_printf (_("Could not find iPod directory structure at '%s'.\nIf you are sure that the iPod is properly mounted at '%s', gtkpod can create the directory structure for you.\n\nDo you want to create the directory structure now?\n"), mountpoint, mountpoint);
+	GtkWidget *dialog = gtk_message_dialog_new (
+	    GTK_WINDOW (gtkpod_window),
+	    GTK_DIALOG_DESTROY_WITH_PARENT,
+	    GTK_MESSAGE_WARNING,
+	    GTK_BUTTONS_YES_NO,
+	    str);
+	gint result = gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+	g_free (str);
+
+	if (result == GTK_RESPONSE_YES)
+	{
+	    load = gp_ipod_init (itdb);
+	}
+	else
+	{
+	    load = FALSE;
+	}
+    }
+    g_free (itunesdb);
+    g_free (mountpoint);
+
+    if (load)
+    {
+	new_itdb = gp_merge_itdb (itdb);
+	if (new_itdb)
+	{
+	    gchar *old_model = get_itdb_prefs_string (new_itdb,
+						      KEY_IPOD_MODEL);
+	    gchar *new_model = itdb_device_get_sysinfo (new_itdb->device,
+							"ModelNumStr");
+
+	    if (!old_model && new_model)
+	    {
+		set_itdb_prefs_string (new_itdb, KEY_IPOD_MODEL, new_model);
+	    }
+	    else if (old_model && !new_model)
+	    {
+		gp_ipod_init_set_model (new_itdb, old_model);
+	    }
+	    else if (!old_model && !new_model)
+	    {
+		gp_ipod_init_set_model (new_itdb, NULL);
+	    }
+	    else
+	    {   /* old_model && new_model are set */
+#if 0
+		const gchar *old_ptr = old_model;
+		const gchar *new_ptr = new_model;
+		/* Normalize model number */
+		if (isalpha (old_model[0]))   ++old_ptr;
+		if (isalpha (new_model[0]))   ++new_ptr;
+		if (strcmp (old_ptr, new_ptr) != 0)
+		{   /* Model number has changed -- confirm */
+                }
+#endif		
+		set_itdb_prefs_string (new_itdb, KEY_IPOD_MODEL, new_model);
+	    }
+	}
+    }
+    return new_itdb;
+}
+
+
+/**
+ * gp_eject_ipod: store @itdb and call ~/.gtkpod/gtkpod.eject with the
+ * mountpoint as parameter. Then @itdb is deleted and replaced with an
+ * empty version. eitdb->ejected is set.
+ *
+ * @itdb: must be an iPod itdb (eject does not make sense otherwise)
+ *
+ * Return value: TRUE if saving was successful, FALSE otherwise.
+ */
+gboolean gp_eject_ipod (iTunesDB *itdb)
+{
+
+    g_return_val_if_fail (itdb, FALSE);
+    g_return_val_if_fail (itdb->usertype & GP_ITDB_TYPE_IPOD, FALSE);
+
+    if (gp_save_itdb (itdb))
+    {
+	gint index;
+	gchar *mountpoint;
+	iTunesDB *new_itdb;
+
+	mountpoint = get_itdb_prefs_string (itdb, KEY_MOUNTPOINT);
+	call_script ("gtkpod.eject", mountpoint, FALSE);
+	g_free (mountpoint);
+
+	index = get_itdb_index (itdb);
+	new_itdb = setup_itdb_n (index);
+	if (new_itdb)
+	{
+	    ExtraiTunesDBData *new_eitdb;
+
+	    new_eitdb = new_itdb->userdata;
+	    g_return_val_if_fail (new_eitdb, TRUE);
+
+	    gp_replace_itdb (itdb, new_itdb);
+
+	    new_eitdb->ipod_ejected = TRUE;
+    	}
+	return TRUE;
+    }
+    return FALSE;
+}
+
+
+/**
+ * gp_save_itdb: Save a repository after updating smart playlists. If
+ * the repository is an iPod, contacts, notes and calendar are also
+ * updated.
+ *
+ * @itdb: repository to save
+ *
+ * return value: TRUE on succes, FALSE when an error occured.
+ */
+gboolean gp_save_itdb (iTunesDB *itdb)
+{
+    Playlist *pl;
+    gboolean success;
+    g_return_val_if_fail (itdb, FALSE);
+
+    /* update smart playlists before writing */
+    itdb_spl_update_live (itdb);
+    pl = pm_get_selected_playlist ();
+    if (pl && (pl->itdb == itdb) &&
+	pl->is_spl && pl->splpref.liveupdate)
+    {   /* Update display if necessary */
+	st_redisplay (0);
+    }
+
+    success = gp_write_itdb (itdb);
+
+    if (itdb->usertype & GP_ITDB_TYPE_IPOD)
+    {
+	if (get_itdb_prefs_int (itdb, "concal_autosync"))
+	{
+	    tools_sync_all (itdb);
+	}
+    }
+
+    return success;
+}
+
+
+
 /*------------------------------------------------------------------*\
  *                                                                  *
  *      Functions concerning deletion of tracks                      *
@@ -935,28 +1116,12 @@ static gboolean write_extended_info (iTunesDB *itdb)
 	  fprintf (fp, "charset=%s\n", etr->charset);
       if (!track->transferred && etr->oldsize)
 	  fprintf (fp, "oldsize=%d\n", etr->oldsize);
-      if (etr->peak_signal_set)
-      {
-	  gchar buf[20];
-	  g_ascii_dtostr (buf, 20, (gdouble)etr->peak_signal);
-	  fprintf (fp, "peak_signal=%s\n", buf);
-      }
-      if (etr->radio_gain_set)
-      {
-	  gchar buf[20];
-	  g_ascii_dtostr (buf, 20, etr->radio_gain);
-	  fprintf (fp, "radio_gain=%s\n", buf);
-      }
-      if (etr->audiophile_gain_set)
-      {
-	  gchar buf[20];
-	  g_ascii_dtostr (buf, 20, etr->audiophile_gain);
-	  fprintf (fp, "audiophile_gain=%s\n", buf);
-      }
+      if (etr->mtime)
+	  fprintf (fp, "pc_mtime=%llu\n", (unsigned long long)etr->mtime);
       fprintf (fp, "transferred=%d\n", track->transferred);
       while (widgets_blocked && gtk_events_pending ())  gtk_main_iteration ();
   }
-  if (prefs_get_offline())
+  if (get_offline(itdb))
   { /* we are offline and also need to export the list of tracks that
        are to be deleted */
       for(gl = eitdb->pending_deletion; gl; gl = gl->next)
@@ -997,18 +1162,17 @@ static gpointer th_remove (gpointer filename)
 
 /* Threaded copy of ipod track */
 /* Returns: GError *error */
-static gpointer th_copy (gpointer s)
+static gpointer th_copy (gpointer data)
 {
-    Track *track = s;
+    Track *track = data;
     ExtraTrackData *etr;
-    gchar *mount = charset_from_utf8 (prefs_get_ipod_mount ());
     GError *error = NULL;
     g_return_val_if_fail (track, NULL);
     etr = track->userdata;
     g_return_val_if_fail (etr, NULL);
 
     itdb_cp_track_to_ipod (track, etr->pc_path_locale, &error);
-    g_free (mount);
+
     /* delete old size */
     if (track->transferred) etr->oldsize = 0;
 #ifdef G_THREADS_ENABLED
@@ -1029,25 +1193,29 @@ static void file_dialog_abort (gboolean *abort_flag)
 
 
 /* check if iPod directory stucture is present */
-static gboolean ipod_dirs_present (gchar *mountpoint)
+static gboolean ipod_dirs_present (const gchar *mountpoint)
 {
-    const gchar *music[] = {"iPod_Control", "Music", "F00", NULL},
-      *itunes[] = {"iPod_Control","iTunes",NULL};
     gchar *file;
+    gchar *dir;
     gboolean result = TRUE;
 
     g_return_val_if_fail (mountpoint, FALSE);
 
-    file = resolve_path(mountpoint, music);
-    if(!file || !g_file_test(file,G_FILE_TEST_IS_DIR))
+    dir = itdb_get_music_dir (mountpoint);
+    if (!dir)
+	return FALSE;
+
+    file = itdb_get_path (dir, "F00");
+    if (!file || !g_file_test(file, G_FILE_TEST_IS_DIR))
+	result = FALSE;
+    g_free (file);
+    g_free (dir);
+
+    dir = itdb_get_itunes_dir (mountpoint);
+    if(!dir || !g_file_test(dir, G_FILE_TEST_IS_DIR))
       result = FALSE;
-    g_free(file);
-    
-    file = resolve_path(mountpoint, itunes);
-    if(!file || !g_file_test(file,G_FILE_TEST_IS_DIR))
-      result = FALSE;
-    g_free(file);
-    
+    g_free(dir);
+
     return result;
 }
 
@@ -1133,7 +1301,7 @@ static gboolean delete_files (iTunesDB *itdb)
 
   if (itdb->usertype & GP_ITDB_TYPE_IPOD)
   {
-      g_return_val_if_fail (itdb->mountpoint, FALSE);
+      g_return_val_if_fail (itdb_get_mountpoint (itdb), FALSE);
   }
 
   abort_flag = FALSE;
@@ -1154,12 +1322,12 @@ static gboolean delete_files (iTunesDB *itdb)
       if (itdb->usertype & GP_ITDB_TYPE_IPOD)
       {
 	  track->itdb = itdb;
-	  filename = itdb_filename_on_ipod (track);
+	  filename = get_file_name_from_source (track, SOURCE_IPOD);
 	  track->itdb = NULL;
       }
       if (itdb->usertype & GP_ITDB_TYPE_LOCAL)
       {
-	  filename = get_file_name_on_harddisk (track);
+	  filename = get_file_name_from_source (track, SOURCE_LOCAL);
       }
 
       if(filename)
@@ -1218,7 +1386,6 @@ static gboolean flush_tracks (iTunesDB *itdb)
 {
   GList *gl;
   gint count, n;
-  gchar *buf;
   gboolean result = TRUE;
   static gboolean abort_flag;
   GtkWidget *dialog, *progress_bar;
@@ -1249,7 +1416,6 @@ static gboolean flush_tracks (iTunesDB *itdb)
   g_free (progtext);
 
   /* count number of tracks to be transferred */
-  if (n != 0)  display_disable_gtkpod_import_buttons();
   count = 0; /* tracks transferred */
   start = time (NULL);
 
@@ -1303,15 +1469,18 @@ static gboolean flush_tracks (iTunesDB *itdb)
 				  and more files to the iPod */
 	  ++count;
 	  if (count == 1)  /* we need longer timeout */
-	      prefs_set_statusbar_timeout (3*STATUSBAR_TIMEOUT);
+	  {
+	      gtkpod_statusbar_timeout (3*STATUSBAR_TIMEOUT);
+	  }
 	  if (count == n)  /* we need to reset timeout */
-	      prefs_set_statusbar_timeout (0);
-	  buf = g_strdup_printf (ngettext ("Copied %d of %d new track.",
-					   "Copied %d of %d new tracks.", n),
-				 count, n);
-	  gtkpod_statusbar_message(buf);
-	  g_free (buf);
-	  
+	  {
+	      gtkpod_statusbar_timeout (0);
+          }
+	  gtkpod_statusbar_message (
+	      ngettext ("Copied %d of %d new track.",
+			"Copied %d of %d new tracks.", n),
+	      count, n);
+
 	  gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR (progress_bar),
 					(gdouble) count/n);
 	  
@@ -1331,17 +1500,18 @@ static gboolean flush_tracks (iTunesDB *itdb)
       }
       while (widgets_blocked && gtk_events_pending ())  gtk_main_iteration ();
   } /* for (.;.;.) */
+
   if (abort_flag)      result = FALSE;   /* negative result if user aborted */
   if (result == FALSE)
       gtkpod_statusbar_message (_("Some tracks were not written to iPod. Export aborted!"));
-  prefs_set_statusbar_timeout (0);
+  gtkpod_statusbar_timeout (0);
   gtk_widget_destroy (dialog);
   while (widgets_blocked && gtk_events_pending ())  gtk_main_iteration ();
   return result;
 }
 
 
-gboolean gp_write_itdb (iTunesDB *itdb)
+static gboolean gp_write_itdb (iTunesDB *itdb)
 {
   gchar *cfgdir;
   gboolean success = TRUE;
@@ -1366,17 +1536,15 @@ gboolean gp_write_itdb (iTunesDB *itdb)
       }
       else if (itdb->usertype & GP_ITDB_TYPE_IPOD)
       {
-	  if (prefs_get_offline ())
+	  if (get_offline (itdb))
 	  {
 	      tunes = g_strdup (eitdb->offline_filename);
 	  }
 	  else
 	  {
-	      const gchar *itunes_components[] = {"iPod_Control",
-						  "iTunes",
-						  "iTunesDB", NULL};
-	      g_return_val_if_fail (itdb->mountpoint, FALSE);
-	      tunes = resolve_path (itdb->mountpoint, itunes_components);
+	      const gchar *mountpoint = itdb_get_mountpoint (itdb);
+	      g_return_val_if_fail (mountpoint, FALSE);
+	      tunes = itdb_get_itunesdb_path (mountpoint);
 	  }
       }
       else
@@ -1406,15 +1574,16 @@ gboolean gp_write_itdb (iTunesDB *itdb)
 
   block_widgets (); /* block user input */
 
-  if((itdb->usertype & GP_ITDB_TYPE_IPOD) && !prefs_get_offline ())
+  if((itdb->usertype & GP_ITDB_TYPE_IPOD) && !get_offline (itdb))
   {
-      g_return_val_if_fail (itdb->mountpoint, FALSE);
+      const gchar *mountpoint = itdb_get_mountpoint (itdb);
+      g_return_val_if_fail (mountpoint, FALSE);
       /* check if iPod directories are present */
-      if (!ipod_dirs_present (itdb->mountpoint))
+      if (!ipod_dirs_present (mountpoint))
       {   /* no -- create them */
-	  ipod_directories_head (itdb->mountpoint);
+	  gp_ipod_init (itdb);
 	  /* if still not present abort */
-	  if (!ipod_dirs_present (itdb->mountpoint))
+	  if (!ipod_dirs_present (mountpoint))
 	  {
 	      gtkpod_warning (_("iPod directory structure must be present before synching to the iPod can be performed.\n"));
 	      success = FALSE;
@@ -1441,12 +1610,12 @@ gboolean gp_write_itdb (iTunesDB *itdb)
   }
 
   if (success)
-      gtkpod_statusbar_message (_("Now writing iTunesDB. Please wait..."));
+      gtkpod_statusbar_message (_("Now writing database. Please wait..."));
   while (widgets_blocked && gtk_events_pending ())
       gtk_main_iteration ();
 
 
-  if (success && !prefs_get_offline () &&
+  if (success && !get_offline (itdb) &&
       (itdb->usertype & GP_ITDB_TYPE_IPOD))
   {   /* write to the iPod */
       GError *error = NULL;
@@ -1476,7 +1645,7 @@ gboolean gp_write_itdb (iTunesDB *itdb)
       }
       if (success)
       {
-	  if (prefs_get_write_extended_info ())
+	  if (prefs_get_int("write_extended_info"))
 	  {   /* write extended information */
 	      success = write_extended_info (itdb);
 	  }
@@ -1487,9 +1656,7 @@ gboolean gp_write_itdb (iTunesDB *itdb)
 	      {
 		  if (remove (ext) != 0)
 		  {
-		      gchar *buf = g_strdup_printf (_("Extended information file not deleted: '%s\'"), ext);
-		      gtkpod_statusbar_message (buf);
-		      g_free (buf);
+		      gtkpod_statusbar_message (_("Extended information file not deleted: '%s\'"), ext);
 		  }
 	      }
 	  }
@@ -1507,7 +1674,7 @@ gboolean gp_write_itdb (iTunesDB *itdb)
 	      g_error_free (error);
 	      error = NULL;
 	  }
-	  if (prefs_get_write_extended_info ())
+	  if (prefs_get_int("write_extended_info"))
 	  {
 	      gchar *from, *to;
 	      from = g_strdup_printf ("%s.ext", itdb->filename);
@@ -1527,7 +1694,7 @@ gboolean gp_write_itdb (iTunesDB *itdb)
       }
   }
 
-  if (success && prefs_get_offline () &&
+  if (success && get_offline (itdb) &&
       (itdb->usertype & GP_ITDB_TYPE_IPOD))
   {   /* write to cfgdir */
       GError *error = NULL;
@@ -1541,7 +1708,7 @@ gboolean gp_write_itdb (iTunesDB *itdb)
 	  g_error_free (error);
 	  error = NULL;
       }
-      if (success && prefs_get_write_extended_info ())
+      if (success && prefs_get_int("write_extended_info"))
       {   /* write extended information */
 	  success = write_extended_info (itdb);
       }
@@ -1570,11 +1737,17 @@ gboolean gp_write_itdb (iTunesDB *itdb)
   /* indicate that files and/or database is saved */
   if (success)
   {
-      eitdb->data_changed = FALSE;
-      /* block menu item and button if successfully written to iPod */
+      Playlist *mpl = itdb_playlist_mpl (itdb);
+      g_return_val_if_fail (mpl, success);
+      data_unchanged (itdb);
       if (itdb->usertype & GP_ITDB_TYPE_IPOD)
-	  display_disable_gtkpod_import_buttons();
-      gtkpod_statusbar_message(_("iPod Database Saved"));
+      {
+	  gtkpod_statusbar_message(_("%s: Database saved"), mpl->name);
+      }
+      else
+      {
+	  gtkpod_statusbar_message(_("%s: Changes saved"), mpl->name);
+      }
   }
 
   g_free (cfgdir);
@@ -1610,36 +1783,16 @@ void handle_export (void)
 
     for (gl=itdbs_head->itdbs; gl; gl=gl->next)
     {
-	iTunesDB *itdb = gl->data;
 	ExtraiTunesDBData *eitdb;
+	iTunesDB *itdb = gl->data;
 	g_return_if_fail (itdb);
 	eitdb = itdb->userdata;
 	g_return_if_fail (eitdb);
-	if (eitdb->data_changed || eitdb->itdb_imported)
-	{
-	    Playlist *pl;
-	    /* update smart playlists before writing */
-	    itdb_spl_update_live (itdb);
-	    pl = pm_get_selected_playlist ();
-	    if (pl && (pl->itdb == itdb) &&
-		pl->is_spl && pl->splpref.liveupdate)
-	    {   /* Update display if necessary */
-		st_redisplay (0);
-	    }
-	    success &= gp_write_itdb (itdb);
-	}
-    }
 
-    if (prefs_get_concal_autosync ())
-    {
-	const gchar *str;
-	gtkpod_statusbar_message (_("Syncing contacts, calendar and notes..."));
-	str = prefs_get_path (PATH_SYNC_CONTACTS);
-	if (str && *str)    tools_sync_contacts ();
-	str = prefs_get_path (PATH_SYNC_CALENDAR);
-	if (str && *str)    tools_sync_calendar ();
-	str = prefs_get_path (PATH_SYNC_NOTES);
-	if (str && *str)    tools_sync_notes ();
+	if (eitdb->data_changed)
+	{
+	    success &= gp_save_itdb (itdb);
+	}
     }
 
     release_widgets ();
@@ -1648,7 +1801,8 @@ void handle_export (void)
 
 
 
-/* indicate that data was changed and update the free space indicator */
+/* indicate that data was changed and update the free space indicator,
+ * as well as the changed indicator in the playlist view */
 void data_changed (iTunesDB *itdb)
 {
     ExtraiTunesDBData *eitdb;
@@ -1657,6 +1811,22 @@ void data_changed (iTunesDB *itdb)
     g_return_if_fail (eitdb);
 
     eitdb->data_changed = TRUE;
+    pm_name_changed (itdb_playlist_mpl (itdb));
+    space_data_update ();
+}
+
+
+/* indicate that data was changed and update the free space indicator,
+ * as well as the changed indicator in the playlist view */
+void data_unchanged (iTunesDB *itdb)
+{
+    ExtraiTunesDBData *eitdb;
+    g_return_if_fail (itdb);
+    eitdb = itdb->userdata;
+    g_return_if_fail (eitdb);
+
+    eitdb->data_changed = FALSE;
+    pm_name_changed (itdb_playlist_mpl (itdb));
     space_data_update ();
 }
 
@@ -1686,59 +1856,3 @@ gboolean files_are_saved (void)
     }
     return !changed;
 }
-
-
-/* FIXME: transitional function to set the mountpoint of all
- * GP_ITDB_TYPE_IPOD itdbs. Should be replaced with a new system to
- * individually define the mountpoints */
-void gp_itdb_set_mountpoint (const gchar *mp)
-{
-    struct itdbs_head *itdbs_head;
-
-    GList *gl;
-    gint i;
-
-    g_return_if_fail (gtkpod_window);
-    itdbs_head = g_object_get_data (G_OBJECT (gtkpod_window),
-				    "itdbs_head");
-    if (!itdbs_head)
-    {  /* this means we are called before all the data is set up, for
-	* example because gtkpod was called with the '-m' flag.
-	* We'll set all itdb_n_mountpoint for which itdb_n_type is
-	* GP_ITDB_TYPE_IPOD */
-	gboolean valid;
-	i=0;
-	do
-	{
-	    gchar *property = g_strdup_printf ("itdb_%d_type", i);
-	    gint type;
-	    valid = prefs_get_int_value (property, &type);
-	    g_free (property);
-	    if (valid && (type & GP_ITDB_TYPE_IPOD))
-	    {
-		gchar *pmp = g_strdup_printf ("itdb_%d_mountpoint", i);
-		prefs_set_string_value (pmp, mp);
-		g_free (pmp);
-	    }
-	    ++i;
-	} while (valid);
-	return;
-    }
-
-    i=0;
-    for (gl=itdbs_head->itdbs; gl; gl=gl->next)
-    {
-	iTunesDB *itdb = gl->data;
-	g_return_if_fail (itdb);
-	if (itdb->usertype & GP_ITDB_TYPE_IPOD)
-	{
-	    /* store to prefs */
-	    gchar *pmp = g_strdup_printf ("itdb_%d_mountpoint", i);
-	    prefs_set_string_value (pmp, mp);
-	    g_free (pmp);
-	    /* store to itdb */
-	    itdb_set_mountpoint (itdb, mp);
-	}
-	++i;
-    }
-}
