@@ -1,4 +1,4 @@
-/* Time-stamp: <2006-03-05 18:25:08 jcs>
+/* Time-stamp: <2006-06-23 23:59:03 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users.sourceforge.net>
 |  Part of the gtkpod project.
@@ -23,7 +23,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: fileselection.c,v 1.5 2006/03/05 09:28:58 jcsjcs Exp $
+|  $Id: fileselection.c,v 1.13 2006/06/23 16:03:06 jcsjcs Exp $
 */
 
 /***********************************************************************
@@ -96,15 +96,19 @@ static void add_files_ok(GtkFileChooser* filechooser)
     current = names;
 
     if (current)
-	prefs_set_last_dir_browse ((gchar *)current->data);
-	
+    {
+	gchar *dirname = gtk_file_chooser_get_current_folder (filechooser);
+	prefs_set_string ("last_dir_browsed", dirname);
+	g_free (dirname);
+    }
+
     /* Get the filenames and add them */
     while (current != NULL)
     {
 	result &= add_track_by_filename(fc_active_itdb,
 					(gchar*)current->data,
 					playlist,
-					prefs_get_add_recursively(),
+					prefs_get_int("add_recursively"),
 					NULL, NULL);
 	current = current->next;
     }
@@ -139,10 +143,15 @@ void create_add_files_dialog (void)
 {
     GtkWidget* fc;  /* The file chooser dialog */
     gint response;  /* The response of the filechooser */
+    gchar *last_dir;
 	
     /* Grab the current playlist to add songs to */
-    fc_active_itdb = gp_get_active_itdb ();
-    g_return_if_fail (fc_active_itdb);
+    fc_active_itdb = gp_get_selected_itdb ();
+    if (!fc_active_itdb)
+    {
+	message_sb_no_itdb_selected ();
+	return;
+    }
 	
     /* Create the file chooser, and handle the response */
     fc = gtk_file_chooser_dialog_new (_("Add Files"),
@@ -155,8 +164,13 @@ void create_add_files_dialog (void)
 				      NULL);
 
     gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (fc), TRUE);
-    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (fc),
-					 prefs_get_last_dir_browse ());
+    last_dir = prefs_get_string ("last_dir_browsed");
+    if (last_dir)
+    {
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (fc),
+					     last_dir);
+	g_free (last_dir);
+    }
 
     response = gtk_dialog_run(GTK_DIALOG(fc));
 	
@@ -191,7 +205,11 @@ static void add_playlists_ok(GtkFileChooser* filechooser)
     current = names;
 
     if (current)
-	prefs_set_last_dir_browse ((gchar *)current->data);
+    {
+	gchar *dirname = gtk_file_chooser_get_current_folder (filechooser);
+	prefs_set_string ("last_dir_browsed", dirname);
+	g_free (dirname);
+    }
 
     while (current != NULL)
     {
@@ -218,10 +236,15 @@ void create_add_playlists_dialog(void)
 {
     GtkWidget* fc ; /* The file chooser dialog */
     gint response;  /* The response of the filechooser */
+    gchar *last_dir;
 	
     /* Grab the current playlist to add songs to */
-    fc_active_itdb = gp_get_active_itdb ();
-    g_return_if_fail (fc_active_itdb);
+    fc_active_itdb = gp_get_selected_itdb ();
+    if (!fc_active_itdb)
+    {
+	message_sb_no_itdb_selected ();
+	return;
+    }
 	
     /* Create the file chooser, and handle the response */
     fc = gtk_file_chooser_dialog_new (_("Add Playlists"),
@@ -234,8 +257,13 @@ void create_add_playlists_dialog(void)
 				      NULL);
 	
     gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (fc), TRUE);
-    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (fc),
-					 prefs_get_last_dir_browse ());
+    last_dir = prefs_get_string ("last_dir_browsed");
+    if (last_dir)
+    {
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (fc),
+					     last_dir);
+	g_free (last_dir);
+    }
 
     response = gtk_dialog_run(GTK_DIALOG(fc));
 	
@@ -263,8 +291,7 @@ gchar *fileselection_get_cover_filename (void)
     GtkWidget* fc;  /* The file chooser dialog */
     gint response;  /* The response of the filechooser */
     gchar *filename = NULL; /* The chosen file */
-    const gchar *dir1;
-    gchar *dir2;
+    gchar *last_dir, *new_dir;
 
     /* Create the file chooser, and handle the response */
     fc = gtk_file_chooser_dialog_new (_("Set Cover"),
@@ -278,17 +305,22 @@ gchar *fileselection_get_cover_filename (void)
 
     gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (fc), FALSE);
 
-    dir1 = prefs_get_last_dir_browse ();
-    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (fc), dir1);
+    last_dir = prefs_get_string ("last_dir_browsed");
+    if (last_dir)
+    {
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (fc),
+					     last_dir);
+	g_free (last_dir);
+    }
 
     response = gtk_dialog_run(GTK_DIALOG(fc));
 
     switch (response)
     {
     case GTK_RESPONSE_ACCEPT:
-	dir2 = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (fc));
-	prefs_set_last_dir_browse (dir2);
-	g_free (dir2);
+	new_dir = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (fc));
+	prefs_set_string ("last_dir_browsed", new_dir);
+	g_free (new_dir);
 	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (fc));
 	break;
     case GTK_RESPONSE_CANCEL:
@@ -301,7 +333,192 @@ gchar *fileselection_get_cover_filename (void)
 }
 
 
+/* BY JCS */
 
+/* Get a file or directory
+ *
+ * @title:    title for the file selection dialog
+ * @cur_file: initial file to be selected. If NULL, then use
+ *            last_dir_browse.
+ * @action:
+ *   GTK_FILE_CHOOSER_ACTION_OPEN   Indicates open mode. The file chooser
+ *                                  will only let the user pick an
+ *                                  existing file.
+ *   GTK_FILE_CHOOSER_ACTION_SAVE   Indicates save mode. The file
+ *                                  chooser will let the user pick an
+ *                                  existing file, or type in a new
+ *                                  filename.
+ *   GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER
+ *                                  Indicates an Open mode for
+ *                                  selecting folders. The file
+ *                                  chooser will let the user pick an
+ *                                  existing folder.
+ *   GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER
+ *                                  Indicates a mode for creating a
+ *                                  new folder. The file chooser will
+ *                                  let the user name an existing or
+ *                                  new folder.
+ */
+gchar *fileselection_get_file_or_dir (const gchar *title,
+				      const gchar *cur_file,
+				      GtkFileChooserAction action)
+{
+    GtkWidget* fc;  /* The file chooser dialog */
+    gint response;  /* The response of the filechooser */
+    gchar *new_file = NULL; /* The chosen file */
+
+    g_return_val_if_fail (title, NULL);
+
+    /* Create the file chooser, and handle the response */
+    fc = gtk_file_chooser_dialog_new (
+	title,
+	NULL,
+	action,
+	GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+	GTK_STOCK_OK,GTK_RESPONSE_ACCEPT,
+	NULL);
+
+    gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (fc), FALSE);
+
+    if (cur_file)
+    {
+	/* Sanity checks: must exist and be absolute */
+	if (g_path_is_absolute (cur_file))
+	    gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (fc),
+					   cur_file);
+	else
+	    cur_file = NULL;
+    }
+    if (cur_file == NULL)
+    {
+	gchar *filename = prefs_get_string ("last_dir_browsed");
+	if (filename)
+	{
+	    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (fc),
+						 filename);
+	    g_free (filename);
+	}
+    }
+
+    response = gtk_dialog_run(GTK_DIALOG(fc));
+
+    switch (response)
+    {
+    case GTK_RESPONSE_ACCEPT:
+	new_file = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (fc));
+	break;
+    case GTK_RESPONSE_CANCEL:
+	break;
+    default:	/* Fall through */
+	break;
+    }		
+    gtk_widget_destroy(fc);
+    return new_file;
+}
+
+
+
+/* BY JCS */
+
+/* Used by the prefs system (prefs_windows.c, repository.c) when a
+ * script should be selected. Takes into account that command line
+ * arguments can be present
+ *
+ * @opath: the current path to the script including command line
+ *         arguments. May be NULL.
+ * @fallback: default dir in case @opath is not set.
+ * @title: title of the file selection window.
+ * @additional_text: additional explanotary text to be displayed
+ *
+ * Return value: The new script including command line arguments. NULL
+ * if the selection was aborted.
+ */
+gchar *fileselection_select_script (const gchar *opath,
+				    const gchar *fallback,
+				    const gchar *title,
+				    const gchar *additional_text)
+{
+    gchar *npath=NULL;
+    gchar *buf, *fbuf;
+    const gchar *opathp;
+    GtkFileChooser *fc;
+    gint response;  /* The response of the filechooser */
+
+    fc = GTK_FILE_CHOOSER (gtk_file_chooser_dialog_new (
+			       title,
+			       NULL,
+			       GTK_FILE_CHOOSER_ACTION_OPEN,
+			       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			       GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
+			       NULL));
+
+    /* find first whitespace separating path from command line
+     * arguments */
+
+    if (opath)
+	opathp = strchr (opath, ' ');
+    else
+	opathp = NULL;
+
+    if (opathp)
+	buf = g_strndup (opath, opathp-opath);
+    else
+	buf = g_strdup (opath);
+
+    /* get full path -- if the file cannot be found it can't be
+     * selected in the filechooser */
+    if (buf)
+    {
+	fbuf = g_find_program_in_path (buf);
+	g_free (buf);
+    }
+    else
+    {
+	fbuf = NULL;
+    }
+
+    if (!fbuf)
+    {   /* set default */
+	fbuf = g_strdup (fallback);
+    }
+
+    if (fbuf && *fbuf)
+    {
+	gchar *fbuf_utf8 = g_filename_from_utf8 (fbuf, -1, NULL, NULL, NULL);
+	if (g_file_test (fbuf, G_FILE_TEST_IS_DIR))
+	{
+	    gtk_file_chooser_set_current_folder (fc, fbuf_utf8);
+	}
+	else
+	{
+	    gtk_file_chooser_set_filename (fc, fbuf_utf8);
+	}
+	g_free (fbuf_utf8);
+    }
+    g_free (fbuf);
+
+    response = gtk_dialog_run(GTK_DIALOG(fc));
+
+    switch (response)
+    {
+    case GTK_RESPONSE_ACCEPT:
+	buf = gtk_file_chooser_get_filename (fc);
+	/* attach command line arguments if present */
+	if (opathp)
+	    npath = g_strdup_printf ("%s%s", buf, opathp);
+	else
+	    npath = g_strdup (buf);
+	g_free (buf);
+	break;
+    case GTK_RESPONSE_CANCEL:
+	break;
+    default:	/* Fall through */
+	break;
+    }
+    gtk_widget_destroy(GTK_WIDGET (fc));
+
+    return npath;
+}
 
 
 
@@ -459,9 +676,9 @@ static void add_dir_selected (gchar *dir)
     {
 	Playlist *plitem = pm_get_selected_playlist ();
 	add_directory_by_name (db_active_itdb, dir, plitem,
-			       prefs_get_add_recursively (),
+			       prefs_get_int("add_recursively"),
 			       NULL, NULL);
-	prefs_set_last_dir_browse(dir);
+	prefs_set_string ("last_dir_browsed", dir);
 	gtkpod_tracks_statusbar_update();
     }
     else
@@ -481,17 +698,28 @@ static void add_dir_selected (gchar *dir)
    separate callback for gtkpod.glade */
 void dirbrowser_create (void)
 {
+    gchar *cur_dir;
+
     if (dirbrowser)
     {   /* file selector already open -- raise to the top */
 	gdk_window_raise (dirbrowser->window);
 	return;
     }
-    db_active_itdb = gp_get_active_itdb ();
+
+    db_active_itdb = gp_get_selected_itdb ();
+    if (!db_active_itdb)
+    {
+	message_sb_no_itdb_selected ();
+	return;
+    }
+
+    cur_dir = prefs_get_string ("last_dir_browsed");
     dirbrowser = xmms_create_dir_browser (
 	_("Select directory to add recursively"),
-	prefs_get_last_dir_browse (),
+	cur_dir,
 	GTK_SELECTION_MULTIPLE,
 	add_dir_selected);
+    g_free (cur_dir);
     gtk_widget_show (dirbrowser);
 }
 
@@ -513,7 +741,8 @@ static void add_dir_close (GtkWidget *w1, GtkWidget *w2)
 
     gtk_window_get_size (GTK_WINDOW (dirbrowser), &x, &y);
     /* store size for next time */
-    prefs_set_size_dirbr (x, y);
+    prefs_set_int("size_dirbr.x", x);
+    prefs_set_int("size_dirbr.y", y);
     gtk_widget_destroy(dirbrowser);
     /* dirbrowser = NULL; -- will be done by the dirbrowser_destroy()
        as part of the callback */
@@ -693,7 +922,8 @@ static GtkWidget *xmms_create_dir_browser (const char *title,
     gint x,y;
 
     window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-    prefs_get_size_dirbr (&x, &y);
+    x = prefs_get_int("size_dirbr.x");
+    y = prefs_get_int("size_dirbr.y");
     gtk_window_set_default_size(GTK_WINDOW(window), x, y);
     gtk_window_set_title(GTK_WINDOW(window), title);
     gtk_container_border_width(GTK_CONTAINER(window), 10);
