@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-11-27 19:05:47 jcs>
+/* Time-stamp: <2006-05-22 23:18:47 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: fileselection.h,v 1.3 2005/11/28 16:25:34 jcsjcs Exp $
+|  $Id: fileselection.h,v 1.6 2006/05/22 15:12:45 jcsjcs Exp $
 */
 
 /***************************************************************************
@@ -39,9 +39,18 @@
 #ifndef _FILESELECTION_H
 #define _FILESELECTION_H
 
+#include "prefs.h"
+
 void create_add_files_dialog(void);
 void create_add_playlists_dialog(void);
 gchar *fileselection_get_cover_filename(void);
+gchar *fileselection_get_file_or_dir (const gchar *title,
+				      const gchar *cur_file,
+				      GtkFileChooserAction action);
+gchar *fileselection_select_script (const gchar *opath,
+				    const gchar *fallback,
+				    const gchar *title,
+				    const gchar *additional_text);
 
 /* dirbrowser */
 void dirbrowser_block (void);
