@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-09-18 22:59:43 jcs>
+/* Time-stamp: <2006-06-24 01:39:23 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 | 
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: info.h,v 1.11 2005/09/20 09:50:13 jcsjcs Exp $
+|  $Id: info.h,v 1.15 2006/06/23 16:46:12 jcsjcs Exp $
 */
 
 #ifndef __INFO_H__
@@ -35,6 +35,7 @@
 #endif
  
 #include <glib.h>
+#include "itdb.h"
 
 /* info window */
 void info_open_window (void);
@@ -50,15 +51,23 @@ void info_update_totals_view_space (void);
 /* statusbar stuff */
 #define STATUSBAR_TIMEOUT 4200
 void gtkpod_statusbar_init(void);
-void gtkpod_statusbar_message(const gchar *message);
-void gtkpod_statusbar_reset_timeout (void);
+void gtkpod_statusbar_message(const gchar *message, ...);
+void gtkpod_statusbar_timeout (guint timeout);
 void gtkpod_space_statusbar_init(void);
 void gtkpod_tracks_statusbar_init(void);
 void gtkpod_tracks_statusbar_update(void);
 
 /* space stuff */
 #define SPACE_TIMEOUT 4000   /* update interval in ms */
-void space_set_ipod_mount (const gchar *mp);
+void space_set_ipod_itdb (iTunesDB *itdb);
+iTunesDB *space_get_ipod_itdb (void);
 void space_data_update (void);
 gboolean ipod_connected (void);
+
+/* standard messages */
+void message_sb_no_itdb_selected (void);
+void message_sb_no_tracks_selected (void);
+void message_sb_no_playlist_selected (void);
+void message_sb_no_ipod_itdb_selected (void);
+
 #endif
