@@ -1,14 +1,16 @@
-/*
+/* Time-stamp: <2006-05-17 22:10:35 jcs>
+|
 |  Copyright (C) 2002 Corey Donohoe <atmos at atmos.org>
+|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
+|  Part of the gtkpod project.
+| 
+|  URL: http://www.gtkpod.org/
+|  URL: http://gtkpod.sourceforge.net/
 |
 |  SHA1 routine: David Puckett <niekze at yahoo.com>
 |  SHA1 implemented from FIPS-160 standard
 |  <http://www.itl.nist.gov/fipspubs/fip180-1.htm>
 |
-|  Part of the gtkpod project.
-|
-|  URL: http://gtkpod.sourceforge.net/
-|
 |  This program is free software; you can redistribute it and/or modify
 |  it under the terms of the GNU General Public License as published by
 |  the Free Software Foundation; either version 2 of the License, or
@@ -27,7 +29,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: md5.c,v 1.32 2004/10/01 16:11:47 jcsjcs Exp $
+|  $Id: md5.c,v 1.44 2006/06/25 23:29:08 freethinkerjim Exp $
 */
 
 #include <errno.h>
@@ -37,12 +39,15 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <glib.h>
 #include "charset.h"
 #include "md5.h"
 #include "file.h"
 #include "prefs.h"
 #include "misc.h"
-#include "support.h"
+#include "misc_track.h"
+#include "display.h"
+#include "display_itdb.h"
 
 
 typedef guint32 chunk;
@@ -87,12 +92,29 @@ static void little_endian(hblock * stupidblock, int blocks);
 #define NR_PATH_MAX_BLOCKS 4
 #define PATH_MAX_MD5 4096
 
-/**
- * filehash
- * a string hash of the files on the ipod already, or are already registered
- * to be copied on next export
- * */
-static GHashTable *filehash = NULL;
+/* Set up or destory the md5 hash table */
+void setup_md5()
+{
+    struct itdbs_head *itdbs_head;
+
+    g_return_if_fail (gtkpod_window);
+    itdbs_head = g_object_get_data (G_OBJECT (gtkpod_window),
+				    "itdbs_head");
+
+    /* gets called before itdbs are set up -> fail silently */
+    if (itdbs_head)
+    {
+	if (prefs_get_int("md5"))  /* MD5 hashing turned on */
+	{
+	    gp_md5_hash_tracks();
+	
+	    /* Display duplicates */
+	    gp_duplicate_remove(NULL, NULL);
+	}
+	else
+	    gp_md5_free_hash();
+    }
+}
 
 /**
  * get_filesize_for_file_descriptor - get the filesize on disk for the given
@@ -100,16 +122,16 @@ static GHashTable *filehash = NULL;
  * @fp - the filepointer we want the filesize for
  * Returns - the filesize in bytes
  */
-static int
+static guint32
 get_filesize_for_file_descriptor(FILE *fp)
 {
-    int result = 0;
+    off_t result = 0;
     struct stat stat_info;
     int file_no = fileno(fp);
 
     if((fstat(file_no, &stat_info) == 0))	/* returns 0 on success */
 	result = (int)stat_info.st_size;
-    return(result);
+    return (guint32)result;
 }
 
 /**
@@ -134,16 +156,17 @@ md5_hash_on_file(FILE * fp)
 
        if(fsize > 0)
        {
+	   guint32 fsize_normal;
 	   guint8 *hash = NULL;
 	   int bread = 0, x = 0, last = 0;
-	   gchar file_chunk[chunk_size + sizeof(int)];
+	   guchar file_chunk[chunk_size + sizeof(int)];
 
 	   /* allocate the digest we're returning */
-	   if((result = (gchar*)g_malloc0(sizeof(gchar) * 41)) == NULL)
-	       gtkpod_main_quit();	/* errno == ENOMEM */
+	   result = g_malloc0(sizeof(gchar) * 41);
 
 	   /* put filesize in the first 32 bits */
-	   memcpy(file_chunk, &fsize, sizeof(int));
+	   fsize_normal = GINT32_TO_LE (fsize);
+	   memcpy(file_chunk, &fsize_normal, sizeof(guint32));
 
 	   /* read chunk_size from fp */
 	   bread = fread(&file_chunk[sizeof(int)], sizeof(gchar),
@@ -175,20 +198,26 @@ md5_hash_on_file(FILE * fp)
 static gchar *
 md5_hash_track(Track * s)
 {
+   ExtraTrackData *etr;
    gchar *result = NULL;
-   gchar *filename = NULL;
+   gchar *filename;
 
-   if (s != NULL)
+   g_return_val_if_fail (s, NULL);
+   etr = s->userdata;
+   g_return_val_if_fail (etr, NULL);
+
+   if (etr->md5_hash != NULL)
    {
-      if (s->md5_hash != NULL)
-      {
-         result = g_strdup(s->md5_hash);
-      }
-      else if ((filename = get_track_name_on_disk(s)) != NULL)
-      {
-	 result = md5_hash_on_filename (filename, FALSE);
-         g_free(filename);
-      }
+       result = g_strdup(etr->md5_hash);
+   }
+   else
+   {
+       filename = get_file_name_from_source (s, SOURCE_PREFER_LOCAL);
+       if (filename)
+       {
+	   result = md5_hash_on_filename (filename, FALSE);
+	   g_free(filename);
+       }
    }
    return (result);
 }
@@ -224,16 +253,28 @@ gchar *md5_hash_on_filename (gchar *name, gboolean silent)
 
 
 /**
- * Free up the dynamically allocated memory in this table
+ * Free up the dynamically allocated memory in @itdb's hash table
  */
-void
-md5_unique_file_free(void)
+void md5_free_eitdb (ExtraiTunesDBData *eitdb)
 {
-   if (filehash)
-   {
-      g_hash_table_destroy(filehash);
-      filehash = NULL;
-   }
+    g_return_if_fail (eitdb);
+
+    if (eitdb->md5hash)
+    {
+	g_hash_table_destroy (eitdb->md5hash);
+	eitdb->md5hash = NULL;
+    }
+}
+
+/**
+ * Free up the dynamically allocated memory in @itdb's hash table
+ */
+void md5_free (iTunesDB *itdb)
+{
+    g_return_if_fail (itdb);
+    g_return_if_fail (itdb->userdata);
+
+    md5_free_eitdb (itdb->userdata);
 }
 
 /**
@@ -242,36 +283,48 @@ md5_unique_file_free(void)
  * is inserted into the hash.
  * Returns a pointer to the duplicate track.
  */
-Track *
-md5_track_exists_insert(Track * s)
+Track *md5_track_exists_insert (iTunesDB *itdb, Track * s)
 {
-   gchar *val = NULL;
-   Track *track = NULL;
+    ExtraiTunesDBData *eitdb;
+    ExtraTrackData *etr;
+    gchar *val = NULL;
+    Track *track = NULL;
 
-   if (prefs_get_md5tracks ())
-   {
-       if (filehash == NULL)
-       {
-	   filehash = g_hash_table_new_full(g_str_hash, g_str_equal,
-					    g_free, NULL);
-       }
-       val = md5_hash_track(s);
-       if (val != NULL)
-       {
-	   if ((track = g_hash_table_lookup(filehash, val)))
-	   {
-	       g_free(val);
-	   }
-	   else                   /* if it doesn't exist we register it in the
-				     hash */
-	   {
-	       C_FREE (s->md5_hash);
-	       s->md5_hash = g_strdup(val);
-	       g_hash_table_insert(filehash, val, s);
-	   }
-       }
-   }
-   return track;
+    g_return_val_if_fail (itdb, NULL);
+    eitdb = itdb->userdata;
+    g_return_val_if_fail (eitdb, NULL);
+
+    g_return_val_if_fail (s, NULL);
+    etr = s->userdata;
+    g_return_val_if_fail (etr, NULL);
+
+    if (prefs_get_int("md5"))
+    {
+	if (eitdb->md5hash == NULL)
+	{
+	    eitdb->md5hash = g_hash_table_new_full(g_str_hash,
+						   g_str_equal,
+						   g_free, NULL);
+	}
+	val = md5_hash_track (s);
+	if (val != NULL)
+	{
+	    track = g_hash_table_lookup (eitdb->md5hash, val);
+	    if (track)
+	    {
+		g_free(val);
+	    }
+	    else
+	    {   /* if it doesn't exist we register it in the hash */
+		g_free (etr->md5_hash);
+		etr->md5_hash = g_strdup (val);
+		/* val is used in the next line -- we don't have to
+		 * g_free() it */
+		g_hash_table_insert (eitdb->md5hash, val, s);
+	    }
+	}
+    }
+    return track;
 }
 
 /**
@@ -279,21 +332,25 @@ md5_track_exists_insert(Track * s)
  * @s - the Track we want to know about.
  * Returns a pointer to the duplicate track.
  */
-Track *
-md5_track_exists(Track * s)
+Track *md5_track_exists (iTunesDB *itdb, Track *s)
 {
-   Track *track = NULL;
+    ExtraiTunesDBData *eitdb;
+    Track *track = NULL;
 
-   if (prefs_get_md5tracks() && filehash)
-   {
-       gchar *val = md5_hash_track(s);
-       if (val != NULL)
-       {
-	   track = g_hash_table_lookup(filehash, val);
-	   g_free (val);
-       }
-   }
-   return track;
+    g_return_val_if_fail (itdb, NULL);
+    eitdb = itdb->userdata;
+    g_return_val_if_fail (eitdb, NULL);
+
+    if (prefs_get_int("md5") && eitdb->md5hash)
+    {
+	gchar *val = md5_hash_track (s);
+	if (val)
+	{
+	    track = g_hash_table_lookup (eitdb->md5hash, val);
+	    g_free (val);
+	}
+    }
+    return track;
 }
 
 /**
@@ -302,19 +359,26 @@ md5_track_exists(Track * s)
  * Returns a pointer to the duplicate track using md5 for
  * identification. If md5 checksums are off, NULL is returned.
  */
-Track *md5_file_exists (gchar *file, gboolean silent)
+Track *md5_file_exists (iTunesDB *itdb, gchar *file, gboolean silent)
 {
+    ExtraiTunesDBData *eitdb;
     Track *track = NULL;
-    if (prefs_get_md5tracks() && filehash && file)
+
+    g_return_val_if_fail (file, NULL);
+    g_return_val_if_fail (itdb, NULL);
+    eitdb = itdb->userdata;
+    g_return_val_if_fail (eitdb, NULL);
+
+    if (prefs_get_int("md5") && eitdb->md5hash)
     {
 	gchar *val = md5_hash_on_filename (file, silent);
 	if (val)
 	{
-	   track = g_hash_table_lookup(filehash, val);
-	   g_free (val);
-       }
-   }
-   return track;
+	    track = g_hash_table_lookup (eitdb->md5hash, val);
+	    g_free (val);
+	}
+    }
+    return track;
 }
 
 
@@ -324,12 +388,19 @@ Track *md5_file_exists (gchar *file, gboolean silent)
  * Returns a pointer to the duplicate track using md5 for
  * identification. If md5 checksums are off, NULL is returned.
  */
-Track *md5_md5_exists (gchar *md5)
+Track *md5_md5_exists (iTunesDB *itdb, gchar *md5)
 {
+    ExtraiTunesDBData *eitdb;
     Track *track = NULL;
-    if (prefs_get_md5tracks() && filehash && md5)
+
+    g_return_val_if_fail (md5, NULL);
+    g_return_val_if_fail (itdb, NULL);
+    eitdb = itdb->userdata;
+    g_return_val_if_fail (eitdb, NULL);
+
+    if (prefs_get_int("md5") && eitdb->md5hash)
     {
-	track = g_hash_table_lookup(filehash, md5);
+	track = g_hash_table_lookup (eitdb->md5hash, md5);
     }
     return track;
 }
@@ -338,20 +409,28 @@ Track *md5_md5_exists (gchar *md5)
  * Free the specified track from the ipod's unique file hash
  * @s - The Track that's being freed from the ipod
  */
-void
-md5_track_removed(Track * s)
+void md5_track_remove (Track *s)
 {
-    gchar *val = NULL;
-    Track *track = NULL;
+    ExtraiTunesDBData *eitdb;
 
-    if ((prefs_get_md5tracks ()) && (filehash) && (s) && (val = md5_hash_track(s)))
+    g_return_if_fail (s);
+    g_return_if_fail (s->itdb);
+    eitdb = s->itdb->userdata;
+    g_return_if_fail (eitdb);
+
+    if (prefs_get_int("md5") && eitdb->md5hash)
     {
-	if ((track = g_hash_table_lookup(filehash, val)))
+	gchar *val = md5_hash_track (s);
+	if (val)
 	{
-	    if (track == s) /* only remove if it's the same track */
-		g_hash_table_remove(filehash, val);
+	    Track *track = g_hash_table_lookup (eitdb->md5hash, val);
+	    if (track)
+	    {
+		if (track == s) /* only remove if it's the same track */
+		    g_hash_table_remove (eitdb->md5hash, val);
+	    }
+	    g_free(val);
 	}
-	g_free(val);
     }
 }
 
@@ -369,14 +448,10 @@ sha1_hash(const guint8 * text, guint32 len)
    guint8 *digest;
    sha1 *message;
 
-   if((digest = (guint8 *) g_malloc0(sizeof(guint8) * 21)) == NULL)
-       gtkpod_main_quit();	/* errno == ENOMEM */
-   if((message = (sha1 *) g_malloc0(sizeof(sha1))) == NULL)
-       gtkpod_main_quit();	/* errno == ENOMEM */
-   if((message->blockdata = (block *) g_malloc0(sizeof(block))) == NULL)
-       gtkpod_main_quit();	/* errno == ENOMEM */
-   if((message->H = (hblock *) g_malloc(sizeof(hblock))) == NULL)
-       gtkpod_main_quit();	/* errno == ENOMEM */
+   digest = g_malloc0(sizeof(guint8) * 21);
+   message = g_malloc0(sizeof(sha1));
+   message->blockdata = g_malloc0(sizeof(block));
+   message->H = g_malloc(sizeof(hblock));
 
    message->H->chunkblock[0] = 0x67452301;
    message->H->chunkblock[1] = 0xefcdab89;
@@ -489,3 +564,5 @@ little_endian(hblock * stupidblock, int blocks)
    }
 }
 #endif
+
+
