@@ -1,5 +1,5 @@
 /* -*- coding: utf-8; -*-
-|  Time-stamp: <2005-12-11 00:15:39 jcs>
+|  Time-stamp: <2006-06-28 01:21:08 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -25,13 +25,14 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: misc.c,v 1.192 2005/12/10 17:04:28 jcsjcs Exp $
+|  $Id: misc.c,v 1.213 2006/06/27 16:30:02 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
+#include <errno.h>
 #include <gtk/gtk.h>
 #include <stdlib.h>
 #include <string.h>
@@ -142,8 +143,10 @@ Steve Jay: use statvfs() instead of df (better portability, faster)\n"),
 Christoph Kunz: address compatibility issues when writing id3v2.4 type mp3 tags\n"),
 		       "\n",
 		       _("\
-James Ligget: replacement of old GTK file selection dialogs with new GTK filechooser dialogs\n"),
-		       "\n",
+James Liggett:\n\
+     replacement of old GTK file selection dialogs with new GTK filechooser dialogs\n\
+     refactored user preferences system.\n"),
+                       "\n",
 		       _("\
 Daniel Kercher: sync scripts for abook and webcalendar\n"),
 		       "\n",
@@ -316,26 +319,6 @@ parse_tracks_from_string(gchar **s, Track **track)
 
 
 
-void cleanup_backup_and_extended_files (void)
-{
-  gchar *cfgdir;
-
-  cfgdir = prefs_get_cfgdir ();
-  /* in offline mode, there are no backup files! */
-  if (cfgdir && !prefs_get_offline ())
-    {
-      gchar *cfe = g_build_filename (cfgdir, "iTunesDB.ext", NULL);
-      if (!prefs_get_write_extended_info ())
-	/* delete extended info file from computer */
-	if (g_file_test (cfe, G_FILE_TEST_EXISTS))
-	  if (remove (cfe) != 0)
-	    gtkpod_warning (_("Could not delete backup file: \"%s\"\n"), cfe);
-      g_free (cfe);
-    }
-  C_FREE (cfgdir);
-}
-
-
 /* Duplicate a GList (shallow copy) */
 GList *glist_duplicate (GList *list)
 {
@@ -350,160 +333,29 @@ GList *glist_duplicate (GList *list)
 }
 
 
-/***************************************************************************
- * Mount Calls
- *
- **************************************************************************/
-/**
- * mount_ipod - attempt to mount the ipod to prefs_get_ipod_mount() This
- * does not check prefs to see if the current prefs want gtkpod itself to
- * mount the ipod drive, that should be checked before making this call.
- */
-#include <sys/param.h>   /* seems to be needed for FreeBSD 5.4
-			    otherwise sys/mount.h throws an error */
-#include <sys/mount.h>
-#include <errno.h>
-#include <stdio.h>
-
-
-static gchar *getdevicename(const gchar *mount)
-{
-    if(mount) {
-        gchar device[256], point[256];
-        FILE *fp = fopen("/proc/mounts","r");
-        if (!fp)
-            return NULL;
-        do {
-            fscanf(fp, "%255s %255s %*s %*s %*s %*s", device, point);
-            if( strcmp(mount,point) == 0 ) {
-                fclose(fp);
-                return strdup(device);
-            }
-        } while(!feof(fp));
-        fclose(fp);
-    }
-    return NULL;
-}
-
-
-
-void
-mount_ipod(void)
-{
-    const gchar *str = prefs_get_ipod_mount ();
-    if(str)
-    {
-	pid_t pid, tpid;
-	int status;
-
-	pid = fork ();
-	switch (pid)
-	{
-	    case 0: /* child */
-		execl(MOUNT_BIN, "mount", str, NULL);
-		exit (1);
-		break;
-	    case -1: /* parent and error */
-		break;
-	    default: /* parent -- let's wait for the child to terminate */
-		tpid = waitpid (pid, &status, 0);
-		/* we could evaluate tpid and status now */
-		break;
-	}
-    }
-}
-
-/**
- * unmount_ipod - attempt to eject the ipod from prefs_get_ipod_mount()
- * This does not check prefs to see if the current prefs want gtkpod itself
- * to eject the ipod drive, that should be checked before making this
- * call.
- */
-void
-unmount_ipod(void)
-{
-    const gchar *mp = prefs_get_ipod_mount ();
-    if (mp)
-    {
-	pid_t pid, tpid;
-	int status;
-	gchar *eject_bin;
-	gchar *ipod_device = getdevicename (mp);
-
-	/* umount */
-	pid = fork ();
-	switch (pid)
-	{
-	    case 0: /* child */
-		execl (UMOUNT_BIN, "umount", mp, NULL);
-		exit (1); /* this is only reached in case of an error */
-		break;
-	    case -1: /* parent and error */
-		break;
-	    default: /* parent -- let's wait for the child to terminate */
-		tpid = waitpid (pid, &status, 0);
-		if (status != 0)
-		{
-		   	gchar *buf;
-		    gchar *str_utf8 = charset_to_utf8 (mp);
-		    GtkWidget *dialog;
-		    if (ipod_device)
-			buf = g_strdup_printf (
-			    _("Unmounting of '%s' (%s) unsuccessful."),
-			    str_utf8, ipod_device);
-		    else
-			buf = g_strdup_printf (
-			    _("Unmounting of '%s' unsuccessful."),
-			    str_utf8);
-		    g_free (str_utf8);
-		    dialog = gtk_message_dialog_new (
-			GTK_WINDOW (gtkpod_window),
-			GTK_DIALOG_DESTROY_WITH_PARENT,
-			GTK_MESSAGE_WARNING,
-			GTK_BUTTONS_OK,
-			buf);
-		    gtk_dialog_run (GTK_DIALOG (dialog));
-		    gtk_widget_destroy (dialog);
-		    g_free (buf);
-		}
-		break;
-	}
-	/* eject -- only if ipod_device and are EJECT_BIN available */
-	eject_bin = which ("eject");
-	if (ipod_device && eject_bin)
-	{
-	    pid = fork ();
-	    switch (pid)
-	    {
-	    case 0: /* child */
-		execl (eject_bin, "eject", ipod_device, NULL);
-		exit (1); /* this is only reached in case of an error */
-		break;
-	    case -1: /* parent and error */
-		break;
-	    default: /* parent -- let's wait for the child to terminate */
-		tpid = waitpid (pid, &status, 0);
-		if (status != 0)
-		{
-		    /* fail silently */
-		}
-		break;
-	    }
-	    g_free (eject_bin);
-	}
-	g_free (ipod_device);
-    }
-}
-
-
 /***************************************************************************
  * gtkpod.in,out calls
  *
  **************************************************************************/
 
-/* tries to call "/bin/sh @script" */
-static void do_script (gchar *script)
+/* tries to call "/bin/sh @script" with command line options */
+static void do_script (const gchar *script, va_list args)
 {
+    char *str;
+    char **argv;
+    GPtrArray *ptra = g_ptr_array_sized_new (10);
+
+    /* prepend args with "sh" and the name of the script */
+    g_ptr_array_add (ptra, "sh");
+    g_ptr_array_add (ptra, (gpointer)script);
+    /* add remaining args */
+    while ((str = va_arg (args, char *)))
+    {
+	g_ptr_array_add (ptra, str);
+    }
+    g_ptr_array_add (ptra, NULL);
+    argv = (char **)g_ptr_array_free (ptra, FALSE);
+
     if (script)
     {
 	pid_t pid, tpid;
@@ -513,7 +365,7 @@ static void do_script (gchar *script)
 	switch (pid)
 	{
 	case 0: /* child */
-	    execl("/bin/sh", "sh", script, NULL);
+	    execv("/bin/sh", argv);
 	    exit(0);
 	break;
 	case -1: /* parent and error */
@@ -524,23 +376,29 @@ static void do_script (gchar *script)
 	    break;
 	}
     }
+    g_free (argv);
 }
 
 
 /* tries to execute "/bin/sh ~/.gtkpod/@script" or
- * "/bin/sh /etc/gtkpod/@script" if the former does not exist */
-void call_script (gchar *script)
+ * "/bin/sh /etc/gtkpod/@script" if the former does not exist. This
+ * function accepts command line arguments that must be terminated by
+ * NULL. */
+void call_script (gchar *script, ...)
 {
     gchar *cfgdir;
+    va_list args;
     gchar *file;
 
     if (!script) return;
 
     cfgdir =  prefs_get_cfgdir ();
     file = g_build_filename (cfgdir, script, NULL);
+
+    va_start (args, script);
     if (g_file_test (file, G_FILE_TEST_EXISTS))
     {
-	do_script (file);
+	do_script (file, args);
     }
     else
     {
@@ -548,11 +406,13 @@ void call_script (gchar *script)
 	file = g_build_filename ("/etc/gtkpod/", script, NULL);
 	if (g_file_test (file, G_FILE_TEST_EXISTS))
 	{
-	    do_script (file);
+	    do_script (file, args);
 	}
     }
-    C_FREE (file);
-    C_FREE (cfgdir);
+    va_end (args);
+
+    g_free (file);
+    g_free (cfgdir);
 }
 
 
@@ -561,7 +421,7 @@ void call_script (gchar *script)
  * depending on prefs settings */
 gint compare_string (const gchar *str1, const gchar *str2)
 {
-    if (prefs_get_case_sensitive ())
+    if (prefs_get_int("case_sensitive"))
 	return strcmp (str1, str2);
     else
 	return compare_string_case_insensitive (str1, str2);
@@ -581,7 +441,9 @@ static GList *csfk_list = NULL;
 void compare_string_fuzzy_generate_keys (void)
 {
     GList *gl;
-    gint i;
+    GList *sort_ign_strings;
+    GList *current;
+
     /* remove old keys */
     for (gl=csfk_list; gl; gl=gl->next)
     {
@@ -594,33 +456,25 @@ void compare_string_fuzzy_generate_keys (void)
     csfk_list = NULL;
 
     /* create new keys */
-    for (i=0; ;++i)
+    sort_ign_strings = prefs_get_list("sort_ign_string_");
+    current = sort_ign_strings;
+    while (current)
     {
-	gchar *buf = g_strdup_printf ("sort_ign_string_%d", i);
-	gchar *str = prefs_get_string (buf);
+	gchar *str = current->data;
 	struct csfk *csfk;
 	gchar *tempStr;
-
-	g_free (buf);
-
-	/* end loop if no string is set or if the the string
-	 * corresponds to the end marker */
-	if (!str)  break;  
-	if (strcmp (str, SORT_IGNORE_STRINGS_END) == 0)
-	{
-	    g_free (str);
-	    break;
-	}
+	
+	current = g_list_next(current);
 
 	csfk = g_malloc (sizeof (struct csfk));
 	tempStr = g_utf8_casefold (str, -1 );
 	csfk->length = g_utf8_strlen (tempStr, -1 );
 	csfk->key = g_utf8_collate_key (tempStr, -1 );
 	g_free (tempStr);
-	g_free (str);
 
 	csfk_list = g_list_append (csfk_list, csfk);
     }
+    prefs_free_list(sort_ign_strings);
 }
 
 
@@ -687,7 +541,7 @@ gint compare_string_fuzzy (const gchar *str1, const gchar *str2)
 	g_free (tempStr);
     }
 
-    if (prefs_get_case_sensitive ())
+    if (prefs_get_int("case_sensitive"))
 	result = strcmp(pstr1, pstr2);
     else
     	result = g_utf8_collate(pcleanStr1, pcleanStr2);
@@ -804,7 +658,7 @@ void option_set_radio_button (GladeXML *win_xml,
 	fprintf (stderr, "Programming error: wnum > num (%d,%d,%s)\n",
 		 wnum, num, prefs_string);
 	/* set to reasonable default value */
-	prefs_set_int_value (prefs_string, 0);
+	prefs_set_int (prefs_string, 0);
 	wnum = 0;
     }
     w = gtkpod_xml_get_widget (win_xml, widgets[wnum]);
@@ -838,7 +692,7 @@ gint option_get_radio_button (GladeXML *win_xml,
 	/* set reasonable default */
 	i=0;
     }
-    prefs_set_int_value (prefs_string, i);
+    prefs_set_int (prefs_string, i);
     return i;
 }
 
@@ -870,7 +724,7 @@ void option_get_folder (GtkFileChooser *fc,
     g_return_if_fail (fc && prefs_string);
 
     folder = gtk_file_chooser_get_current_folder (fc);
-    prefs_set_string_value (prefs_string, folder);
+    prefs_set_string (prefs_string, folder);
 
     if (value) *value = folder;
     else       g_free (folder);
@@ -904,7 +758,7 @@ void option_get_filename (GtkFileChooser *fc,
     g_return_if_fail (fc && prefs_string);
 
     filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(fc));
-    prefs_set_string_value (prefs_string, filename);
+    prefs_set_string (prefs_string, filename);
 
     if (value) *value = filename;
     else       g_free (filename);
@@ -952,7 +806,7 @@ void option_get_string (GladeXML *win_xml,
     if (entry)
     {
 	const gchar *str = gtk_entry_get_text (GTK_ENTRY (entry));
-	prefs_set_string_value (name, str);
+	prefs_set_string (name, str);
 	if (value) *value = g_strdup (str);
     }
 }
@@ -995,7 +849,7 @@ gboolean option_get_toggle_button (GladeXML *win_xml,
     if (button)
     {
 	active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(button));
-	prefs_set_int_value (name, active);
+	prefs_set_int (name, active);
     }
     return active;
 }
@@ -1064,15 +918,25 @@ static gchar *fix_path(gchar *orig)
 static gchar *select_template (Track *track, const gchar *p)
 {
     gchar **templates, **tplp;
-    gchar *tname, *ext = NULL;
+    gchar *ext = NULL;
+    const gchar *tname;
     gchar *result;
     ExtraTrackData *etr;
 
     g_return_val_if_fail (track, strdup (""));
     etr = track->userdata;
     g_return_val_if_fail (etr, strdup (""));
-    tname = get_file_name (track);
-    if (!tname) return (NULL);         /* this should not happen... */
+    if (etr->pc_path_locale && strlen(etr->pc_path_locale))
+	tname = etr->pc_path_locale;
+    else
+	tname = track->ipod_path;
+    if (!tname)
+    {   /* this should not happen... */
+	gchar *buf = get_track_info (track, TRUE);
+	gtkpod_warning (_("Could not process '%s' (no filename available)"),
+			buf);
+	g_free (buf);
+    }
     ext = strrchr (tname, '.');        /* pointer to filename extension */
 
     templates = g_strsplit (p, ";", 0);
@@ -1085,7 +949,7 @@ static gchar *select_template (Track *track, const gchar *p)
 	    if (etr->pc_path_locale && strlen(etr->pc_path_locale))  break;
 	}
 	else if (strrchr (*tplp, '.') == NULL)
-	{   /* this templlate does not have an extension and therefore
+	{   /* this template does not have an extension and therefore
 	     * matches */
 	    if (ext)
 	    {   /* if we have an extension, add it */
@@ -1097,7 +961,8 @@ static gchar *select_template (Track *track, const gchar *p)
 	}
 	else if (ext && (strlen (*tplp) >= strlen (ext)))
 	{  /* this template is valid if the extensions match */
-	    if (strcmp (&((*tplp)[strlen (*tplp) - strlen (ext)]), ext) == 0)
+	    if (strcasecmp (&((*tplp)[strlen (*tplp) - strlen (ext)]),
+			    ext) == 0)
 		break;
 	}
 	++tplp;
@@ -1297,7 +1162,7 @@ gchar *get_string_from_full_template (Track *track,
 
     if (!template)
     {
-	gchar *fn = get_file_name (track);
+	gchar *fn = get_file_name_from_source (track, SOURCE_PREFER_LOCAL);
 	gtkpod_warning (_("Template ('%s') does not match file type '%s'\n"), full_template, fn ? fn:"");
 	g_free (fn);
 	return NULL;
@@ -1413,9 +1278,15 @@ gboolean mkdirhierfile(const gchar *filename)
  */
 gchar *convert_filename (const gchar *filename)
 {
+  if (filename)
+  {    
     if (strncmp ("~/", filename, 2) == 0)
-	 return g_build_filename (g_get_home_dir(), filename+2, NULL);
-    else return g_strdup (filename);
+      return g_build_filename (g_get_home_dir(), filename+2, NULL);
+    else 
+      return g_strdup (filename);
+  }
+  
+  return NULL;
 }
 
 
@@ -1435,3 +1306,271 @@ GtkWidget *gtkpod_xml_get_widget (GladeXML *xml, const gchar *name)
 
     return w;
 }
+
+
+  gchar    *ipod_mount;     /* mount point of iPod */
+/* ------------------------------------------------------------
+ *
+ *        Helper functions for pref keys
+ *
+ * ------------------------------------------------------------ */
+
+
+/**
+ * Helper function to construct prefs key for itdb,
+ * e.g. itdb_1_mountpoint...
+ * gfree() after use
+ *
+ **/
+gchar *get_itdb_prefs_key (gint index, const gchar *subkey)
+{
+    g_return_val_if_fail (subkey, NULL);
+
+    return g_strdup_printf ("itdb_%d_%s", index, subkey);
+}
+
+
+/**
+ * Helper function to construct prefs key for playlists,
+ * e.g. itdb_1_playlist_xxxxxxxxxxx_syncmode...
+ *
+ * gfree() after use
+ **/
+gchar *get_playlist_prefs_key (gint index,
+			       Playlist *pl, const gchar *subkey)
+{
+    g_return_val_if_fail (pl, NULL);
+    g_return_val_if_fail (subkey, NULL);
+
+    return g_strdup_printf ("itdb_%d_playlist_%llu_%s",
+			    index, (unsigned long long)pl->id, subkey);
+}
+
+
+/**
+ * Helper function to retrieve the index number of @itdb needed to
+ * construct any keys (see above)
+ **/
+gint get_itdb_index (iTunesDB *itdb)
+{
+	struct itdbs_head *itdbs_head;
+
+	itdbs_head = gp_get_itdbs_head (gtkpod_window);
+	g_return_val_if_fail (itdbs_head, 0);
+	
+	return g_list_index (itdbs_head->itdbs, itdb);
+}
+
+
+/**
+ * Helper function to retrieve a string prefs entry for @itdb.
+ * 
+ * gfree() after use
+ **/
+gchar *get_itdb_prefs_string (iTunesDB *itdb, const gchar *subkey)
+{
+    gchar *key, *value;
+
+    g_return_val_if_fail (itdb, NULL);
+    g_return_val_if_fail (subkey, NULL);
+
+    key = get_itdb_prefs_key (get_itdb_index (itdb), subkey);
+    value = prefs_get_string (key);
+    g_free (key);
+
+    return value;
+}
+
+
+/**
+ * Helper function to retrieve a string prefs entry for @playlist.
+ * 
+ **/
+gchar *get_playlist_prefs_string (Playlist *playlist, const gchar *subkey)
+{
+    gchar *key, *value;
+
+    g_return_val_if_fail (playlist, 0);
+    g_return_val_if_fail (subkey, 0);
+
+    key = get_playlist_prefs_key (get_itdb_index (playlist->itdb),
+				  playlist, subkey);
+    value = prefs_get_string (key);
+    g_free (key);
+
+    return value;
+}
+
+/**
+ * Helper function to retrieve an int prefs entry for @itdb.
+ * 
+ **/
+gint get_itdb_prefs_int (iTunesDB *itdb, const gchar *subkey)
+{
+    gchar *key;
+    gint value;
+
+    g_return_val_if_fail (itdb, 0);
+    g_return_val_if_fail (subkey, 0);
+
+    key = get_itdb_prefs_key (get_itdb_index (itdb), subkey);
+    value = prefs_get_int (key);
+    g_free (key);
+
+    return value;
+}
+
+/**
+ * Helper function to retrieve an int prefs entry for @playlist.
+ * 
+ **/
+gint get_playlist_prefs_int (Playlist *playlist, const gchar *subkey)
+{
+    gchar *key;
+    gint value;
+
+    g_return_val_if_fail (playlist, 0);
+    g_return_val_if_fail (subkey, 0);
+
+    key = get_playlist_prefs_key (get_itdb_index (playlist->itdb),
+				  playlist, subkey);
+    value = prefs_get_int (key);
+    g_free (key);
+
+    return value;
+}
+
+/**
+ * Helper function to retrieve a string prefs entry for @itdb.
+ *
+ * Returns TRUE if the key was actually set in the prefs. 
+ * 
+ * gfree() after use
+ **/
+gboolean get_itdb_prefs_string_value (iTunesDB *itdb, const gchar *subkey,
+				      gchar **value)
+{
+    gchar *key;
+    gboolean result;
+
+    g_return_val_if_fail (itdb, FALSE);
+    g_return_val_if_fail (subkey, FALSE);
+
+    key = get_itdb_prefs_key (get_itdb_index (itdb), subkey);
+    result = prefs_get_string_value (key, value);
+    g_free (key);
+
+    return result;
+}
+
+
+/**
+ * Helper function to retrieve an in prefs entry for @itdb.
+ * 
+ **/
+gboolean get_itdb_prefs_int_value (iTunesDB *itdb, const gchar *subkey,
+				   gint *value)
+{
+    gchar *key;
+    gboolean result;
+
+    g_return_val_if_fail (itdb, FALSE);
+    g_return_val_if_fail (subkey, FALSE);
+
+    key = get_itdb_prefs_key (get_itdb_index (itdb), subkey);
+    result = prefs_get_int_value (key, value);
+    g_free (key);
+
+    return result;
+}
+
+
+/**
+ * Helper function to set a string prefs entry for @itdb.
+ * 
+ * gfree() after use
+ **/
+void set_itdb_prefs_string (iTunesDB *itdb,
+			    const gchar *subkey, const gchar *value)
+{
+    gchar *key;
+
+    g_return_if_fail (itdb);
+    g_return_if_fail (subkey);
+
+    key = get_itdb_prefs_key (get_itdb_index (itdb), subkey);
+    prefs_set_string (key, value);
+    g_free (key);
+}
+
+
+/**
+ * Helper function to set a string prefs entry for @itdb.
+ * 
+ * gfree() after use
+ **/
+void set_itdb_index_prefs_string (gint index,
+				  const gchar *subkey, const gchar *value)
+{
+    gchar *key;
+
+    g_return_if_fail (subkey);
+
+    key = get_itdb_prefs_key (index, subkey);
+    prefs_set_string (key, value);
+    g_free (key);
+}
+
+
+/**
+ * Helper function to set an in prefs entry for @itdb.
+ * 
+ **/
+void set_itdb_prefs_int (iTunesDB *itdb, const gchar *subkey, gint value)
+{
+    gchar *key;
+
+    g_return_if_fail (itdb);
+    g_return_if_fail (subkey);
+
+    key = get_itdb_prefs_key (get_itdb_index (itdb), subkey);
+    prefs_set_int (key, value);
+    g_free (key);
+}
+
+
+/**
+ * Helper function to set an in prefs entry for @itdb.
+ * 
+ **/
+void set_itdb_index_prefs_int (gint index,
+			       const gchar *subkey, gint value)
+{
+    gchar *key;
+
+    g_return_if_fail (subkey);
+
+    key = get_itdb_prefs_key (index, subkey);
+    prefs_set_int (key, value);
+    g_free (key);
+}
+
+/* Sorting gets disabled if either tmp disable sort or
+ * block display are true */
+gboolean sorting_disabled()
+{
+    return (prefs_get_int("block_display") || 
+	    prefs_get_int("tmp_disable_sort"));
+}
+
+/* retrieve offline mode from itdb (convenience function) */
+gboolean get_offline (iTunesDB *itdb)
+{
+    ExtraiTunesDBData *eitdb;
+
+    g_return_val_if_fail (itdb, FALSE);
+    eitdb = itdb->userdata;
+    g_return_val_if_fail (eitdb, FALSE);
+
+    return eitdb->offline;
+}
