@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-12-11 00:15:38 jcs>
+/* Time-stamp: <2006-06-25 00:25:35 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: misc.h,v 1.107 2005/12/10 17:04:29 jcsjcs Exp $
+|  $Id: misc.h,v 1.119 2006/06/24 15:39:22 jcsjcs Exp $
 */
 
 #ifndef __MISC_H__
@@ -68,6 +68,27 @@ extern gboolean widgets_blocked;
 /* full path to 'gtkpod.glade', defined in main.c */
 extern gchar *xml_file;
 
+/* Some symbols not necessarily defined */
+
+/* Introduced in 2.6 */
+#ifndef GTK_STOCK_DISCONNECT
+#define GTK_STOCK_DISCONNECT GTK_STOCK_GOTO_TOP
+#define GTK_STOCK_CONNECT GTK_STOCK_GOTO_BOTTOM
+#endif
+#ifndef GTK_STOCK_EDIT
+#define GTK_STOCK_EDIT NULL
+#endif
+#ifndef G_IS_DIR_SEPARATOR
+#define G_IS_DIR_SEPARATOR(c) (c == G_DIR_SEPARATOR)
+#endif
+/* SEMIBOLD was introduced with version 2.8 */
+#ifndef PANGO_WEIGHT_SEMIBOLD
+#define PANGO_WEIGHT_SEMIBOLD 600
+#endif
+
+
+
+
 gchar *get_user_string (gchar *title, gchar *message, gchar *dflt,
 			gchar *opt_msg, gboolean *opt_state);
 Playlist *add_new_pl_user_name (iTunesDB *itdb, gchar *dflt, gint32 pos);
@@ -82,9 +103,7 @@ gint get_sort_tab_number (gchar *text);
 void open_about_window (void);
 void close_about_window (void);
 gboolean parse_tracks_from_string(gchar **s, Track **track);
-void cleanup_backup_and_extended_files (void);
 gboolean gtkpod_main_quit(void);
-void gtkpod_main_window_set_active(gboolean active);
 
 T_item TM_to_T (TM_item sm);
 T_item ST_to_T (ST_CAT_item st);
@@ -93,9 +112,6 @@ const gchar *get_tm_tooltip (TM_item tm);
 const gchar *get_t_string (T_item t);
 const gchar *get_t_tooltip (T_item t);
 
-gchar *get_track_info (Track *track, gboolean prefer_filename);
-
-void ipod_directories_head (const gchar *mountpoint);
 void delete_playlist_head (DeleteAction deleteaction);
 void delete_track_head (DeleteAction deleteaction);
 void delete_entry_head (gint entry_inst, DeleteAction deleteaction);
@@ -103,16 +119,16 @@ void delete_entry_head (gint entry_inst, DeleteAction deleteaction);
 void delete_populate_settings (struct DeleteData *dd,
 			       gchar **label, gchar **title,
 			       gboolean *confirm_again,
-			       ConfHandlerOpt *confirm_again_handler,
+			       gchar **confirm_again_key,
 			       GString **str);
 
 void block_widgets (void);
 void release_widgets (void);
 void update_blocked_widget (GtkWidget *w, gboolean sens);
 
-void mount_ipod(void);
-void unmount_ipod(void);
-void call_script (gchar *script);
+/*void mount_ipod(void);
+  void unmount_ipod(void);*/
+void call_script (gchar *script, ...);
 
 void delete_track_ok (struct DeleteData *dd);
 
@@ -212,4 +228,25 @@ GList *glist_duplicate (GList *list);
 gchar *which (const gchar *exe);
 
 GtkWidget *gtkpod_xml_get_widget (GladeXML *xml, const gchar *name);
+
+gchar *get_itdb_prefs_key (gint index, const gchar *subkey);
+gchar *get_playlist_prefs_key (gint index, Playlist *pl, const gchar *subkey);
+gint get_itdb_index (iTunesDB *itdb);
+gchar *get_itdb_prefs_string (iTunesDB *itdb, const gchar *subkey);
+gchar *get_playlist_prefs_string (Playlist *playlist, const gchar *subkey);
+gint get_itdb_prefs_int (iTunesDB *itdb, const gchar *subkey);
+gint get_playlist_prefs_int (Playlist *playlist, const gchar *subkey);
+gboolean get_itdb_prefs_string_value (iTunesDB *itdb, const gchar *subkey,
+				      gchar **value);
+gboolean get_itdb_prefs_int_value (iTunesDB *itdb, const gchar *subkey,
+				   gint *value);
+void set_itdb_prefs_string (iTunesDB *itdb,
+			    const gchar *subkey, const gchar *value);
+void set_itdb_index_prefs_string (gint index,
+				  const gchar *subkey, const gchar *value);
+void set_itdb_prefs_int (iTunesDB *itdb, const gchar *subkey, gint value);
+void set_itdb_index_prefs_int (gint index,
+			       const gchar *subkey, gint value);
+gboolean get_offline (iTunesDB *itdb);
+gboolean sorting_disabled();
 #endif
