@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-12-09 23:52:06 jcs>
+/* Time-stamp: <2006-09-24 20:49:39 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: misc_confirm.c,v 1.25 2005/12/09 14:55:34 jcsjcs Exp $
+|  $Id: misc_confirm.c,v 1.44 2006/09/24 12:16:26 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -40,7 +40,7 @@
 #include "prefs.h"
 #include "info.h"
 
-#ifdef HAVE_statvfs
+#ifdef HAVE_STATVFS
 #include <sys/types.h>
 #include <sys/statvfs.h>
 #endif
@@ -74,7 +74,7 @@ void gtkpod_warning (const gchar *format, ...)
 			 NULL, 0, NULL,       /* option 1 */
 			 NULL, 0, NULL,       /* option 2 */
 			 TRUE,                /* gboolean confirm_again, */
-			 NULL, /* ConfHandlerOpt confirm_again_handler, */
+			 NULL, /* ConfHandlerOpt confirm_again_key, */
 			 CONF_NULL_HANDLER,   /* ConfHandler ok_handler,*/
 			 NULL,                /* don't show "Apply" */
 			 NULL,                /* cancel_handler,*/
@@ -95,7 +95,7 @@ void gtkpod_warning (const gchar *format, ...)
 void delete_populate_settings (struct DeleteData *dd,
 			       gchar **label, gchar **title,
 			       gboolean *confirm_again,
-			       ConfHandlerOpt *confirm_again_handler,
+			       gchar **confirm_again_key,
 			       GString **str)
 {
     Track *s;
@@ -104,10 +104,9 @@ void delete_populate_settings (struct DeleteData *dd,
 
     g_return_if_fail (dd);
     g_return_if_fail (dd->itdb);
-    g_return_if_fail (dd->pl);
 
     /* write title and label */
-    n = g_list_length (dd->selected_tracks);
+    n = g_list_length (dd->tracks);
 
     if (dd->itdb->usertype & GP_ITDB_TYPE_IPOD)
     {
@@ -128,11 +127,12 @@ void delete_populate_settings (struct DeleteData *dd,
 		    ngettext ("Delete Track Completely from iPod?",
 			      "Delete Tracks Completey from iPod?", n));
 	    if (confirm_again)
-		*confirm_again = prefs_get_track_ipod_file_deletion ();
-	    if (confirm_again_handler)
-		*confirm_again_handler = prefs_set_track_ipod_file_deletion;
+		*confirm_again = prefs_get_int("delete_ipod");
+	    if (confirm_again_key)
+		*confirm_again_key = g_strdup("delete_ipod");
 	    break;
 	case DELETE_ACTION_PLAYLIST:
+	    g_return_if_fail (dd->pl);
 	    if (label)
 		*label = g_strdup_printf(
 		    ngettext ("Are you sure you want to remove the following track from the playlist \"%s\"?",
@@ -142,9 +142,9 @@ void delete_populate_settings (struct DeleteData *dd,
 		    ngettext ("Remove Track From Playlist?",
 			      "Remove Tracks From Playlist?", n));
 	    if (confirm_again)
-		*confirm_again = prefs_get_track_playlist_deletion ();
-	    if (confirm_again_handler)
-		*confirm_again_handler = prefs_set_track_playlist_deletion;
+		*confirm_again = prefs_get_int("delete_track");
+	    if (confirm_again_key)
+		*confirm_again_key = g_strdup("delete_track");
 	    break;
 	default:
 	    g_return_if_reached ();
@@ -168,11 +168,12 @@ void delete_populate_settings (struct DeleteData *dd,
 		    ngettext ("Delete Track from Harddisk?",
 			      "Delete Tracks from Harddisk?", n));
 	    if (confirm_again)
-		*confirm_again = prefs_get_track_local_file_deletion ();
-	    if (confirm_again_handler)
-		*confirm_again_handler = prefs_set_track_local_file_deletion;
+		*confirm_again = prefs_get_int("delete_local_file");
+	    if (confirm_again_key)
+		*confirm_again_key = g_strdup("delete_local_file");
 	    break;
 	case DELETE_ACTION_PLAYLIST:
+	    g_return_if_fail (dd->pl);
 	    if (label)
 		*label = g_strdup_printf(
 		    ngettext ("Are you sure you want to remove the following track from the playlist \"%s\"?",
@@ -182,9 +183,9 @@ void delete_populate_settings (struct DeleteData *dd,
 		    ngettext ("Remove Track From Playlist?",
 			      "Remove Tracks From Playlist?", n));
 	    if (confirm_again)
-		*confirm_again = prefs_get_track_playlist_deletion ();
-	    if (confirm_again_handler)
-		*confirm_again_handler = prefs_set_track_playlist_deletion;
+		*confirm_again = prefs_get_int("delete_file");
+	    if (confirm_again_key)
+		*confirm_again_key = g_strdup("delete_file");
 	    break;
 	case DELETE_ACTION_DATABASE:
 	    if (label)
@@ -196,9 +197,9 @@ void delete_populate_settings (struct DeleteData *dd,
 		    ngettext ("Remove Track from Local Database?",
 			      "Remove Tracks from Local Database?", n));
 	    if (confirm_again)
-		*confirm_again = prefs_get_track_database_deletion ();
-	    if (confirm_again_handler)
-		*confirm_again_handler = prefs_set_track_database_deletion;
+		*confirm_again = prefs_get_int("delete_database");
+	    if (confirm_again_key)
+		*confirm_again_key = g_strdup("delete_database");
 	    break;
 	default:
 	    g_return_if_reached ();
@@ -209,7 +210,7 @@ void delete_populate_settings (struct DeleteData *dd,
     if (str)
     {
 	*str = g_string_sized_new (2000);
-	for(l = dd->selected_tracks; l; l = l->next)
+	for(l = dd->tracks; l; l = l->next)
 	{
 	    s = l->data;
 	    g_return_if_fail (s);
@@ -227,7 +228,7 @@ static void delete_track_cancel (struct DeleteData *dd)
 {
     g_return_if_fail (dd);
 
-    g_list_free (dd->selected_tracks);
+    g_list_free (dd->tracks);
     g_free (dd);
 }
 
@@ -238,7 +239,6 @@ static void delete_track_cancel (struct DeleteData *dd)
 void delete_track_ok (struct DeleteData *dd)
 {
     gint n;
-    gchar *buf = NULL;
     GList *l;
 
     g_return_if_fail (dd);
@@ -246,22 +246,22 @@ void delete_track_ok (struct DeleteData *dd)
     g_return_if_fail (dd->itdb);
 
     /* should never happen */
-    if (!dd->selected_tracks)	delete_track_cancel (dd);
+    if (!dd->tracks)	delete_track_cancel (dd);
 
     /* nr of tracks to be deleted */
-    n = g_list_length (dd->selected_tracks);
+    n = g_list_length (dd->tracks);
     if (dd->itdb->usertype & GP_ITDB_TYPE_IPOD)
     {
 	switch (dd->deleteaction)
 	{
 	case DELETE_ACTION_IPOD:
-	    buf = g_strdup_printf (
+	    gtkpod_statusbar_message (
 		ngettext ("Deleted one track completely from iPod",
 			  "Deleted %d tracks completely from iPod",
 			  n), n);
 	    break;
 	case DELETE_ACTION_PLAYLIST:
-	    buf = g_strdup_printf (
+	    gtkpod_statusbar_message (
 		ngettext ("Deleted %d track from playlist '%s'",
 			  "Deleted %d tracks from playlist '%s'",
 			  n), n, dd->pl->name);
@@ -279,19 +279,19 @@ void delete_track_ok (struct DeleteData *dd)
 	switch (dd->deleteaction)
 	{
 	case DELETE_ACTION_LOCAL:
-	    buf = g_strdup_printf (
+	    gtkpod_statusbar_message (
 		ngettext ("Deleted one track from harddisk",
 			  "Deleted %d tracks from harddisk",
 			  n), n);
 	    break;
 	case DELETE_ACTION_PLAYLIST:
-	    buf = g_strdup_printf (
+	    gtkpod_statusbar_message (
 		ngettext ("Deleted %d track from playlist '%s'",
 			  "Deleted %d tracks from playlist '%s'",
 			  n), n, dd->pl->name);
 	    break;
 	case DELETE_ACTION_DATABASE:
-	    buf = g_strdup_printf (
+	    gtkpod_statusbar_message (
 		ngettext ("Deleted track from local database",
 			  "Deleted %d tracks from local database",
 			  n), n);
@@ -303,14 +303,12 @@ void delete_track_ok (struct DeleteData *dd)
 	    break;
 	}
     }
-    for (l = dd->selected_tracks; l; l = l->next)
+    for (l = dd->tracks; l; l = l->next)
     {
 	gp_playlist_remove_track (dd->pl, l->data, dd->deleteaction);
     }
-    gtkpod_statusbar_message (buf);
-    g_free (buf);
 
-    g_list_free (dd->selected_tracks);
+    g_list_free (dd->tracks);
     g_free (dd);
 
     gtkpod_tracks_statusbar_update ();
@@ -328,12 +326,12 @@ void delete_track_head (DeleteAction deleteaction)
     struct DeleteData *dd;
     iTunesDB *itdb;
     GtkResponseType response;
-    ConfHandlerOpt confirm_again_handler;
+    gchar *confirm_again_key;
 
     pl = pm_get_selected_playlist ();
     if (pl == NULL)
     { /* no playlist??? Cannot happen, but... */
-	gtkpod_statusbar_message (_("No playlist selected."));
+	message_sb_no_playlist_selected ();
 	return;
     }
     itdb = pl->itdb;
@@ -342,19 +340,19 @@ void delete_track_head (DeleteAction deleteaction)
     selected_tracks = tm_get_selected_tracks();
     if (selected_tracks == NULL)
     {  /* no tracks selected */
-	gtkpod_statusbar_message (_("No tracks selected."));
+	message_sb_no_tracks_selected ();
 	return;
     }
 
     dd = g_malloc0 (sizeof (struct DeleteData));
     dd->deleteaction = deleteaction;
-    dd->selected_tracks = selected_tracks;
+    dd->tracks = selected_tracks;
     dd->pl = pl;
     dd->itdb = itdb;
 
     delete_populate_settings (dd,
 			      &label, &title,
-			      &confirm_again, &confirm_again_handler,
+			      &confirm_again, &confirm_again_key,
 			      &str);
     /* open window */
     response = gtkpod_confirmation
@@ -366,7 +364,7 @@ void delete_track_head (DeleteAction deleteaction)
 	 NULL, 0, NULL,        /* option 1 */
 	 NULL, 0, NULL,        /* option 2 */
 	 confirm_again,        /* gboolean confirm_again, */
-	 confirm_again_handler,/* ConfHandlerOpt confirm_again_handler,*/
+	 confirm_again_key,/* ConfHandlerOpt confirm_again_key,*/
 	 CONF_NULL_HANDLER,    /* ConfHandler ok_handler,*/
 	 NULL,                 /* don't show "Apply" button */
 	 CONF_NULL_HANDLER,    /* cancel_handler,*/
@@ -386,18 +384,10 @@ void delete_track_head (DeleteAction deleteaction)
 
     g_free (label);
     g_free (title);
+    g_free(confirm_again_key);
     g_string_free (str, TRUE);
 }
 
-void
-gtkpod_main_window_set_active(gboolean active)
-{
-    if(gtkpod_window)
-    {
-	gtk_widget_set_sensitive(gtkpod_window, active);
-    }
-}
-
 
 
 /*------------------------------------------------------------------*\
@@ -418,18 +408,18 @@ void delete_entry_head (gint inst, DeleteAction deleteaction)
     GString *str;
     gchar *label = NULL, *title = NULL;
     gboolean confirm_again;
-    ConfHandlerOpt confirm_again_handler;
+    gchar *confirm_again_key;
     TabEntry *entry;
     GtkResponseType response;
     iTunesDB *itdb;
 
     g_return_if_fail (inst >= 0);
-    g_return_if_fail (inst <= prefs_get_sort_tab_num ());
+    g_return_if_fail (inst <= prefs_get_int("sort_tab_num"));
 
     pl = pm_get_selected_playlist();
     if (pl == NULL)
     { /* no playlist??? Cannot happen, but... */
-	gtkpod_statusbar_message (_("No playlist selected."));
+	message_sb_no_playlist_selected ();
 	return;
     }
     itdb = pl->itdb;
@@ -453,13 +443,13 @@ void delete_entry_head (gint inst, DeleteAction deleteaction)
 
     dd = g_malloc0 (sizeof (struct DeleteData));
     dd->deleteaction = deleteaction;
-    dd->selected_tracks = selected_tracks;
+    dd->tracks = selected_tracks;
     dd->pl = pl;
     dd->itdb = itdb;
 
     delete_populate_settings (dd,
 			      &label, &title,
-			      &confirm_again, &confirm_again_handler,
+			      &confirm_again, &confirm_again_key,
 			      &str);
 
     /* open window */
@@ -472,7 +462,7 @@ void delete_entry_head (gint inst, DeleteAction deleteaction)
 	 NULL, 0, NULL,        /* option 1 */
 	 NULL, 0, NULL,        /* option 2 */
 	 confirm_again,        /* gboolean confirm_again, */
-	 confirm_again_handler,/* ConfHandlerOpt confirm_again_handler,*/
+	 confirm_again_key,/* ConfHandlerOpt confirm_again_key,*/
 	 CONF_NULL_HANDLER,    /* ConfHandler ok_handler,*/
 	 NULL,                 /* don't show "Apply" button */
 	 CONF_NULL_HANDLER,    /* cancel_handler,*/
@@ -494,6 +484,7 @@ void delete_entry_head (gint inst, DeleteAction deleteaction)
 
     g_free (label);
     g_free (title);
+    g_free(confirm_again_key);
     g_string_free (str, TRUE);
 }
 
@@ -510,7 +501,7 @@ static void delete_playlist_cleanup (struct DeleteData *dd)
 {
     g_return_if_fail (dd);
 
-    g_list_free (dd->selected_tracks);
+    g_list_free (dd->tracks);
     g_free (dd);
 }
 
@@ -550,13 +541,14 @@ static void delete_playlist_ok (struct DeleteData *dd)
 	    }
 	    else
 	    {
-		/* remove playlist */
-		gp_playlist_remove (dd->pl);
+		/* first use playlist name */
 		msg = g_strdup_printf (
 		    ngettext ("Deleted playlist '%s' including %d member track",
 			      "Deleted playlist '%s' including %d member tracks",
 			      n),
 		    dd->pl->name, n);
+		/* then remove playlist */
+		gp_playlist_remove (dd->pl);
 	    }
 	    break;
 	case DELETE_ACTION_PLAYLIST:
@@ -566,9 +558,11 @@ static void delete_playlist_ok (struct DeleteData *dd)
 	    }
 	    else
 	    {
-		gp_playlist_remove (dd->pl);
+		/* first use playlist name */
 		msg = g_strdup_printf (_("Deleted playlist '%s'"),
 				       dd->pl->name);
+		/* then remove playlist */
+		gp_playlist_remove (dd->pl);
 	    }
 	    break;
 	case DELETE_ACTION_LOCAL:
@@ -585,7 +579,7 @@ static void delete_playlist_ok (struct DeleteData *dd)
 	case DELETE_ACTION_LOCAL:
 	    if (itdb_playlist_is_mpl (dd->pl))
 	    {   /* for safety reasons this is not implemented (would
-		   remove all tracks from your local harddisk */
+		   remove all tracks from your local harddisk) */
 		g_return_if_reached ();
 	    }
 	    else
@@ -597,13 +591,14 @@ static void delete_playlist_ok (struct DeleteData *dd)
 					      dd->pl->members->data,
 					      dd->deleteaction);
 		}
-		/* remove playlist */
-		gp_playlist_remove (dd->pl);
+		/* first use playlist name */
 		msg = g_strdup_printf (
 		    ngettext ("Deleted playlist '%s' including %d member track on harddisk",
 			      "Deleted playlist '%s' including %d member tracks on harddisk",
 			      n),
 		    dd->pl->name, n);
+		/* then remove playlist */
+		gp_playlist_remove (dd->pl);
 	    }
 	    break;
 	case DELETE_ACTION_DATABASE:
@@ -620,13 +615,14 @@ static void delete_playlist_ok (struct DeleteData *dd)
 	    }
 	    else
 	    {
-		/* remove playlist */
-		gp_playlist_remove (dd->pl);
+		/* first use playlist name */
 		msg = g_strdup_printf (
 		    ngettext ("Deleted playlist '%s' including %d member track",
 			      "Deleted playlist '%s' including %d member tracks",
 			      n),
 		    dd->pl->name, n);
+		/* then remove playlist */
+		gp_playlist_remove (dd->pl);
 	    }
 	    break;
 	case DELETE_ACTION_PLAYLIST:
@@ -636,9 +632,11 @@ static void delete_playlist_ok (struct DeleteData *dd)
 	    }
 	    else
 	    {
-		gp_playlist_remove (dd->pl);
+		/* first use playlist name */
 		msg = g_strdup_printf (_("Deleted playlist '%s'"),
 				       dd->pl->name);
+		/* then remove playlist */
+		gp_playlist_remove (dd->pl);
 	    }
 	    break;
 	case DELETE_ACTION_IPOD:
@@ -662,16 +660,16 @@ void delete_playlist_head (DeleteAction deleteaction)
     Playlist *pl;
     iTunesDB *itdb;
     GtkResponseType response = GTK_RESPONSE_NONE;
-    GString *str;
     gchar *label = NULL, *title = NULL;
     gboolean confirm_again;
-    ConfHandlerOpt confirm_again_handler;
+    gchar *confirm_again_key;
     guint32 n = 0;
+    GString *str;
 
     pl = pm_get_selected_playlist();
     if (!pl)
     { /* no playlist selected */
-	gtkpod_statusbar_message (_("No playlist selected."));
+	message_sb_no_playlist_selected ();
 	return;
     }
 
@@ -694,14 +692,14 @@ void delete_playlist_head (DeleteAction deleteaction)
 	    }
 	    else if (itdb_playlist_is_podcasts (pl))
 	    {   /* podcasts playlist */
-		dd->selected_tracks = g_list_copy (pl->members);
+		dd->tracks = g_list_copy (pl->members);
 		label = g_strdup_printf (_("Are you sure you want to remove all podcasts from your iPod?"));
 	    }
 	    else 
 	    {   /* normal playlist */
 		/* we set selected_tracks to get a list printed by
 		 * delete_populate_settings() further down */
-		dd->selected_tracks = g_list_copy (pl->members);
+		dd->tracks = g_list_copy (pl->members);
 		label = g_strdup_printf (
 		    ngettext ("Are you sure you want to delete playlist '%s' and the following track completely from your ipod? The number of playlists this track is a member of is indicated in parentheses.",
 			      "Are you sure you want to delete playlist '%s' and the following tracks completely from your ipod? The number of playlists the tracks are member of is indicated in parentheses.", n), pl->name);
@@ -738,7 +736,7 @@ void delete_playlist_head (DeleteAction deleteaction)
 	    {
 		/* we set selected_tracks to get a list printed by
 		 * delete_populate_settings() further down */
-		dd->selected_tracks = g_list_copy (pl->members);
+		dd->tracks = g_list_copy (pl->members);
 		label = g_strdup_printf (
 		    ngettext ("Are you sure you want to delete playlist '%s' and remove the following track from your harddisk? The number of playlists this track is a member of is indicated in parentheses.",
 			      "Are you sure you want to delete playlist '%s' and remove the following tracks from your harddisk? The number of playlists the tracks are member of is indicated in parentheses.",
@@ -755,7 +753,7 @@ void delete_playlist_head (DeleteAction deleteaction)
 	    {
 		/* we set selected_tracks to get a list printed by
 		 * delete_populate_settings() further down */
-		dd->selected_tracks = g_list_copy (pl->members);
+		dd->tracks = g_list_copy (pl->members);
 		label = g_strdup_printf (
 		    ngettext ("Are you sure you want to delete playlist '%s' and remove the following track from the database? The number of playlists this track is a member of is indicated in parentheses.",
 			      "Are you sure you want to delete playlist '%s' and remove the following tracks from the database? The number of playlists the tracks are member of is indicated in parentheses.",
@@ -782,7 +780,7 @@ void delete_playlist_head (DeleteAction deleteaction)
     delete_populate_settings (dd,
 			      NULL, &title,
 			      &confirm_again,
-			      &confirm_again_handler,
+			      &confirm_again_key,
 			      &str);
 
     response = gtkpod_confirmation
@@ -794,7 +792,7 @@ void delete_playlist_head (DeleteAction deleteaction)
 	 NULL, 0, NULL,          /* option 1 */
 	 NULL, 0, NULL,          /* option 2 */
 	 confirm_again,          /* gboolean confirm_again, */
-	 confirm_again_handler,  /* ConfHandlerOpt confirm_again_handler,*/
+	 confirm_again_key,  /* ConfHandlerOpt confirm_again_key,*/
 	 CONF_NULL_HANDLER,      /* ConfHandler ok_handler,*/
 	 NULL,                   /* don't show "Apply" button */
 	 CONF_NULL_HANDLER,      /* cancel_handler,*/
@@ -803,6 +801,7 @@ void delete_playlist_head (DeleteAction deleteaction)
 
     g_free (label);
     g_free (title);
+    g_free(confirm_again_key);
     g_string_free (str, TRUE);
 
     switch (response)
@@ -818,212 +817,6 @@ void delete_playlist_head (DeleteAction deleteaction)
 
 
 
-/*------------------------------------------------------------------*\
- *                                                                  *
- *             Create iPod directory hierarchy                      *
- *                                                                  *
-\*------------------------------------------------------------------*/
-
-/* deterimine the number of directories that should be created */
-/*
- * <20 GB: 20
- * <30 GB: 30
- * <40 GB: 40
- * <50 GB: 50
- * else:   60
- *
- */
-static gint ipod_directories_number (gchar *mp)
-{
-    const gint default_nr = 20;
-#ifdef HAVE_statvfs
-    struct statvfs stat;
-    int	status;
-    gdouble size;
-
-    g_return_val_if_fail (mp, default_nr);
-
-    status = statvfs (mp, &stat);
-    if (status != 0)
-    {
-	return default_nr;
-    }
-    /* Get size in GB */
-    size = ((gdouble)stat.f_blocks * stat.f_frsize) / 1024 / 1024 / 1024;
-    if (size < 20) return 20;
-    if (size < 30) return 30;
-    if (size < 40) return 40;
-    if (size < 50) return 50;
-    return 60;
-#else
-    return default_nr;
-#endif
-}
-
-
-/* ok handler for ipod directory creation */
-/* @user_data1 is the mount point of the iPod */
-static void ipod_directories_ok (gchar *mp)
-{
-    gchar *buf, *errordir=NULL;
-    gchar *pbuf;
-    gint i, dirnum;
-
-    g_return_if_fail (mp);
-
-    if (!errordir)
-    {
-	pbuf = g_build_filename (mp, "Calendars", NULL);
-	if (!g_file_test (pbuf, G_FILE_TEST_EXISTS))
-	{
-	    if((mkdir(pbuf, 0777) != 0))
-		errordir = pbuf;
-	}
-	if (!errordir)
-	    g_free (pbuf);
-    }
-    if (!errordir)
-    {
-	pbuf = g_build_filename (mp, "Contacts", NULL);
-	if (!g_file_test (pbuf, G_FILE_TEST_EXISTS))
-	{
-	    if((mkdir(pbuf, 0777) != 0))
-		errordir = pbuf;
-	}
-	if (!errordir)
-	    g_free (pbuf);
-    }
-    if (!errordir)
-    {
-	pbuf = g_build_filename (mp, "iPod_Control", NULL);
-	if (!g_file_test (pbuf, G_FILE_TEST_EXISTS))
-	{
-	    if((mkdir(pbuf, 0777) != 0))
-		errordir = pbuf;
-	}
-	if (!errordir)
-	    g_free (pbuf);
-    }
-    if (!errordir)
-    {
-	pbuf = g_build_filename (mp, "iPod_Control", "Music", NULL);
-	if (!g_file_test (pbuf, G_FILE_TEST_EXISTS))
-	{
-	    if((mkdir(pbuf, 0777) != 0))
-		errordir = pbuf;
-	}
-	if (!errordir)
-	    g_free (pbuf);
-    }
-    if (!errordir)
-    {
-	pbuf = g_build_filename (mp, "iPod_Control", "iTunes", NULL);
-	if (!g_file_test (pbuf, G_FILE_TEST_EXISTS))
-	{
-	    if((mkdir(pbuf, 0777) != 0))
-		errordir = pbuf;
-	}
-	if (!errordir)
-	    g_free (pbuf);
-    }
-    if (!errordir)
-    {
-	pbuf = g_build_filename (mp, "iPod_Control", "Artwork", NULL);
-	if (!g_file_test (pbuf, G_FILE_TEST_EXISTS))
-	{
-	    if((mkdir(pbuf, 0777) != 0))
-		errordir = pbuf;
-	}
-	if (!errordir)
-	    g_free (pbuf);
-    }
-    dirnum = ipod_directories_number (mp);
-    for(i = 0; i < dirnum; i++)
-    {
-	if (!errordir)
-	{
-	    gchar *num = g_strdup_printf ("F%02d", i);
-	    pbuf = g_build_filename (mp,
-				     "iPod_Control", "Music", num, NULL);
-	    if (!g_file_test (pbuf, G_FILE_TEST_EXISTS))
-	    {
-		if((mkdir(pbuf, 0777) != 0))
-		    errordir = pbuf;
-		g_free (num);
-	    }
-	    if (!errordir)
-		g_free (pbuf);
-	}
-    }
-
-    if (errordir)
-	buf = g_strdup_printf (_("Problem creating iPod directory: '%s'."), errordir);
-    else
-	buf = g_strdup_printf (_("Successfully created iPod directories in '%s'."), mp);
-    gtkpod_statusbar_message(buf);
-    g_free (buf);
-    g_free (errordir);
-}
-
-
-/* Pop up the confirmation window for creation of ipod directory
-   hierarchy. */
-void ipod_directories_head (const gchar *mountpoint)
-{
-    gchar *mp;
-    GString *str;
-    GtkResponseType response;
-
-    g_return_if_fail (mountpoint);
-    mp = g_strdup (mountpoint);
-
-    if (strlen (mp) > 0)
-    { /* make sure the mount point does not end in "/" */
-	if (mp[strlen (mp) - 1] == '/')
-	    mp[strlen (mp) - 1] = 0;
-    }
-    else
-    {
-	mp = g_strdup (".");
-    }
-    str = g_string_sized_new (2000);
-    g_string_append_printf (str, "%s/Calendars\n", mp);
-    g_string_append_printf (str, "%s/Contacts\n", mp);
-    g_string_append_printf (str, "%s/iPod_Control\n", mp);
-    g_string_append_printf (str, "%s/iPod_Control/Music\n", mp);
-    g_string_append_printf (str, "%s/iPod_Control/iTunes\n", mp);
-    g_string_append_printf (str, "%s/iPod_Control/Music/F00\n...\n", mp);
-    g_string_append_printf (str, "%s/iPod_Control/Music/F%d\n",
-			    mp, ipod_directories_number (mp)-1);
-
-    response = gtkpod_confirmation (CONF_ID_IPOD_DIR,    /* gint id, */
-			 TRUE,               /* gboolean modal, */
-			 _("Create iPod directories"), /* title */
-			 _("OK to create the following directories?"),
-			 str->str,
-			 NULL, 0, NULL,       /* option 1 */
-			 NULL, 0, NULL,       /* option 2 */
-			 TRUE,                /* gboolean confirm_again, */
-			 NULL, /* ConfHandlerOpt confirm_again_handler, */
-			 CONF_NULL_HANDLER,   /* ConfHandler ok_handler,*/
-			 NULL,                /* don't show "Apply" */
-			 CONF_NULL_HANDLER,   /* cancel_handler,*/
-			 NULL,                /* gpointer user_data1,*/
-			 NULL);               /* gpointer
-					       * user_data2,*/
-    switch (response)
-    {
-    case GTK_RESPONSE_OK:
-	ipod_directories_ok (mp);
-	break;
-    default:
-	break;
-    }
-    g_free (mp);
-    g_string_free (str, TRUE);
-}
-
-
 /* callback for quit menu entry */
 void
 on_quit1_activate                      (GtkMenuItem     *menuitem,
@@ -1072,7 +865,7 @@ gtkpod_main_quit(void)
     {
 	server_shutdown (); /* stop accepting requests for playcount updates */
 
-	write_prefs ();
+	cleanup_prefs();
 
 /* FIXME: release memory in a clean way */
 #if 0
@@ -1083,11 +876,7 @@ gtkpod_main_quit(void)
 #endif
 	display_cleanup ();
 
-	if(prefs_get_automount())
-	{
-	    unmount_ipod ();
-	}
-	call_script ("gtkpod.out");
+	call_script ("gtkpod.out", NULL);
 	gtk_main_quit ();
 	return FALSE;
     }
