@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-12-03 20:39:08 jcs>
+/* Time-stamp: <2006-09-21 23:16:41 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: misc_conversion.c,v 1.25 2005/12/03 16:36:26 jcsjcs Exp $
+|  $Id: misc_conversion.c,v 1.28 2006/09/21 15:03:13 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -93,6 +93,11 @@ static const gchar *t_strings[] = {
     N_("Subtitle"),
     N_("Date released"),
     N_("Checked"),           /* 35 */
+    N_("Start time"),
+    N_("Stop time"),
+    N_("Remember Playback Position"),
+    N_("Skip when Shuffling"),
+    N_("Artwork Path"),      /* 40 */
     NULL };
 
 /* Tooltips for prefs window */
@@ -136,6 +141,10 @@ static const gchar *t_tooltips[] = {
     NULL,
     N_("Release date (for podcasts displayed next to the title on the iPod)"),
     NULL,   /* 35 */
+    NULL,
+    NULL,
+    NULL,
+    NULL,
     NULL
  };
 
@@ -181,7 +190,8 @@ T_item TM_to_T (TM_item tm)
     case TM_COLUMN_PODCASTRSS:    return T_PODCASTRSS;
     case TM_COLUMN_SUBTITLE:      return T_SUBTITLE;
     case TM_COLUMN_TIME_RELEASED: return T_TIME_RELEASED;
-    case TM_NUM_COLUMNS:          return -1;
+    case TM_COLUMN_THUMB_PATH:    return T_THUMB_PATH;
+    case TM_NUM_COLUMNS:          g_return_val_if_reached (-1);
     }
     return -1;
 }
@@ -201,7 +211,7 @@ T_item ST_to_T (ST_CAT_item st)
     case ST_CAT_TITLE:       return T_TITLE;
     case ST_CAT_YEAR:        return T_YEAR;
     case ST_CAT_SPECIAL:
-    case ST_CAT_NUM:         return -1;
+    case ST_CAT_NUM:         g_return_val_if_reached (-1);
     }
     return -1;
 }
