@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-11-21 00:27:51 jcs>
+/* Time-stamp: <2006-06-24 01:53:45 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: misc_playlist.c,v 1.27 2005/11/23 04:16:12 jcsjcs Exp $
+|  $Id: misc_playlist.c,v 1.38 2006/06/23 16:59:23 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -115,7 +115,16 @@ void
 on_smart_playlist_activate             (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-    spl_edit_new (gp_get_active_itdb(), NULL, -1);
+    iTunesDB *itdb = gp_get_selected_itdb();
+
+    if (itdb)
+    {
+	spl_edit_new (itdb, NULL, -1);
+    }
+    else
+    {
+	message_sb_no_itdb_selected ();
+    }
 }
 
 
@@ -211,9 +220,13 @@ void generate_category_playlists (iTunesDB *itdb, T_item cat)
 Playlist *generate_displayed_playlist (void)
 {
     GList *tracks = tm_get_all_tracks ();
-    Playlist *result = generate_new_playlist (gp_get_active_itdb (),
-					      tracks);
-    g_list_free (tracks);
+    Playlist *result = NULL;
+
+    if (tracks)
+    {
+	result= generate_new_playlist (gp_get_selected_itdb (), tracks);
+	g_list_free (tracks);
+    }
     return result;
 }
 
@@ -223,15 +236,19 @@ Playlist *generate_displayed_playlist (void)
 Playlist *generate_selected_playlist (void)
 {
     GList *tracks = tm_get_selected_tracks ();
-    Playlist *result = generate_new_playlist (gp_get_active_itdb (),
-					      tracks);
-    g_list_free (tracks);
+    Playlist *result=NULL;
+
+    if (tracks)
+    {
+	result= generate_new_playlist (gp_get_selected_itdb (), tracks);
+	g_list_free (tracks);
+    }
     return result;
 }
 
 
 /* Generates a playlist containing a random selection of
-   prefs_get_misc_track_nr() tracks in random order from the currently
+   prefs_get_int("misc_track_nr") tracks in random order from the currently
    displayed tracks */
 Playlist *generate_random_playlist (iTunesDB *itdb)
 {
@@ -240,7 +257,7 @@ Playlist *generate_random_playlist (iTunesDB *itdb)
     gchar *pl_name, *pl_name1;
     GList *rtracks = NULL;
     GList *tracks = tm_get_all_tracks ();
-    gint tracks_max = prefs_get_misc_track_nr();
+    gint tracks_max = prefs_get_int("misc_track_nr");
     gint tracks_nr = 0;
 
 
@@ -271,13 +288,13 @@ void randomize_current_playlist (void)
 
     if (!pl)
     {
-	gtkpod_statusbar_message (_("No playlist selected"));
+	message_sb_no_playlist_selected ();
 	return;
     }
 
-    if (prefs_get_tm_autostore ())
+    if (prefs_get_int("tm_autostore"))
     {
-	prefs_set_tm_autostore (FALSE);
+	prefs_set_int("tm_autostore", FALSE);
 	gtkpod_warning (_("Auto Store of track view disabled.\n\n"));
 /* 	sort_window_update (); */
     }
@@ -354,7 +371,6 @@ Playlist *generate_playlist_with_name (iTunesDB *itdb,GList *tracks,
 {
     Playlist *new_pl=NULL;
     gint n = g_list_length (tracks);
-    gchar *str;
 
     g_return_val_if_fail (itdb, new_pl);
 
@@ -387,7 +403,7 @@ Playlist *generate_playlist_with_name (iTunesDB *itdb,GList *tracks,
 	    g_return_val_if_fail (track, new_pl);
 	    gp_playlist_add_track (new_pl, track, TRUE);
 	}
-	str = g_strdup_printf (
+	gtkpod_statusbar_message (
 	    ngettext ("Created playlist '%s' with %d track.",
 		      "Created playlist '%s' with %d tracks.",
 		      n), pl_name, n);
@@ -399,11 +415,9 @@ Playlist *generate_playlist_with_name (iTunesDB *itdb,GList *tracks,
     }
     else
     {   /* n==0 */
-	str = g_strdup_printf (_("No tracks available, playlist not created"));
+	gtkpod_statusbar_message (_("No tracks available, playlist not created"));
     }
-    gtkpod_statusbar_message (str);
     gtkpod_tracks_statusbar_update();
-    g_free (str);
     return new_pl;
 }
 
@@ -523,7 +537,7 @@ static gboolean Most_Listened_IF (Track *track, gpointer userdata)
 
 void most_listened_pl (iTunesDB *itdb)
 {
-    gint tracks_nr = prefs_get_misc_track_nr();
+    gint tracks_nr = prefs_get_int("misc_track_nr");
     gchar *str;
 
     g_return_if_fail (itdb);
@@ -598,13 +612,13 @@ static gint Most_Rated_CF (gconstpointer aa, gconstpointer bb)
 /* Insert function: determines whether a track is entered into the playlist */
 static gboolean Most_Rated_IF (Track *track, gpointer userdata)
 {
-    if (track) return ((track->playcount != 0) || prefs_get_not_played_track());
+    if (track) return ((track->playcount != 0) || prefs_get_int("not_played_track"));
     return FALSE;
 }
 
 void most_rated_pl (iTunesDB *itdb)
 {
-    gint tracks_nr = prefs_get_misc_track_nr();
+    gint tracks_nr = prefs_get_int("misc_track_nr");
     gchar *str;
 
     g_return_if_fail (itdb);
@@ -694,7 +708,7 @@ static gboolean Last_Listened_IF (Track *track, gpointer userdata)
 
 void last_listened_pl (iTunesDB *itdb)
 {
-    gint tracks_nr = prefs_get_misc_track_nr();
+    gint tracks_nr = prefs_get_int("misc_track_nr");
     gchar *str;
 
     g_return_if_fail (itdb);
@@ -889,20 +903,20 @@ check_db_danglingok1 (gpointer user_data1, gpointer user_data2)
 	 tlist != NULL;
 	 tlist = g_list_next(tlist))
     {
-	gchar *buf;
 	Track *oldtrack;
 	Track *track = tlist->data;
 	ExtraTrackData *etr;
+	gchar *buf;
 
 	g_return_if_fail (track);
 	etr = track->userdata;
 	g_return_if_fail (etr);
         /* printf("Handling track %d\n", track->ipod_id); */
 
-	buf = g_strdup_printf (_("Processing '%s'"),
-			       get_track_info (track, TRUE));
-	gtkpod_statusbar_message (buf);
+	buf = get_track_info (track, TRUE);
+	gtkpod_statusbar_message (_("Processing '%s'"), buf);
 	g_free (buf);
+				  
 	while (widgets_blocked && gtk_events_pending ())
 	    gtk_main_iteration ();
 
@@ -934,8 +948,8 @@ check_db_danglingok1 (gpointer user_data1, gpointer user_data2)
 
 
 
-/* checks iTunesDB for presence of dangling links and checks IPODs Music directory
- * on subject of orphaned files */
+/* checks iTunesDB for presence of dangling links and checks IPODs
+ * Music directory on subject of orphaned files */
 void check_db (iTunesDB *itdb)
 {
 
@@ -967,7 +981,6 @@ void check_db (iTunesDB *itdb)
 
     gchar *pathtrack=NULL;
     gchar *ipod_filename = NULL;
-    gchar *buf = NULL;
 #   define localdebug  0      /* may be later becomes more general verbose param */
     Playlist* pl_orphaned = NULL;
     GList * l_dangling[2] = {NULL, NULL}; /* 2 kinds of dangling tracks: with approp
@@ -980,9 +993,10 @@ void check_db (iTunesDB *itdb)
     gint norphaned = 0;
     gint ndangling = 0;
     gchar ** tokens;
-    gchar *ipod_path_as_filename = charset_from_utf8 (prefs_get_ipod_mount ());
+    const gchar *mountpoint = itdb_get_mountpoint (itdb);
     ExtraiTunesDBData *eitdb;
     GList *gl;
+    gchar *music_dir = NULL;
 
     g_return_if_fail (itdb);
     eitdb = itdb->userdata;
@@ -992,10 +1006,8 @@ void check_db (iTunesDB *itdb)
        a mistake and we should tell him about it */
     if (!eitdb->itdb_imported)
     {
-	const gchar *itunes_components[] = {"iPod_Control", "iTunes", NULL};
-	gchar *itunes_filename = resolve_path(ipod_path_as_filename,
-					      itunes_components);
-	if (g_file_test (itunes_filename, G_FILE_TEST_EXISTS))
+	gchar *itunesdb_filename = itdb_get_itunesdb_path (mountpoint);
+	if (itunesdb_filename)
 	{
 	    GtkWidget *dialog = gtk_message_dialog_new (
 		GTK_WINDOW (gtkpod_window),
@@ -1005,17 +1017,15 @@ void check_db (iTunesDB *itdb)
 		_("You did not import the existing iTunesDB. This is most likely incorrect and will result in the loss of the existing database.\n\nPress 'OK' if you want to proceed anyhow or 'Cancel' to abort. If you cancel, you can import the existing database before calling this function again.\n"));
 	    gint result = gtk_dialog_run (GTK_DIALOG (dialog));
 	    gtk_widget_destroy (dialog);
+	    g_free (itunesdb_filename);
 	    if (result == GTK_RESPONSE_CANCEL)
 	    {
-		g_free (ipod_path_as_filename);
-		g_free (itunes_filename);
 		return;
 	    }
 	}
-	g_free (itunes_filename);
     }
 
-    prefs_set_statusbar_timeout (30*STATUSBAR_TIMEOUT);
+    gtkpod_statusbar_timeout (30*STATUSBAR_TIMEOUT);
     block_widgets();
 
     gtkpod_statusbar_message(_("Creating a tree of known files"));
@@ -1066,15 +1076,15 @@ void check_db (iTunesDB *itdb)
     gtkpod_tracks_statusbar_update();
     process_gtk_events_blocked();
 
+    music_dir = itdb_get_music_dir (mountpoint);
+
     for(h=0; h<itdb_musicdirs_number (itdb); h++)
     {
 	/* directory name */
 	gchar *ipod_dir=g_strdup_printf("F%02d",h); /* just directory name */
 	gchar *ipod_fulldir;
-	const gchar * music[] = {"iPod_Control","Music", NULL, NULL,};
-	music[2] = ipod_dir;
 	/* full path */
-	ipod_fulldir = resolve_path(ipod_path_as_filename,music);
+	ipod_fulldir = itdb_get_path (music_dir, ipod_dir);
 	if(ipod_fulldir && (dir_des=g_dir_open(ipod_fulldir,0,NULL))) {
 	    while ((ipod_filename=g_strdup(g_dir_read_name(dir_des))))
 		/* we have a file in the directory*/
@@ -1098,12 +1108,10 @@ void check_db (iTunesDB *itdb)
 		    gchar *num_str = g_strdup_printf ("F%02d", h);
 		    Track *dupl_track;
 
-		    const gchar *dcomps[] =
-			{ "iPod_Control", "Music", num_str,
-			  ipod_filename, NULL };
+		    const gchar *p_dcomps[] =
+			{ num_str, ipod_filename, NULL };
 
-		    fn_orphaned = resolve_path (
-			prefs_get_ipod_mount(), dcomps);
+		    fn_orphaned = itdb_resolve_path (music_dir, p_dcomps);
 
 		    if (!pl_orphaned)
 		    {
@@ -1122,9 +1130,15 @@ void check_db (iTunesDB *itdb)
 			  It will be removed with the next sync */
 			Track *track = gp_track_new ();
 			gchar *fn_utf8 = charset_to_utf8 (fn_orphaned);
+			const gchar *dir_rel = music_dir + strlen (mountpoint);
+			if (*dir_rel == G_DIR_SEPARATOR) ++dir_rel;
 			track->ipod_path = g_strdup_printf (
-			    ":iPod_Control:Music:%s:%s",
-			    num_str, ipod_filename);
+			    "%c%s%c%s%c%s",
+			    G_DIR_SEPARATOR, dir_rel,
+			    G_DIR_SEPARATOR, num_str,
+			    G_DIR_SEPARATOR, ipod_filename);
+			itdb_filename_fs2ipod (track->ipod_path);
+
 			gp_track_validate_entries (track);
 			mark_track_for_deletion (itdb, track);
 			gtkpod_warning (_(
@@ -1150,20 +1164,19 @@ void check_db (iTunesDB *itdb)
 	    }
             g_dir_close(dir_des);
 	}
-        music[3] = NULL;
         g_free(ipod_dir);
  	g_free(ipod_fulldir);
 	process_gtk_events_blocked();
     }
 
     ndangling=g_tree_nnodes(files_known);
-    buf=g_strdup_printf(_("Found %d orphaned and %d dangling files. Processing..."),
-			norphaned, ndangling);
-
-    gtkpod_statusbar_message(buf);
+    gtkpod_statusbar_message (_("Found %d orphaned and %d dangling files. Processing..."),
+			      norphaned, ndangling);
+    
     gtkpod_tracks_statusbar_update();
 
-    g_free(buf);
+    g_free(music_dir);
+    music_dir = NULL;
 
     /* Now lets deal with dangling tracks */
     /* Traverse the tree - leftovers are dangling - put them in two lists */
@@ -1173,6 +1186,7 @@ void check_db (iTunesDB *itdb)
     {
 	GString *str_dangs = g_string_sized_new(2000);
 	gint ndang=0;
+	gchar *buf;
 
 	glist_list_tracks(l_dangling[i], str_dangs); /* compose String list of the tracks */
 	ndang = g_list_length(l_dangling[i]);
@@ -1214,12 +1228,9 @@ void check_db (iTunesDB *itdb)
     }
 
     if (pl_orphaned) data_changed (itdb);
-    g_free (ipod_path_as_filename);
     g_tree_destroy (files_known);
-    buf = g_strdup_printf (_("Found %d orphaned and %d dangling files. Done."),
-			   norphaned, ndangling);
-    gtkpod_statusbar_message(buf);
-    g_free (buf);
-    prefs_set_statusbar_timeout (0);
+    gtkpod_statusbar_message (_("Found %d orphaned and %d dangling files. Done."),
+			      norphaned, ndangling);
+    gtkpod_statusbar_timeout (0);
     release_widgets ();
 }
