@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-12-11 02:00:14 jcs>
+/* Time-stamp: <2006-09-24 15:25:29 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: misc_track.c,v 1.33 2005/12/10 17:04:29 jcsjcs Exp $
+|  $Id: misc_track.c,v 1.52 2006/09/24 06:42:52 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -56,12 +56,11 @@
 void gp_md5_hash_tracks_itdb (iTunesDB *itdb)
 {
    gint ns, count;
-   gchar *buf;
    GList *gl;
 
    g_return_if_fail (itdb);
 
-   if (!prefs_get_md5tracks ()) return;
+   if (!prefs_get_int("md5")) return;
    ns = itdb_tracks_number (itdb);   /* number of tracks */
    if (ns == 0)                 return;
 
@@ -88,13 +87,11 @@ void gp_md5_hash_tracks_itdb (iTunesDB *itdb)
        ++count;
        if (((count % 20) == 1) || (count == ns))
        { /* update for count == 1, 21, 41 ... and for count == n */
-	   buf = g_strdup_printf (ngettext ("Hashed %d of %d track.",
-					    "Hashed %d of %d tracks.", ns),
-				  count, ns);
-	   gtkpod_statusbar_message (buf);
+	   gtkpod_statusbar_message  (ngettext ("Hashed %d of %d track.",
+						"Hashed %d of %d tracks.", ns),
+				      count, ns);
 	   while (widgets_blocked && gtk_events_pending ())
 	       gtk_main_iteration ();
-	   g_free (buf);
        }
    }
    gp_duplicate_remove (NULL, NULL); /* show info dialogue */
@@ -192,7 +189,7 @@ void gp_duplicate_remove (Track *oldtrack, Track *track)
 
 /*   printf ("%p, %p, '%s'\n", oldtrack, track, str?str->str:"empty");*/
 
-   if (prefs_get_show_duplicates() && !oldtrack && !track && str)
+   if (prefs_get_int("show_duplicates") && !oldtrack && !track && str)
    {
        if (str->len)
        { /* Some tracks have been deleted. Print a notice */
@@ -219,7 +216,7 @@ void gp_duplicate_remove (Track *oldtrack, Track *track)
 		NULL, 0, NULL,      /* option 1 */
 		NULL, 0, NULL,      /* option 2 */
 		TRUE,               /* gboolean confirm_again, */
-		prefs_set_show_duplicates,
+		"show_duplicates",
 		                    /* ConfHandlerCA confirm_again_handler,*/
 		CONF_NULL_HANDLER,  /* ConfHandler ok_handler,*/
 		NULL,               /* don't show "Apply" button */
@@ -248,7 +245,7 @@ void gp_duplicate_remove (Track *oldtrack, Track *track)
        g_return_if_fail (oldetr);
        g_return_if_fail (etr);
 
-       if (prefs_get_show_duplicates ())
+       if (prefs_get_int("show_duplicates"))
        {
 	   /* add info about it to str */
 	   buf = get_track_info (track, TRUE);
@@ -336,17 +333,16 @@ void gp_duplicate_remove (Track *oldtrack, Track *track)
 /**
  * Register all tracks in the md5 hash and remove duplicates (while
  * preserving playlists).
- * Call  remove_duplicate (NULL, NULL); to show an info dialogue
+ * Call  gp_duplicate_remove (NULL, NULL); to show an info dialogue
  */
 void gp_itdb_hash (iTunesDB *itdb)
 {
    gint ns, count, track_nr;
-   gchar *buf;
    Track *track, *oldtrack;
 
    g_return_if_fail (itdb);
 
-   if (!prefs_get_md5tracks ()) return;
+   if (!prefs_get_int("md5")) return;
 
    ns = itdb_tracks_number (itdb);
    if (ns == 0)                 return;
@@ -362,15 +358,13 @@ void gp_itdb_hash (iTunesDB *itdb)
        oldtrack = md5_track_exists_insert (itdb, track);
        ++count;
 /*        printf("%d:%d:%p:%p\n", count, track_nr, track, oldtrack); */
-       if (!prefs_get_block_display() &&
+       if (!prefs_get_int("block_display") &&
 	   (((count % 20) == 1) || (count == ns)))
        { /* update for count == 1, 21, 41 ... and for count == n */
-	   buf = g_strdup_printf (ngettext ("Hashed %d of %d track.",
-					    "Hashed %d of %d tracks.", ns),
-				  count, ns);
-	   gtkpod_statusbar_message(buf);
+	   gtkpod_statusbar_message (ngettext ("Hashed %d of %d track.",
+					       "Hashed %d of %d tracks.", ns),
+				     count, ns);
 	   while (widgets_blocked && gtk_events_pending ())  gtk_main_iteration ();
-	   g_free (buf);
        }
        if (oldtrack)
        {
@@ -399,29 +393,41 @@ void gp_itdb_hash (iTunesDB *itdb)
  * found. This function also works if @name is on the iPod. */
 Track *gp_track_by_filename (iTunesDB *itdb, gchar *filename)
 {
+    gchar *musicdir = NULL;
+    Track *result = NULL;
+
     g_return_val_if_fail (itdb, NULL);
     g_return_val_if_fail (filename, NULL);
 
+    if (itdb->usertype & GP_ITDB_TYPE_IPOD)
+    {
+	gchar *mountpoint = get_itdb_prefs_string (itdb, KEY_MOUNTPOINT);
+	g_return_val_if_fail (mountpoint, NULL);
+	musicdir = itdb_get_music_dir (mountpoint);
+	if (!musicdir)
+	{
+	    /* FIXME: guess */
+	    musicdir = g_build_filename (mountpoint, "iPod_Control",
+					 "Music", NULL);
+	}
+	g_free (mountpoint);
+    }
     if ((itdb->usertype & GP_ITDB_TYPE_IPOD) &&
-	(strncmp (filename, prefs_get_ipod_mount (),
-		  strlen (prefs_get_ipod_mount ())) == 0))
-    {   /* handle track on iPod */
+	(musicdir != NULL) &&
+	(strncmp (filename, musicdir, strlen (musicdir)) == 0))
+    {   /* handle track on iPod (in music dir) */
 	GList *gl;
-	for (gl=itdb->tracks; gl; gl=gl->next)
+	for (gl=itdb->tracks; gl&&!result; gl=gl->next)
 	{
 	    Track *track = gl->data;
-	    gchar *mount;
 	    gchar *ipod_path;
 	    g_return_val_if_fail (track, NULL);
-	    mount = charset_from_utf8 (prefs_get_ipod_mount ());
 	    ipod_path = itdb_filename_on_ipod (track);
-	    g_free (mount);
 	    if (ipod_path)
 	    {
-		if (strcmp (ipod_path, filename) == 0)
+		if (strcasecmp (ipod_path, filename) == 0)
 		{
-		    g_free (ipod_path);
-		    return track;
+		    result = track;
 		}
 		g_free (ipod_path);
 	    }
@@ -440,11 +446,12 @@ Track *gp_track_by_filename (iTunesDB *itdb, gchar *filename)
 	    if (etr->pc_path_locale)
 	    {
 		if (strcmp (etr->pc_path_locale, filename) == 0)
-		    return track;
+		    result = track;
 	    }
 	}
     }
-    return NULL;
+    g_free (musicdir);
+    return result;
 }
 
 
@@ -516,12 +523,17 @@ gchar **track_get_item_pointer (Track *track, T_item t_item)
     case T_SUBTITLE:
 	result = &track->subtitle;
 	break;
+    case T_THUMB_PATH:
+	result = &etr->thumb_path_utf8;
+	break;
     case T_ALL:
     case T_IPOD_ID:
     case T_TRACK_NR:
     case T_TRANSFERRED:
     case T_SIZE:
     case T_TRACKLEN:
+    case T_STARTTIME:
+    case T_STOPTIME:
     case T_BITRATE:
     case T_SAMPLERATE:
     case T_BPM:
@@ -535,6 +547,8 @@ gchar **track_get_item_pointer (Track *track, T_item t_item)
     case T_SOUNDCHECK:
     case T_CD_NR:
     case T_COMPILATION:
+    case T_REMEMBER_PLAYBACK_POSITION:
+    case T_SKIP_WHEN_SHUFFLING:
     case T_CHECKED:
     case T_ITEM_NUM:
 	g_return_val_if_reached (NULL);
@@ -598,6 +612,7 @@ gboolean track_copy_item (Track *frtrack, Track *totrack, T_item item)
     case T_PODCASTURL:
     case T_PODCASTRSS:
     case T_SUBTITLE:
+    case T_THUMB_PATH:
 	fritem = track_get_item (frtrack, item);
 	toitem_ptr = track_get_item_pointer (totrack, item);
 	g_return_val_if_fail (fritem, FALSE);
@@ -608,6 +623,14 @@ gboolean track_copy_item (Track *frtrack, Track *totrack, T_item item)
 	    *toitem_ptr = g_strdup (fritem);
 	    changed = TRUE;
 	}
+	if (item == T_YEAR)
+	{
+	    if (totrack->year != frtrack->year)
+	    {
+		totrack->year = frtrack->year;
+		changed = TRUE;
+	    }
+	}
 	break;
     case T_IPOD_ID:
 	if (frtrack->id != totrack->id)
@@ -649,6 +672,20 @@ gboolean track_copy_item (Track *frtrack, Track *totrack, T_item item)
 	    changed = TRUE;
 	}
 	break;
+    case T_STARTTIME:
+	if (frtrack->starttime != totrack->starttime)
+	{
+	    totrack->starttime = frtrack->starttime;
+	    changed = TRUE;
+	}
+	break;
+    case T_STOPTIME:
+	if (frtrack->stoptime != totrack->stoptime)
+	{
+	    totrack->stoptime = frtrack->stoptime;
+	    changed = TRUE;
+	}
+	break;
     case T_BITRATE:
 	if (frtrack->bitrate != totrack->bitrate)
 	{
@@ -728,6 +765,20 @@ gboolean track_copy_item (Track *frtrack, Track *totrack, T_item item)
 	    changed = TRUE;
 	}
 	break;
+    case T_REMEMBER_PLAYBACK_POSITION:
+	if (frtrack->remember_playback_position != totrack->remember_playback_position)
+	{
+	    totrack->remember_playback_position = frtrack->remember_playback_position;
+	    changed = TRUE;
+	}
+	break;
+    case T_SKIP_WHEN_SHUFFLING:
+	if (frtrack->skip_when_shuffling != totrack->skip_when_shuffling)
+	{
+	    totrack->skip_when_shuffling = frtrack->skip_when_shuffling;
+	    changed = TRUE;
+	}
+	break;
     case T_CHECKED:
 	if (frtrack->checked != totrack->checked)
 	{
@@ -781,6 +832,15 @@ guint32 track_get_timestamp (Track *track, T_item t_item)
 }
 
 
+/* unified format for TRACKLEN, STARTTIME, STOPTIME */
+static gchar *track_get_length_string (gint32 length)
+{
+    return g_strdup_printf ("%d:%06.3f",
+			    length/60000,
+			    ((float)(length%60000)) / 1000);
+}
+
+
 /* Return text for display. g_free() after use. */
 gchar *track_get_text (Track *track, T_item item)
 {
@@ -844,13 +904,36 @@ gchar *track_get_text (Track *track, T_item item)
 	    text = g_strdup (_("Local Database"));
 	}
 	break;
+    case T_THUMB_PATH:
+	text = g_strdup (etr->thumb_path_utf8);
+	if (!text || (strlen (text) == 0))
+	{   /* no path set */
+	    g_free (text);
+	    text = NULL;
+	    if (track->artwork && track->artwork->thumbnails)
+	    {   /* artwork is set */
+		text = g_strdup (_("Embedded or filename was lost"));
+	    }
+	    else
+	    {
+		text = g_strdup (_("Artwork not set"));
+	    }
+	}
+	break;
     case T_SIZE:
 	text = g_strdup_printf ("%d", track->size);
 	break;
     case T_TRACKLEN:
-	text = g_strdup_printf ("%d:%02d",
-				track->tracklen/60000,
-				(track->tracklen/1000)%60);
+	text = track_get_length_string (track->tracklen);
+	break;
+    case T_STARTTIME:
+	text = track_get_length_string (track->starttime);
+	break;
+    case T_STOPTIME:
+	if (track->stoptime == 0)
+	    text = track_get_length_string (track->tracklen);
+	else
+	    text = track_get_length_string (track->stoptime);
 	break;
     case T_BITRATE:
 	text = g_strdup_printf ("%dk", track->bitrate);
@@ -884,6 +967,8 @@ gchar *track_get_text (Track *track, T_item item)
 	break;
     case T_TRANSFERRED:
     case T_COMPILATION:
+    case T_REMEMBER_PLAYBACK_POSITION:
+    case T_SKIP_WHEN_SHUFFLING:
     case T_ALL:
     case T_CHECKED:
     case T_ITEM_NUM:
@@ -893,6 +978,30 @@ gchar *track_get_text (Track *track, T_item item)
 }
 
 
+/* unified scanner for TRACKLEN, STARTTIME, STOPTIME */
+static gint32 track_scan_length (const gchar *new_text)
+{
+    gint32 nr;
+    const gchar *str;
+
+    g_return_val_if_fail (new_text, 0);
+
+    str = strrchr (new_text, ':');
+    if (str)
+    {   /* MM:SS */
+	/* A simple cast to gint32 can sometimes produce a number
+	   that's "1" too small (14.9999999999 -> 14 instead of 15) ->
+	   add 0.1 */
+	nr = 1000 * (((gdouble)(60 * atoi (new_text))) + atof (str+1)) + 0.1;
+    }
+    else
+    {   /* SS */
+	nr = 1000 * atof (new_text) + 0.1;
+    }
+
+    return nr;
+}
+
 
 /* Set track data according to @new_text
 
@@ -1060,29 +1169,50 @@ gboolean track_set_text (Track *track, const gchar *new_text, T_item item)
         }
         break;
     case T_TRACKLEN:
-	str = strrchr (new_text, ':');
-	if (str)
-	{   /* MM:SS */
-	    nr = 1000 * (60 * atoi (new_text) + atoi (str+1));
-	}
-	else
-	{   /* SS */
-	    nr = 1000 * atoi (new_text);
-	}
+	nr = track_scan_length (new_text);
 	if (nr != track->tracklen)
 	{
 	    track->tracklen = nr;
 	    changed = TRUE;
 	}
 	break;
+    case T_STARTTIME:
+	nr = track_scan_length (new_text);
+	if (nr != track->starttime)
+	{
+	    track->starttime = nr;
+	    changed = TRUE;
+	    /* Set stoptime to 0 if stoptime is the same as tracklen */
+	    if (track->stoptime == track->tracklen)
+		track->stoptime = 0;
+	}
+	break;
+    case T_STOPTIME:
+	nr = track_scan_length (new_text);
+	/* if stoptime is identical to tracklen, set stoptime to 0 if
+	 * starttime is 0 as well */
+	if ((nr == track->tracklen) &&
+	    (track->starttime == 0))
+	{
+	    nr = 0;
+	}
+	if (nr != track->stoptime)
+	{
+	    track->stoptime = nr;
+	    changed = TRUE;
+	}
+	break;
     case T_PC_PATH:
     case T_IPOD_PATH:
     case T_IPOD_ID:
     case T_TRANSFERRED:
     case T_COMPILATION:
+    case T_REMEMBER_PLAYBACK_POSITION:
+    case T_SKIP_WHEN_SHUFFLING:
     case T_CHECKED:
     case T_ALL:
     case T_ITEM_NUM:
+    case T_THUMB_PATH: // TODO: this should in fact be settable
 	gtkpod_warning ("Programming error: track_set_text() called with illegal argument (item: %d)\n", item);
 	break;
     }
@@ -1347,7 +1477,7 @@ Playlist *add_text_plain_to_playlist (iTunesDB *itdb, Playlist *pl,
 			if (!pl)  break; /* while (*filesp) */
 		    }
 		    add_directory_by_name (itdb, decoded_file, pl,
-					   prefs_get_add_recursively (),
+					   prefs_get_int("add_recursively"),
 					   trackaddfunc, data);
 		    added = TRUE;
 		}
@@ -1372,7 +1502,7 @@ Playlist *add_text_plain_to_playlist (iTunesDB *itdb, Playlist *pl,
 			    if (!pl)  break; /* while (*filesp) */
 			}
 			add_track_by_filename (itdb, decoded_file, pl,
-					       prefs_get_add_recursively (),
+					       prefs_get_int("add_recursively"),
 					       trackaddfunc, data);
 			added = TRUE;
 			break;
@@ -1384,6 +1514,7 @@ Playlist *add_text_plain_to_playlist (iTunesDB *itdb, Playlist *pl,
 			added = TRUE;
 			break;
 		    case FILE_TYPE_UNKNOWN:
+		    case FILE_TYPE_DIRECTORY:
 		    case FILE_TYPE_IMAGE:
 			break;
 		    }
@@ -1423,9 +1554,9 @@ Playlist *add_text_plain_to_playlist (iTunesDB *itdb, Playlist *pl,
 void gp_track_set_flags_podcast (Track *track)
 {
     g_return_if_fail (track);
-    track->flag1 = 0x02;  /* unknown */
-    track->flag2 = 0x01;  /* skip when shuffling */
-    track->flag3 = 0x01;  /* remember playback position */
+    track->skip_when_shuffling = 0x01;         /* skip when shuffling */
+    track->remember_playback_position = 0x01;  /* remember playback
+						* position */
     track->flag4 = 0x01;  /* Show Title/Album on the 'Now Playing' page */
 }
 
@@ -1433,9 +1564,9 @@ void gp_track_set_flags_podcast (Track *track)
 void gp_track_set_flags_default (Track *track)
 {
     g_return_if_fail (track);
-    track->flag1 = 0x02;  /* unknown */
-    track->flag2 = 0x00;  /* do not skip when shuffling */
-    track->flag3 = 0x00;  /* do not remember playback position */
+    track->skip_when_shuffling = 0x00;  /* do not skip when shuffling */
+    track->remember_playback_position = 0x00;  /* do not remember
+						* playback position */
     track->flag4 = 0x00;  /* Show Title/Album/Artist on the 'Now
 			     Playing' page */
 }
@@ -1474,7 +1605,7 @@ void gp_do_selected_entry (void (*do_func)(GList *tracks), gint inst)
 
     g_return_if_fail (do_func);
 
-    g_return_if_fail ((inst >= 0) && (inst <= prefs_get_sort_tab_num()));
+    g_return_if_fail ((inst >= 0) && (inst <= prefs_get_int("sort_tab_num")));
 
     entry = st_get_selected_entry (inst);
     if (entry == NULL)
@@ -1506,7 +1637,7 @@ void gp_do_selected_playlist (void (*do_func)(GList *tracks))
     pl = pm_get_selected_playlist();
     if (!pl)
     { /* no playlist selected */
-	gtkpod_statusbar_message (_("No playlist selected."));
+	message_sb_no_playlist_selected ();
 	return;
     }
     for (gl=pl->members; gl; gl=gl->next)
@@ -1520,7 +1651,7 @@ void gp_do_selected_playlist (void (*do_func)(GList *tracks))
 }
 
 
-/* return some sensible input about the "track". Yo must free the
+/* return some sensible input about the "track". You must free the
  * return string after use. */
 gchar *get_track_info (Track *track, gboolean prefer_filename)
 {
@@ -1551,4 +1682,3 @@ gchar *get_track_info (Track *track, gboolean prefer_filename)
 
     return g_strdup_printf ("iPod ID: %d", track->id);
 }
-
