@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-11-23 02:36:10 jcs>
+/* Time-stamp: <2006-09-24 12:52:49 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,9 +24,13 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: mp3file.c,v 1.59 2006/02/19 10:20:06 jcsjcs Exp $
+|  $Id: mp3file.c,v 1.71 2006/09/24 03:57:10 jcsjcs Exp $
 */
 
+
+#define LOCALDEBUG 0
+
+
 /* The code in the first section of this file is taken from the
  * mp3info (http://www.ibiblio.org/mp3info/) project. Only the code
  * needed for the playlength calculation has been extracted. */
@@ -48,6 +52,8 @@
  * Description of each item of the TagList list
  */
 typedef struct _File_Tag File_Tag;
+typedef struct _GainData GainData;
+
 struct _File_Tag
 {
     gchar *title;          /* Title of track */
@@ -69,8 +75,25 @@ struct _File_Tag
     gchar *podcastrss;
     gchar *time_released;
     gchar *subtitle;
+    gchar *lyrics;        /* does not appear to be the full lyrics --
+			     only used to set the flag 'lyrics_flag'
+			     of the Track structure */
 };
 
+
+struct _GainData
+{
+  guint32 peak_signal;	  /* LAME Peak Signal * 0x800000             */
+  gdouble radio_gain;	  /* RadioGain in dB
+			     (as defined by www.replaygain.org)      */
+  gdouble audiophile_gain;/* AudiophileGain in dB 
+			     (as defined by www.replaygain.org)      */
+  gboolean peak_signal_set;    /* has the peak signal been set?      */
+  gboolean radio_gain_set;     /* has the radio gain been set?       */
+  gboolean audiophile_gain_set;/* has the audiophile gain been set?  */
+};
+
+
 /* This code is taken from the mp3info code. Only the code needed for
  * the playlength calculation has been extracted */
 
@@ -940,6 +963,123 @@ static guint get_track_time (gchar *path)
 
 
 
+static const gchar* id3_get_binary (struct id3_tag *tag,
+				    char *frame_name,
+				    id3_length_t *len)
+{
+    const id3_byte_t *binary = NULL;
+    struct id3_frame *frame;
+    union id3_field *field;
+
+    g_return_val_if_fail (len, NULL);
+
+    *len = 0;
+
+    frame = id3_tag_findframe (tag, frame_name, 0);
+#if LOCALDEBUG
+    printf ("frame: %p\n", frame); 
+#endif
+
+    if (!frame) return NULL;
+
+#if LOCALDEBUG
+    printf (" nfields: %d\n", frame->nfields); 
+#endif
+
+
+#if 0
+/*-----------------*/
+/* just to show that this field (before last) contains the d8 ff e0 ff
+   part of the start of a jpeg file when the coverart war embedded by iTunes */
+
+    const id3_ucs4_t *string = NULL;
+    gchar *raw = NULL;
+
+    /* The last field contains the data */
+    field = id3_frame_field (frame, frame->nfields-2);
+
+#if LOCALDEBUG
+     printf (" field: %p\n", field);
+#endif
+
+    if (!field) return NULL;
+
+#if LOCALDEBUG
+     printf (" type: %d\n", field->type);
+#endif
+
+    switch (field->type)
+    {
+    case ID3_FIELD_TYPE_STRING:
+	string = id3_field_getstring (field);
+	break;
+    default:
+	break;
+    }
+  
+    /* ISO_8859_1 is just a "marker" -- most people just drop
+       whatever coding system they are using into it, so we use
+       charset_to_utf8() to convert to utf8 */
+
+    if (string)
+    {
+	raw = id3_ucs4_latin1duplicate (string);
+    }
+
+
+#if LOCALDEBUG
+    {
+	FILE *file;
+	printf (" string len: %d\n", raw?strlen(raw):0);
+	file = fopen ("/tmp/folder1.jpg", "w");
+	fwrite (raw, 1, raw?strlen(raw):0, file);
+	fclose (file);
+    }
+#endif
+    g_free (raw);
+
+/*-----------------*/
+#endif
+
+    /* The last field contains the data */
+    field = id3_frame_field (frame, frame->nfields-1);
+
+#if LOCALDEBUG
+     printf (" field: %p\n", field);
+#endif
+
+    if (!field) return NULL;
+
+#if LOCALDEBUG
+     printf (" type: %d\n", field->type);
+#endif
+
+    switch (field->type)
+    {
+    case ID3_FIELD_TYPE_BINARYDATA:
+	binary = id3_field_getbinarydata(field, len);
+	break;
+    default:
+	break;
+    }
+
+#if LOCALDEBUG
+    {
+	FILE *file;
+	printf (" binary len: %ld\n", *len);
+	file = fopen ("/tmp/folder2.jpg", "w");
+	fwrite (binary, 1, *len, file);
+	fclose (file);
+    }
+#endif
+
+
+
+    return binary;
+}
+
+
+
 static gchar* id3_get_string (struct id3_tag *tag, char *frame_name)
 {
     const id3_ucs4_t *string = NULL;
@@ -951,27 +1091,41 @@ static gchar* id3_get_string (struct id3_tag *tag, char *frame_name)
     enum id3_field_textencoding encoding = ID3_FIELD_TEXTENCODING_ISO_8859_1;
 
     frame = id3_tag_findframe (tag, frame_name, 0);
-/*     printf ("frame: %p\n", frame); */
+#if LOCALDEGUB
+    printf ("frame: %p\n", frame); 
+#endif
 
     if (!frame) return NULL;
 
     /* Find the encoding used for the field */
     field = id3_frame_field (frame, 0);
-/*     printf ("field: %p\n", field); */
+#if LOCALDEBUG
+    printf ("field: %p\n", field); 
+    printf ("type: %d\n", id3_field_type (field));
+#endif
 
     if (field && (id3_field_type (field) == ID3_FIELD_TYPE_TEXTENCODING))
     {
 	encoding = field->number.value;
-/*	printf ("encoding: %d\n", encoding); */
+#if LOCALDEBUG
+	printf ("encoding: %d\n", encoding);
+#endif
     }
 
     /* The last field contains the data */
     field = id3_frame_field (frame, frame->nfields-1);
 
-/*     printf ("field: %p\n", field); */
+#if LOCALDEBUG
+     printf ("field: %p\n", field);
+#endif
 
     if (!field) return NULL;
 
+#if LOCALDEBUG
+     printf ("type: %d\n", field->type);
+#endif
+
+
     switch (field->type)
     {
     case ID3_FIELD_TYPE_STRINGLIST:
@@ -982,6 +1136,9 @@ static gchar* id3_get_string (struct id3_tag *tag, char *frame_name)
 	break;
     case ID3_FIELD_TYPE_BINARYDATA:
 	binary = id3_field_getbinarydata(field, &len);
+#if LOCALDEBUG
+	printf ("len: %ld\nbinary: %s\n", len, binary+1);
+#endif
 	if (len > 0)
 	    return charset_to_utf8 (binary+1);
 	break;
@@ -1030,6 +1187,7 @@ static void id3_set_string (struct id3_tag *tag,
     while ((frame = id3_tag_findframe (tag, frame_name, 0)))
     {
 	id3_tag_detachframe (tag, frame);
+        id3_frame_delete (frame);
     }
 
     if ((data == NULL) || (strlen(data) == 0))
@@ -1119,6 +1277,78 @@ static void id3_set_string (struct id3_tag *tag,
 }
 
 
+/***
+ * Reads id3v1.x / id3v2 apic data
+ * @returns: TRUE on success, else FALSE.
+ */
+static gboolean id3_apic_read (gchar *filename,
+			       guchar **image_data, gsize *image_data_len)
+{
+    struct id3_file *id3file;
+    struct id3_tag *id3tag;
+
+    g_return_val_if_fail (filename, FALSE);
+    g_return_val_if_fail (image_data, FALSE);
+    g_return_val_if_fail (image_data_len, FALSE);
+
+    *image_data = NULL;
+    *image_data_len = 0;
+
+    if (!(id3file = id3_file_open (filename, ID3_FILE_MODE_READONLY)))
+    {
+	gchar *fbuf = charset_to_utf8 (filename);
+	g_print(_("ERROR while opening file: '%s' (%s).\n"),
+		fbuf, g_strerror(errno));
+	g_free (fbuf);
+	return FALSE;
+    }
+
+    if ((id3tag = id3_file_tag(id3file)))
+    {
+	id3_length_t len;
+	const guchar *coverart;
+
+	coverart = id3_get_binary (id3tag, "APIC", &len);
+
+	if (coverart)
+	{   /* I guess iTunes is doing something wrong -- the
+	     * beginning of the coverart data ends up in a different
+	       field... We'll just add the missing data manually. */
+	    const guchar itunes_broken_jfif_marker[] =
+		{ 0x10, 'J', 'F', 'I', 'F'};
+	    if (len >= 5)
+	    {
+		if (strncmp (itunes_broken_jfif_marker, coverart,  5) == 0)
+		{
+		    const guchar itunes_missing_header[] =
+			{ 0xff, 0xd8, 0xff, 0xe0, 0x00 };
+		    *image_data = g_malloc (len+5);
+		    memcpy (*image_data, itunes_missing_header, 5);
+		    memcpy ((*image_data)+5, coverart, len);
+		    *image_data_len = len+5;
+		}
+	    }
+	    if (!*image_data)
+	    {
+		*image_data = g_malloc (len);
+		memcpy (*image_data, coverart, len);
+		*image_data_len = len;
+	    }
+#if LOCALDEBUG
+	    if (*image_data)
+	    {
+		FILE *file;
+		file = fopen ("/tmp/folder.jpg", "w");
+		fwrite (*image_data, 1, *image_data_len, file);
+		fclose (file);
+	    }
+#endif
+	}    
+    }
+    id3_file_close (id3file);
+    return TRUE;
+}
+
 /***
  * Reads id3v1.x / id3v2 tag and load data into the Id3tag structure.
  * If a tag entry exists (ex: title), we allocate memory, else value
@@ -1162,7 +1392,7 @@ gboolean id3_tag_read (gchar *filename, File_Tag *tag)
 	tag->genre = id3_get_string (id3tag, ID3_FRAME_GENRE);
 	tag->compilation = id3_get_string (id3tag, "TCMP");
 	tag->subtitle = id3_get_string (id3tag, "TIT3");
-
+	tag->lyrics = id3_get_string (id3tag, "USLT");
 	tag->podcasturl = id3_get_string (id3tag, "YTID");
 	tag->podcastrss = id3_get_string (id3tag, "YWFD");
 	tag->description = id3_get_string (id3tag, "YTDS");
@@ -1207,6 +1437,8 @@ gboolean id3_tag_read (gchar *filename, File_Tag *tag)
     return TRUE;
 }
 
+
+
 static enum id3_field_textencoding get_encoding_of (struct id3_tag *tag, const char *frame_name)
 {
     struct id3_frame *frame;
@@ -1303,7 +1535,7 @@ gboolean mp3_write_file_info (gchar *filename, Track *track)
 	encoding = get_encoding (id3tag);
 	/* ...unless it's ISO_8859_1 and prefs say we should use
 	   unicode (i.e. ID3v2.4) */
-	if (prefs_get_id3_write_id3v24 () &&
+	if (prefs_get_int("id3_write_id3v24") &&
 	    (encoding == ID3_FIELD_TEXTENCODING_ISO_8859_1))
 	    encoding = ID3_FIELD_TEXTENCODING_UTF_8;
 
@@ -1330,6 +1562,10 @@ gboolean mp3_write_file_info (gchar *filename, Track *track)
 
 	id3_set_string (id3tag, "TCOM", track->composer, encoding);
 
+	string1 = g_strdup_printf("%d", track->year);
+	id3_set_string(id3tag, ID3_FRAME_YEAR, string1, encoding);
+	g_free(string1);
+
 	if (track->tracks)
 	    string1 = g_strdup_printf ("%d/%d",
 				       track->track_nr, track->tracks);
@@ -1429,14 +1665,11 @@ static gint lame_vcmp(gchar a[5], gchar b[5]) {
 /* buf[] must be declared unsigned -- otherwise the casts, shifts and
    additions below produce funny results */             
 static void read_lame_replaygain(unsigned char buf[],
-				 Track *track, int gain_adjust) {
+				 GainData *gd, int gain_adjust) {
 	char oc, nc;
 	gint gain;
-	ExtraTrackData *etr;
 
-	g_return_if_fail (track);
-	etr = track->userdata;
-	g_return_if_fail (etr);
+	g_return_if_fail (gd);
 
 	/* buf[0] and buf[1] are a bit field:
 	   3 bits: name  (mask: 0xe0 = 11100000)
@@ -1466,17 +1699,17 @@ static void read_lame_replaygain(unsigned char buf[],
 
 	switch (nc) {
 		case 0x20:
-			if (etr->radio_gain_set) return;
-			etr->radio_gain = (gdouble)gain / 10;
-			etr->radio_gain_set = TRUE;
-/*			printf("radio_gain (lame): %i\n", etr->radio_gain); */
+			if (gd->radio_gain_set) return;
+			gd->radio_gain = (gdouble)gain / 10;
+			gd->radio_gain_set = TRUE;
+/*			printf("radio_gain (lame): %i\n", gd->radio_gain); */
 			break;
 		case 0x40:
-			if (etr->audiophile_gain_set) return;
-			etr->audiophile_gain = (gdouble)gain / 10;
-			etr->audiophile_gain_set = TRUE;
+			if (gd->audiophile_gain_set) return;
+			gd->audiophile_gain = (gdouble)gain / 10;
+			gd->audiophile_gain_set = TRUE;
 /*			printf("audiophile_gain (lame): %i\n", 
-					etr->audiophile_gain);*/
+					gd->audiophile_gain);*/
 			break;
 	}
 }
@@ -1505,7 +1738,7 @@ static inline guint32 parse_lame_uint32(char *buf) {
  * TODO: Check CRC.
  */
 
-gboolean mp3_get_track_lame_replaygain (gchar *path, Track *track)
+gboolean mp3_get_track_lame_replaygain (gchar *path, GainData *gd)
 {
 	struct {
 		/* All members are defined in terms of chars so padding does not
@@ -1523,18 +1756,15 @@ gboolean mp3_get_track_lame_replaygain (gchar *path, Track *track)
 	int gain_adjust = 0;
 	int sideinfo;
 	guint32 ps;
-	ExtraTrackData *etr;
 
-	g_return_val_if_fail (track, FALSE);
-	etr = track->userdata;
-	g_return_val_if_fail (etr, FALSE);
+	g_return_val_if_fail (gd, FALSE);
 
-	etr->radio_gain = 0;
-	etr->audiophile_gain = 0;
-	etr->peak_signal = 0;
-	etr->radio_gain_set = FALSE;
-	etr->audiophile_gain_set = FALSE;
-	etr->peak_signal_set = FALSE;
+	gd->radio_gain = 0;
+	gd->audiophile_gain = 0;
+	gd->peak_signal = 0;
+	gd->radio_gain_set = FALSE;
+	gd->audiophile_gain_set = FALSE;
+	gd->peak_signal_set = FALSE;
 	
 	if (!path)
 		goto rg_fail;
@@ -1628,18 +1858,18 @@ gboolean mp3_get_track_lame_replaygain (gchar *path, Track *track)
 	/* Don't know when fixed-point PeakSingleAmplitude
 	 * was introduced exactly. 3.94b will be used for now.) */
 	if ((lame_vcmp(version, "3.94b") >= 0)) {
-		if ((!etr->peak_signal_set) && ps) {
-			etr->peak_signal = ps;
-			etr->peak_signal_set = TRUE;
+		if ((!gd->peak_signal_set) && ps) {
+			gd->peak_signal = ps;
+			gd->peak_signal_set = TRUE;
 /*			printf("peak_signal (lame): %f\n", (double)
-					etr->peak_signal / 0x800000);*/
+					gd->peak_signal / 0x800000);*/
 		}
 	} else {
 		float f = *((float *) (void *) (&ps)) * 0x800000;
-		etr->peak_signal = (guint32) f;
+		gd->peak_signal = (guint32) f;
 		/* I would like to see an example of that. */
 /*		printf("peak_signal (lame floating point): %f. PLEASE report.\n", 
-				(double) etr->peak_signal / 0x800000);*/
+				(double) gd->peak_signal / 0x800000);*/
 	}
 
 	/*
@@ -1656,13 +1886,13 @@ gboolean mp3_get_track_lame_replaygain (gchar *path, Track *track)
 		goto rg_fail;
 
 	/* radio gain */
-	read_lame_replaygain (buf, track, gain_adjust);
+	read_lame_replaygain (buf, gd, gain_adjust);
 
 	if (fread(&buf[0], 1, 2, file) != 2)
 		goto rg_fail;
 
 	/* audiophile gain */
-	read_lame_replaygain (buf, track, gain_adjust);
+	read_lame_replaygain (buf, gd, gain_adjust);
 
 	fclose(file);
 	return TRUE;
@@ -1684,7 +1914,7 @@ rg_fail:
  * The function only modifies the gains if they have not previously been set.
  */
 
-gboolean mp3_get_track_ape_replaygain(gchar *path, Track *track)
+gboolean mp3_get_track_ape_replaygain(gchar *path, GainData *gd)
 {
 	/* The Ape Tag is located a t the end of the file. Or at least that
 	 * seems where it can most likely be found. Either it is at the very end
@@ -1705,11 +1935,8 @@ gboolean mp3_get_track_ape_replaygain(gchar *path, Track *track)
 	guint32 entry_length = 0;
 	guint32 entries;
 	double d;
-	ExtraTrackData *etr;
 
-	g_return_val_if_fail (track, FALSE);
-	etr = track->userdata;
-	g_return_val_if_fail (etr, FALSE);
+	g_return_val_if_fail (gd, FALSE);
 	g_return_val_if_fail (path, FALSE);
 
 	file = fopen (path, "r");
@@ -1778,7 +2005,7 @@ gboolean mp3_get_track_ape_replaygain(gchar *path, Track *track)
 		goto rg_fail;
 	
 	for (i = 0; i < entries; i++) {
-		if (etr->radio_gain_set && etr->peak_signal_set) break;
+		if (gd->radio_gain_set && gd->peak_signal_set) break;
 		pos = pos2 + entry_length;
 		if (pos > data_length - 10) break;
 		
@@ -1793,7 +2020,7 @@ gboolean mp3_get_track_ape_replaygain(gchar *path, Track *track)
 		if (entry_length + 1 > sizeof(buf))
 			continue;
 /* 		printf ("%s:%d:%d\n",&dbuf[pos], pos2, pos); */
-		if (!etr->radio_gain_set && !strcasecmp(&dbuf[pos],
+		if (!gd->radio_gain_set && !strcasecmp(&dbuf[pos],
 					"REPLAYGAIN_TRACK_GAIN")) {
 			memcpy(buf, &dbuf[pos2], entry_length);
 			buf[entry_length] = '\0';
@@ -1802,14 +2029,14 @@ gboolean mp3_get_track_ape_replaygain(gchar *path, Track *track)
 /* 			printf("%f\n", d); */
 			if ((ep == buf + entry_length - 3) 
 					&& (!strncasecmp(ep, " dB", 3))) {
-			    etr->radio_gain = d;
-				etr->radio_gain_set = TRUE;
-/*				printf("radio_gain (ape): %i\n", etr->radio_gain);*/
+			    gd->radio_gain = d;
+				gd->radio_gain_set = TRUE;
+/*				printf("radio_gain (ape): %i\n", gd->radio_gain);*/
 			}
 			
 			continue;
 		}
-		if (!etr->peak_signal_set && !strcasecmp(&dbuf[pos],
+		if (!gd->peak_signal_set && !strcasecmp(&dbuf[pos],
 					"REPLAYGAIN_TRACK_PEAK")) {
 			memcpy(buf, &dbuf[pos2], entry_length);
 			buf[entry_length] = '\0';
@@ -1817,9 +2044,9 @@ gboolean mp3_get_track_ape_replaygain(gchar *path, Track *track)
 			d = g_ascii_strtod(buf, &ep);
 			if (ep == buf + entry_length) {
 				d *= 0x800000;
-				etr->peak_signal = (guint32) floor(d + 0.5);
-				etr->peak_signal_set = TRUE;
-/*				printf("peak_signal (ape): %f\n", (double) etr->peak_signal / 0x800000);*/
+				gd->peak_signal = (guint32) floor(d + 0.5);
+				gd->peak_signal_set = TRUE;
+/*				printf("peak_signal (ape): %f\n", (double) gd->peak_signal / 0x800000);*/
 			}
 
 			continue;
@@ -1849,154 +2076,49 @@ rg_fail:
 #include <fcntl.h>
 
 
-/* this function initiates mp3gain on a given track and causes it to write the
- * Radio Replay Gain and Peak to an Ape Tag */
-
-/* mp3gain version 1.4.2 */
-static gboolean mp3_calc_gain (gchar *path, Track *track)
-{
-    gint k,n;  /*for's counter*/
-    gchar *mp3gain_path;
-    gchar *mp3gain_exec;
-    const gchar *mp3gain_set;
-    pid_t pid,tpid;
-    int ret = 2;
-    int status;
-    int errsv;
-    int fdnull;
-
-    k=0;
-    n=0;
-
-    /* see if full path to mp3gain was set using the prefs dialogue */
-    mp3gain_set = prefs_get_path (PATH_MP3GAIN);
-    /* use default if not */
-    if (!mp3gain_set || !(*mp3gain_set)) mp3gain_set = "mp3gain";
-    /* find full path */
-    mp3gain_path = g_find_program_in_path (mp3gain_set);
-    /* show error message if mp3gain cannot be found */
-    if (!mp3gain_path)
-    {
-	gtkpod_warning (_("Could not find mp3gain. I tried to use the following "
-				"executable: '%s'.\n\nIf the mp3gain executable "
-				"is not in your path or named differently, you "
-				"can set the full path in the 'Tools' section of "
-				"the preferences dialog.\n\nIf you do not have "
-				"mp3gain installed, you can download it from "
-				"http://www.sourceforge.net/projects/mp3gain."),
-			mp3gain_set);
-	return FALSE;
-    }
-
-    mp3gain_exec = g_path_get_basename (mp3gain_path);
-
-    /*fork*/
-    pid=fork();
-
-    /*and cast mp3gain*/
-    switch (pid)
-    {
-    case -1: /* parent and error, now what?*/
-	break;
-    case 0: /*child*/
-	/* redirect output to /dev/null */
-	if ((fdnull = open("/dev/null", O_WRONLY | O_NDELAY)) != -1) {
-		dup2(fdnull, fileno(stdout));
-	}
-	
-	/* this call may add a tag to the mp3file!! */
-        execl(mp3gain_path, mp3gain_exec, 
-			"-q", /* quiet */
-			"-k", /* set ReplayGain so that clipping is prevented */
-			path, NULL);
-	errsv = errno;
-        fprintf(stderr, "execl() failed: %s\n", strerror(errsv));
-	/* mp3gain (can) return 1 on success. So only values greater 1 can
-	 * designate failure. */
-	exit(2);
-    default: /*parent*/
-	tpid = waitpid (pid, &status, 0); /*wait mp3gain termination */
-	if WIFEXITED(status) ret = WEXITSTATUS(status);
-	if (ret > 1) gtkpod_warning (_("Execution of mp3gain ('%s') failed."),
-			mp3gain_set);
-	break;
-    }/*end switch*/
-
-    /*free everything left*/
-    g_free (mp3gain_path);
-    g_free (mp3gain_exec);
-
-    /*and happily return the right value*/
-    return (ret > 1) ? FALSE : TRUE;
-}
 
 
 
 
-/* 
- * mp3_get_gain - try to read the ReplayGain values from the LAME or Ape
- * Tags. If that does not work run mp3gain. And try to read again.
+/** 
+ * mp3_read_soundcheck:
+ *
+ * try to read the ReplayGain values from the LAME or Ape Tags and set
+ * the track's soundcheck field accordingly.
  *
  * @path: localtion of the file
  * @track: structure holding track information
  *
- * The function always rereads the gains. Even if they have been
- * previuosly known.
+ * The function always rereads the gain from the file.
  *
- * Returns TRUE if at least the radio_gain could be read.
+ * Returns TRUE if the soundcheck field could be set.
  */
-
-gboolean mp3_get_gain (gchar *path, Track *track) 
+gboolean mp3_read_soundcheck (gchar *path, Track *track) 
 {
-    ExtraTrackData *etr;
+    GainData gd;
 
     g_return_val_if_fail (track, FALSE);
-    etr = track->userdata;
-    g_return_val_if_fail (etr, FALSE);
 
-    etr->radio_gain_set = FALSE;
-    etr->audiophile_gain_set = FALSE;
-    etr->peak_signal_set = FALSE;
+    memset (&gd, 0, sizeof (GainData));
 
-    mp3_get_track_lame_replaygain (path, track);
-/*    printf ("%d:%d\n", track->radio_gain_set, track->peak_signal_set); */
-    if (etr->radio_gain_set && etr->peak_signal_set) return TRUE;
-    mp3_get_track_ape_replaygain (path, track);
-    if (etr->radio_gain_set) return TRUE;
-	    
-    if (mp3_calc_gain (path, track))
-	mp3_get_track_ape_replaygain (path, track);
-    if (etr->radio_gain_set) return TRUE;
-    return FALSE;
-}
+    gd.radio_gain_set = FALSE;
+    gd.audiophile_gain_set = FALSE;
+    gd.peak_signal_set = FALSE;
 
+    mp3_get_track_lame_replaygain (path, &gd);
+    if (gd.radio_gain_set)
+    {
+	track->soundcheck = replaygain_to_soundcheck (gd.radio_gain);
+	return TRUE;
+    }
 
+    mp3_get_track_ape_replaygain (path, &gd);
+    if (gd.radio_gain_set)
+    {
+	track->soundcheck = replaygain_to_soundcheck (gd.radio_gain);
+	return TRUE;
+    }
 
-/* 
- * mp3_read_gain - as mp3_get_gain() above, but will never run
- * mp3gain.
- *
- * Returns TRUE if at least the radio_gain could be read.
- */
-
-gboolean mp3_read_gain (gchar *path, Track *track) 
-{
-    ExtraTrackData *etr;
-
-    g_return_val_if_fail (track, FALSE);
-    etr = track->userdata;
-    g_return_val_if_fail (etr, FALSE);
-
-    etr->radio_gain_set = FALSE;
-    etr->audiophile_gain_set = FALSE;
-    etr->peak_signal_set = FALSE;
-
-    mp3_get_track_lame_replaygain (path, track);
-    if (etr->radio_gain_set && etr->peak_signal_set) return TRUE;
-
-    mp3_get_track_ape_replaygain (path, track);
-    if (etr->radio_gain_set) return TRUE;
-    
     return FALSE;
 }
 
@@ -2016,6 +2138,8 @@ Track *mp3_get_file_info (gchar *name)
     File_Tag filetag;
     MP3Info *mp3i=NULL;
     FILE *file;
+    guchar *image_data = NULL;
+    gsize image_data_len = 0;
 
     g_return_val_if_fail (name, NULL);
 
@@ -2040,9 +2164,10 @@ Track *mp3_get_file_info (gchar *name)
     }
 
     track = gp_track_new ();
-    if (prefs_get_readtags() && (id3_tag_read (name, &filetag) == TRUE))
+    track->filetype = g_strdup ("MPEG audio file");
+
+    if (prefs_get_int("readtags") && (id3_tag_read (name, &filetag) == TRUE))
     {
-	track->filetype = g_strdup ("MPEG audio file");
 
 	if (filetag.album)
 	{
@@ -2153,9 +2278,31 @@ Track *mp3_get_file_info (gchar *name)
 	    track->compilation = atoi(filetag.compilation);
 	    g_free (filetag.compilation);
 	}
+
+	if (filetag.lyrics)
+	{
+	    track->lyrics_flag = 0x01;
+	    g_free (filetag.lyrics);
+	}
+	else
+	{
+	    track->lyrics_flag = 0x00;
+	}
     }
 
-    mp3_read_gain (name, track);
+    if (prefs_get_int("coverart_apic") &&
+	(id3_apic_read (name, &image_data, &image_data_len) == TRUE))
+    {
+	if (image_data)
+	{
+	    gp_track_set_thumbnails_from_data (track,
+					       image_data,
+					       image_data_len);
+	    g_free (image_data);
+	}
+    }
+
+    mp3_read_soundcheck (name, track);
 
     /* Get additional info (play time and bitrate */
     if (mp3i)
