@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-06-17 22:12:13 jcs>
+/* Time-stamp: <2006-06-11 13:57:01 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: mp3file.h,v 1.23 2005/06/17 13:30:04 jcsjcs Exp $
+|  $Id: mp3file.h,v 1.24 2006/06/11 16:04:51 jcsjcs Exp $
 */
 
 #ifndef MP3FILEH_INCLUDED
@@ -34,6 +34,6 @@
 
 gboolean mp3_write_file_info (gchar *filename, Track *track);
 Track *mp3_get_file_info (gchar *name);
-gboolean mp3_get_gain (gchar *path, Track *track);
+gboolean mp3_read_soundcheck (gchar *path, Track *track);
 
 #endif
