@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-11-22 22:34:20 jcs>
+/* Time-stamp: <2006-09-18 16:03:15 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: mp4file.c,v 1.26 2005/11/30 14:21:37 jcsjcs Exp $
+|  $Id: mp4file.c,v 1.34 2006/09/18 07:10:44 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -54,7 +54,7 @@
    gchar   *genre;            /+ genre (utf8)          +/
    gchar   *comment;          /+ comment (utf8)        +/
    gchar   *composer;         /+ Composer (utf8)       +/
-   gchar   *fdesc;            /+ Format description (utf8)   +/
+   gchar   *filetype;         /+ Format description (utf8)   +/
    gchar   *charset;          /+ charset used for tags       +/
    gchar   *description;      /+ Description text (podcasts) +/
    gchar   *podcasturl;       /+ URL/Title (podcasts)        +/
@@ -72,17 +72,14 @@
    gint32  bitrate;           /+ bitrate in kbps       +/
    guint16 samplerate;        /+ e.g.: CD is 44100     +/
    guint32 peak_signal;	      /+ LAME Peak Signal * 0x800000         +/
-   gint32  radio_gain;	      /+ RadioGain in dB*10
-				(as defined by www.replaygain.org)   +/
-   gint32  audiophile_gain;   /+ AudiophileGain in dB*10 
-				(as defined by www.replaygain.org)   +/
-   gboolean peak_signal_set;  /+ has the peak signal been set?       +/
-   gboolean radio_gain_set;   /+ has the radio gain been set?        +/
-   gboolean audiophile_gain_set;/+ has the audiophile gain been set? +/
    gboolean compilation;      /+ Track is part of a compilation CD   +/
 
-   If prefs_get_tag_readtags() returns FALSE you only should fill in
-   tracklen, bitrate and fdesc
+   If prefs_get_int("readtags") returns FALSE you only should fill in
+   tracklen, bitrate, samplerate, soundcheck and filetype
+
+   If prefs_get_int("coverart_apic") returns TRUE you should try to
+   read APIC coverart data from the tags and set it with
+   gp_set_thumbnails_from_data().
 
    Please note that the iPod will only play as much of the track as
    specified in "tracklen".
@@ -101,26 +98,22 @@
    You also have to write a function to write TAGs back to the
    file. That function should be named
 
-   gboolean xxx_write_file_info (gchar *filename, Track *track, T_item tag_id)
+   gboolean xxx_write_file_info (gchar *filename, Track *track)
 
    and return TRUE on success or FALSE on error. In that case it
    should log an error message using gtkpod_warning().
 
-   @tag_id determines which TAGs of @track out of the list above are
-   to be updated. If @tag_id==T_ALL, upudate all (supported) TAGs.
-
    You need to add your handler to file_write_info() in file.c
 
 
    Finally, you may want to provide a function that can
-   read/calculate the replay gain
+   read and set the soundcheck field: 
 
-   gboolean xxx_get_gain (gchar *filename, Track *track)
+   gboolean xxx_read_soundcheck (gchar *filename, Track *track)
 
-   and return TRUE when the replay gain could be determined (as an
-   example see mp3_get_gain() in mp3file.c.
+   and return TRUE when the soundcheck value could be determined.
 
-   You need to add your handler to get_gain() in file.c.
+   You need to add your handler to read_soundcheck() in file.c.
 
    ------------------------------------------------------------ */
 
@@ -147,10 +140,65 @@
    PACKAGE */
 #include <config.h>
 
-Track *mp4_get_file_info (gchar *mp4FileName)
+
+static gboolean mp4_scan_soundcheck (MP4FileHandle mp4File, Track *track)
 {
-    Track *track = NULL;
-    MP4FileHandle mp4File = MP4Read(mp4FileName, 0);
+    gboolean success = FALSE;
+    u_int8_t *ppValue;
+    u_int32_t pValueSize;
+
+
+    g_return_val_if_fail (mp4File != MP4_INVALID_FILE_HANDLE, FALSE);
+
+    if (MP4GetMetadataFreeForm(mp4File, "iTunNORM",
+			       &ppValue, &pValueSize))
+    {
+	gchar *str;
+	guint sc1=0, sc2=0;
+	str = g_malloc0((pValueSize+1)*sizeof(gchar));
+	memcpy(str, ppValue, pValueSize*sizeof(gchar));
+	/* This field consists of a number of hex numbers
+	   represented in ASCII, e.g. " 00000FA7 00000B3F
+	   000072CF 00006AB6 0001CF53 00016310 0000743A
+	   00007C1F 00023DD5 000162E2". iTunes seems to
+	   choose the larger one of the first two numbers
+	   as the value for track->soundcheck */
+	sscanf (str, "%x %x", &sc1, &sc2);
+	g_free (str);
+	if (sc1 > sc2)
+	    track->soundcheck = sc1;
+	else
+	    track->soundcheck = sc2;
+	success = TRUE;
+    }
+
+    if (MP4GetMetadataFreeForm(mp4File, "replaygain_track_gain",
+			       &ppValue, &pValueSize))
+    {
+	gchar *str;
+	gdouble rg;
+	str = g_malloc0((pValueSize+1)*sizeof(gchar));
+	memcpy(str, ppValue, pValueSize*sizeof(gchar));
+	rg = g_strtod (str, NULL);
+	track->soundcheck = replaygain_to_soundcheck (rg);
+	g_free (str);
+
+	success = TRUE;
+    }
+
+    return success;
+}
+
+
+gboolean mp4_read_soundcheck (gchar *mp4FileName, Track *track)
+{
+    gboolean success = FALSE;
+    MP4FileHandle mp4File;
+
+    g_return_val_if_fail (mp4FileName, FALSE);
+    g_return_val_if_fail (track, FALSE);
+
+    mp4File = MP4Read(mp4FileName, 0);
 
     if (mp4File != MP4_INVALID_FILE_HANDLE)
     {
@@ -159,7 +207,57 @@ Track *mp4_get_file_info (gchar *mp4FileName)
 
 	trackId = MP4FindTrackId(mp4File, 0, NULL, 0);
 	trackType = MP4GetTrackType(mp4File, trackId);
-	if (trackType && ((strcmp(trackType, MP4_AUDIO_TRACK_TYPE) == 0) || (strcmp(trackType, MP4_VIDEO_TRACK_TYPE) == 0)))
+
+	if (trackType &&
+	    ((strcmp(trackType, MP4_AUDIO_TRACK_TYPE) == 0) ||
+	     (strcmp(trackType, MP4_VIDEO_TRACK_TYPE) == 0) ||
+	     (strcmp(trackType, MP4_OD_TRACK_TYPE) == 0)))
+	{
+	    success = mp4_scan_soundcheck (mp4File, track);
+	}
+	else
+	{
+	    gchar *filename = charset_to_utf8 (mp4FileName);
+	    gtkpod_warning (_("'%s' does not appear to be a mp4 audio file.\n"),
+			    filename);
+	    g_free (filename);
+	}
+	MP4Close(mp4File);
+    }
+    else
+    {
+	gchar *filename = charset_to_utf8 (mp4FileName);
+	gtkpod_warning (
+	    _("Could not open '%s' for reading, or file is not an mp4 file.\n"),
+	    filename);
+	g_free (filename);
+    }
+
+    return success;
+}
+
+
+
+Track *mp4_get_file_info (gchar *mp4FileName)
+{
+    Track *track = NULL;
+    MP4FileHandle mp4File;
+
+    g_return_val_if_fail (mp4FileName, NULL);
+
+    mp4File = MP4Read(mp4FileName, 0);
+
+    if (mp4File != MP4_INVALID_FILE_HANDLE)
+    {
+	MP4TrackId trackId;
+	const char *trackType;
+
+	trackId = MP4FindTrackId(mp4File, 0, NULL, 0);
+	trackType = MP4GetTrackType(mp4File, trackId);
+	if (trackType &&
+	    ((strcmp(trackType, MP4_AUDIO_TRACK_TYPE) == 0) ||
+	     (strcmp(trackType, MP4_VIDEO_TRACK_TYPE) == 0) ||
+	     (strcmp(trackType, MP4_OD_TRACK_TYPE) == 0)))
 	{
 	    gchar *value;
 	    guint16 numvalue, numvalue2;
@@ -188,7 +286,7 @@ Track *mp4_get_file_info (gchar *mp4FileName)
 		if (g_strcasecmp (value, ".mp4") == 0)
 		    track->filetype = g_strdup ("MP4 video file");
 	    }
-	    if (prefs_get_readtags ())
+	    if (prefs_get_int("readtags"))
 	    {
 		if (MP4GetMetadataName(mp4File, &value) && value != NULL)
 		{
@@ -236,6 +334,7 @@ Track *mp4_get_file_info (gchar *mp4FileName)
 		    g_free(value);
 		}
 	    }
+	    mp4_scan_soundcheck (mp4File, track);
 	}
 	else
 	{
@@ -392,4 +491,10 @@ gboolean mp4_write_file_info (gchar *filename, Track *track)
     gtkpod_warning (_("m4a/m4p/m4b metadata update for '%s' failed: m4a/m4p/m4b not supported without the mp4v2 library. You must compile the gtkpod source together with the mp4v2 library.\n"), filename);
     return FALSE;
 }
+
+gboolean mp4_read_soundcheck (gchar *filename, Track *track)
+{
+    gtkpod_warning (_("m4a/m4p/m4b soundcheck update for '%s' failed: m4a/m4p/m4b not supported without the mp4v2 library. You must compile the gtkpod source together with the mp4v2 library.\n"), filename);
+    return FALSE;
+}
 #endif
