@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-11-12 17:16:12 jcs>
+/* Time-stamp: <2006-05-21 12:24:56 jcs>
 |
 |  Copyright (C) 2002 Corey Donohoe <atmos at atmos.org>
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
@@ -25,7 +25,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: prefs_window.h,v 1.68 2005/11/12 08:54:31 jcsjcs Exp $
+|  $Id: prefs_window.h,v 1.70 2006/06/10 08:29:44 freethinkerjim Exp $
 */
 #ifndef _GTKPOD_PREFS_WINDOW_H
 #define _GTKPOD_PREFS_WINDOW_H
@@ -43,9 +43,6 @@ void prefs_window_apply (void);
 void prefs_window_cancel (void);
 void prefs_window_create (gint page);
 void prefs_window_delete (void);
-void prefs_window_podcasts (void);
-
-void prefs_window_set_unused_gboolean3 (gboolean val);
 
 void sort_window_create (void);
 void sort_window_block (void);
