@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-06-17 22:12:12 jcs>
+/* Time-stamp: <2006-05-10 00:01:02 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 | 
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: tools.h,v 1.14 2005/06/17 13:30:05 jcsjcs Exp $
+|  $Id: tools.h,v 1.15 2006/05/13 16:41:06 jcsjcs Exp $
 */
 
 #ifndef __NORMALIZE_H__
@@ -46,10 +46,10 @@
 void nm_new_tracks (iTunesDB *itdb);
 void nm_tracks_list (GList *list);
 
-gboolean tools_sync_all (void);
-gboolean tools_sync_contacts (void);
-gboolean tools_sync_calendar (void);
-gboolean tools_sync_notes (void);
+gboolean tools_sync_all (iTunesDB *itdb);
+gboolean tools_sync_contacts (iTunesDB *itdb);
+gboolean tools_sync_calendar (iTunesDB *itdb);
+gboolean tools_sync_notes (iTunesDB *itdb);
 
 void do_command_on_entries (const gchar *command, const gchar *what,
 			    GList *selected_tracks);
